'Filename   ATU_Contoller_3.22_EADIP.bas
'Version    3.22 for use with PCB Version 09/2016 and EA DIP203/204-4 LCD
'           Set Const HD44780 = 0 (see below)
'           =0: EA DIP 203/204 is used
'           =1: HD44780 LCD is used
'Date       2022-04-23
'Purpose    Controller for ATU remote unit via RS485
'           Band data are preset by ATU_Contoller_EEPROM_Data_300.bas
'           This file is included at compile time by $Include, see below
'           SWR and power meter:
'           - 2 log amps for forward and reverse power measurement
'           - display of SWR & RF forward average or peak envelope power (PEP)
'           - Choose toroid turns of tandem match coupler fitting max. RF power
'           - 2 point calibration at selectable min and max log amp power levels
'           - Forward/reverse power is calculated from straight cal line
'           - ADC0 and ADC1 are used for forward and reverse signal resp.
'             ADC is in single mode, control via ADC registers
'           - Emitter follower for TX frequency measurement
'           Option TRX Com: Request TRX frequency from Si570 LO
'Author     Georg Latzel, DL6GL
'Copyright  (c) Georg Latzel, free for private use under GNU license
'Hardware   ATmega 1284P, LCD 20x4, rotary encoder, 5 keys
'           Rotary encoder ALPS STEC11B03 (reichelt.de)
'           Keys DIGITAST DIT1/DTL2 (without/with LED, reichelt.de)
'           LCD 20x4: EA DIP204-4 LCD with KS00783 controller (5V)
'           or        EA DIP203-3 LCD with SSD1803 controller (3.3V)
'           with appropriate adapter PCB
'           Set conditional compile "Const HD44780 = 0"
'           Optional HD44780 compatible 20x4 LCD
'           Set conditional compile "Const HD44780 = 1"
'Compiler   BASCOM-AVR 2.0.8.2
'           For readable "camel case" variables mark "Don't change case"
'           in BASCOM Options-Programmer-Environment-Editor
'           For ISP-Programming via AVRDUDE use V5.11 for ATmega1284P
'           Fuse bits may be set with AVR Studio 4.0 & ISP-PROG via USB
'Edit history
'V1.00      2009-07-13
'           first release
'V1.03      2010-10-10
'           Start mode is Auto
'           Sub FindBand:  Exit in Auto Mode if TRX switched to RX
'                          Holds relay settings in RX mode
'           TX must be switched on before for TX freq. to be known
'           Main, Send data to remote unit:
'           In case of communication error encoder values are not updated
'V1.04      2010-04-24
'           Bug fixed TX/RX switching
'V1.05      2010-04-30
'           New: bit7L, =1: 7 Coils Christian ATU, =0: 8 Coils
'V1.06      2011-11-02
'           bit7L, now "byt8Coils", is set by jumper on port PB0
'           PB0 open:   byt8Coils = 1 (8 coils)
'           PB0 to GND: byt8Coils = 0 (7 coils Christian ATU)
'V2.00      2013-02-22
'           New PCB , Controller ATmega1284P for future automatic ATU
'           20x4 LCD and new front panel board
'           SWR/Power Tandem Match Coupler & ADC included
'           EEPROM initialization data in ATU_Contoller_EEPROM_Data.bas
'           EEPROM data setup added
'           60 & 6 m band added
'V2.01      2013-03-29
'           Max. ATU tune power setup > 9W (1 digit now 2 digits)
'           HWStack & Framesize enlarged from 100 to 250
'V2.02      2013-04-11
'           New:  bytTXDelay in EEPROM @ address 33
'                 Wait bytTXDelay (1/10 sec) for TX to mute before ATU tuning.
'           Having checked muted TX power with the tune key,
'           TX muting may be maintained by pressing the tune key once again.
'           Tuning LED is lit. ATU tuning works without delay.
'           Pressing the tune key once more, muting is released.
'           Otherwise one tuning step takes bytTXDelay 1/10 seconds.
'V2.03      2013-09-13
'           Only one C-Bank is switched parallel to antenna output (Lowpass),
'           only one L-Bank is switched parallel to antenna output (Highpass).
'           In all versions before the two L-/C-Banks were series connected.
'           Display of half C and double L resp. removed.
'V2.04      2014-01-30
'           Bug fixed improper resetting of Reset and Lock key pressed flag
'V2.10      2016-09-29
'           Added: TRX-Com to get TRX frequency in RX mode instead of counter
'           PortB.0 is now input for TRX Com enable jumper
'           Changed: RS485 protocoll, compatible to ATU_RemoteUnit_210.bas
'           ADC runs for power/SWR mean values increased from 3 to 7
'           Several improvements
'           Flash used: 29,130 bytes (22%)
'V3.00      2017-05-04
'           New coupler with log amps AD8307, setup upgraded
'           Automatic tuning and auto tuning setup included
'           EEPROM storage changed
'           Forward power as PEP or avg. power (W)
'           New controller PCB, old PCB may be used with minor modifications
'           Flash used: 46,450 bytes (35%)
'V3.01      2018-09-30
'           Optimized init for EA DIP203-4 & EA DIP204-4 LCD
'           Initialize Com error flag in subs SendRemote & GetTRXFreq
'           Remote unit and TRX data read by UART receive interrupt
'           Flash used: 46,338 bytes (35%)
'V3.02      2018-10-02
'           Some minor bugs fixed
'           Flash used: 46,474 bytes (35%)
'V3.03      2018-11-16
'           Ranges for EEPROM setup data extended
'           Check of TX automatic mute function or min. TX power resp.
'           for ATU switching at program start changed.
'           Current TX power (auto muted or by hand) is checked before
'           manual or automatic tuning.
'           Flash used: 46,962 bytes (35%)
'V3.04      2018-11-23
'           Auto tune: If 1st scan failed set ATU for best SWR found in the 2 scans
'                      with hi- and lopass, no reset to old values.
'                      Check in first scan if no return loss peak is found.
'                      This was bullshit, see V3.10
'           Flash used: 47,224 bytes (36%)
'V3.10      2018-12-14
'           Auto tune: Scan logic changed.
'                      (1) Coarse scan to find return loss peak > 6dB
'                          If successfull, continue with (2)
'                          If not, toggle hi/lopass and try it again
'                          If successfull, continue with (2), if not abort scan
'                      (2) Refine L/C steps to find the target SWR/RL
'                      (3) Scan is aborted if TX is turned off while scanning
'           All counter increments from Timer0_isr to Timer2_isr.
'           Timer0 handles only frequency counter so it can be stopped.
'           Counter is started only if TX is on, stopped if off.
'           Timer0 gate time increased from 64 to 256msec to get a better
'           frequency average of SSB signals.
'           Sub IsTXon now uses FWD ADC instead of counter measurement.
'           bytBand_Data(i) introduced, =1: tuner data stored in band i
'           Manual calibration "by hand" option added.
'           TX mute delay (bytTXDelay) from units 0.1sec (1, 2...) to msec (10, 20...)
'           Flash used: 49,976 bytes (38%)
'V3.11      2019-01-08
'           Cancel first ADC conversion in FWD power measurement. Wrong data.
'           Power calculation in CheckTXMute with Sub GetPower.
'           SWR & power display Timer2 scheduler removed.
'           Bug fix in TX power control before switching ATU relays,
'           set TX power to 0W if lower than ADC detection limit.
'           Min. TX mute delay 0 (no delay) instead of 10msec in setup #3.
'           BASCOM Deflcdchar replaced by character codes for SWR bar display
'           from EA DIP203/204 data sheet.
'           Some speed approvements:
'           - Timer0 frequency counter gate time decreased from 256 to 128msec
'           - Check if Sub Findband is to be called in main program
'           Flash used: 49,850 bytes (38%)
'V3.12      2019-04-15
'           V3.11 is split into 2 versions:
'           - Use EA DIP203/204 as before, firmware index "EADIP"
'           - Use HD44780 compatible 20x4 LCD, firmware index "HD44780"
'           No other modifications
'           Flash used: 49,852 bytes (38%)
'V3.12a     2019-09-20
'           Conditional compilation according to Const HD44780
'           Config Lcd = 20 * 4A , Chipset = KS077  'HD44780=0: EA DIP203/204
'           Call LCDInit                     Special init for DIP204/203 LCD
'           or
'           Config Lcd = 20 * 4                     'HD44780=1: HD44780
'           as well as special characters for bar graph
'           Flash used: 49,852 bytes (38%)
'V3.13      2019-11-08
'           Number of repetitive ADC runs in calibration reduced from 100 to 50.
'           Overflow occured in mean sum. 50 is sufficient for mean value.
'           Flash used: 49,848 bytes (38%)
'V3.14      2020-02-12
'           BASCOM-AVR 2.0.8.2 introduced with some problems:
'           - For...Next loop with byte variables in Sub AutoTune does not work.
'             Bug in V2.0.8.2 reported to MCS. Workaround programmed.
'           - Trouble storing band segment data, bytes are shifted 1 to the left.
'             MCS: V2.0.8.2 needs a dummy byte declared just as the first
'             Dim statement for big chips like the ATmega1284 (?!?)
'           Calibration data in EEPROM init perserved if they are already stored.
'           Mixed mode calculations, e.g. "Word + Byte", eliminated.
'           Bit variables now stored in control bytes and aliassed.
'           In Setup #19 Timer2 (encoder interrups) disabled while data displayed.
'           In Auto Tune Setup lower & upper L/C limits checked, i.e. low < high.
'           Flash used: 51,832 bytes (39%)
'V3.20      2021-04-28
'           BASCOM-AVR 2.0.8.3 introduced
'           New Christian (DL3LAC) coupler option in setup 3/21 introduced
'           Normal operation in L banks was relays off = L shortened (L incactive)
'           New option: L shortened (inactive) if relay is on, i.e. logic is inverted
'           Flash used: 52,354 bytes (39%)
'V3.21      2021-04-30
'           Sorry, overlooked: Since Nov 2019, the high/low pass switching unit
'           can be configured for either single L/C banks or series wiring.
'           Series wiring highpass: 2 L banks in series, lowpass: 2 C banks.
'           The display of L and C was left on single wiring.
'           It can now be adjusted in setup #9.
'           Flash used: 52,916 bytes (40%)
'V3.22      2022-04-23
'           Sub CheckEEPROM, Part Read EEPROM: restore bytTXDelay=10 removed.
'           Was introduced in update V3.04 to V3.10 (new bytTXDelay definition)
'           After check of remote unit and TRX Com new feature inserted:
'           If TRX Com is active, RX frequency is read and remote unit tuned to RX freq.
'           So TX finds a well tuned antenna
'           Flash used: 53,604 bytes (40%)
'
'==============================================================================

'PortA   ADC for SWR/Power
'ADC0    Pwr Forward       PA0
'ADC1    Pwr Reverse       PA1
'ADC2    For future use    PA2
'ADC3    For future use    PA3
'LED     D5, Lock          PA4
'LED     D4, SWR           PA5
'LED     D3, Auto          PA6
'LED     D1, Man           PA7

'PortB   TRX com jumper, Counter input, LCD + ISP
'TRX com Jumper to GND     PB0 Jumper in: enable TRX com
'Counter Timer1 (T1)       PB1
'LCD     RS                PB2
'LCD     E                 PB3
'LCD     D7                PB4
'ISP+LCD ISP MOSI + LCD D6 PB5
'ISP+LCD ISO MISO + LCD D5 PB6
'ISP+LCD ISP SCK  + LCD D4 PB7

'PortC   Keys
'Keys    Not used          PC0
'        Not used          PC1
'        Encoder Key       PC2
'        Mode key          PC3
'        Store key         PC4
'        Reset key         PC5
'        Tune key          PC6
'        Lock Encoder key  PC7
'Mode    switched by Mode-Key:
'        (1)   (A) Automatic: Lookup EEPROM data according to frequency
'        (2)   (M) Manual set L/C with Encoder
'        (3)   (P) Program, store Values to EEPROM with Store key
'        (4)   (S) Setup menue
'        (5)       Coupler calibration
'Polling of Keys by Timer2-Interrupt

'PortD   RS485-Interface, Encoder and TRX control
'RS485   RXD               PD0
'RS485   TXD               PD1
'RS485   RE/DE             PD2
'        TX PTT relay      PD3
'        TX mute           PD4
'        Disable remote    PD5 Jumper in: Remote Unit disabled
'        Encoder B         PD6
'        Encoder A         PD7

'==============================================================================
'FUSE-Bits (AVR Studio 4.0-Settings, [ ]=no check, [x]=check)
'In ATMEGA-Data sheets: Bit = 1 -> not set = no check
'                       Bit = 0 -> set     = check
'BODLEVEL   Brown-out 4.3V  (BODLEVEL=100)
'OCDEN      [ ]
'JTAGEN     [ ] (disabled)
'SPIEN      [x] Do not change!
'WDTON      [ ]
'EESAVE     [x]
'BOOTSZ     Boot Flash size=512 words (minimal boot size)
'BOOTRST    [ ]
'CKDIV8     [ ]
'CKOUT      [ ]
'SUT_CKSEL  Ext. Crystal Osc. 8.0- MHz, Start-up time 16k + 65ms
'Extended:  0xFC
'High:      0xD7
'Low:       0xFF

$regfile = "m1284pdef.dat"                        'ATMega1284P
$crystal = 18432000                               'Crystal 18.432 MHz
$baud = 9600                                      'Baud rate RS485
$hwstack = 128
$swstack = 128
$framesize = 128
Const txtVers = "ATU V3.22"                       'Version number

'Set LCD used *****************************************************************
'=1: HD44780 LCD is used
'=0: EA DIP 203/204 is used
'
Const HD44780 = 0
'
'******************************************************************************
'With BASCOM-AVR 2.0.8.2 bytes were shifted 1 position to the left when writing
'a band segment back to EEPROM. This problem did not exist with BASCOM-AVR 2.0.7.6.
'A test program shortened only to the EEPROM handling and compiled with V2.0.8.2
'had the above problem with the ATmega1284P. On an ATmega16 it worked however.
'MCS recommended to declare 1 dummy byte as first Dim statement on big chips
'like the ATmega1284P, so that the compiler arranges the variables in RAM correctly.
'For my understanding a curious workaround.
'Here it is and it works with it, for whatever reason:

Dim DummyByte As Byte                             'Needed by V2.0.8.2 compiler

'EEPROM Variables band segments -----------------------------------------------
'Band segments:   Byte 1: HiByte(wrdSeg_FreqLo), Segment low  limit (kHz)
'                 Byte 2: LoByte(wrdSeg_FreqLo), Segment low  limit (kHz)
'                 Byte 3: HiByte(wrdSeg_FreqHi), Segment high limit (kHz)
'                 Byte 4: LoByte(wrdSeg_FreqHi), Segment high limit (kHz)
'                 Byte 5: Encoder value L (0...255, 0...127 for Christian tuner)
'                 Byte 6: Encoder value C (0...255)
'                 Byte 7: High pass(1), low pass(0)
Dim wrdSeg_FreqLo As Word                         'Low  band segment limit from EEPROM
Dim wrdSeg_FreqHi As Word                         'High band segment limit from EEPROM
Dim bytSeg_L As Byte                              'Encoder value for L from EEPROM
Dim bytSeg_C As Byte                              'Encoder value for C from EEPROM
Dim bytSeg_HL As Byte                             'High pass(1), Low pass(0)
Dim bytSeg_Arr(7) As Byte At wrdSeg_freqLo Overlay
Dim wrdSeg_width As Word                          'Segment width from EEPROM
Dim bytSeg_Bands As Byte                          'Number of band segments in EEPROM
Dim wrdSeg_FreqCe As Word                         'Band segment center freq (kHz)
Dim wrdEEPROM_Addr As Word                        'EEPROM address
Dim wrdAddr As Word
Dim wrdSeg As Word
Dim bytInit_LC As Byte                            '=1: L/C data are stored in EEPROM
Dim bytInit_EEP As Byte                           '=1: init EEPROM
Const wrdSeg_Bytes = 7                            'Nmbr of EEPROM bytes per segment
Const wrdCal_FirstAddr = 27                       'First EEPROM address calibration data
Const wrdTune_FirstAddr = 50                      'First EEPROM address auto tune settings
Const wrdSeg_FirstAddr = 150                      'First EEPROM address of band segments

'Variables Keys ---------------------------------------------------------------
Dim bytKey_enc_old As Byte
Dim bytIwr0 As Byte
Dim bytKey_ct0 As Byte
Dim bytKey_ct1 As Byte
Dim bytKey_state As Byte
Dim bytKey_press As Byte
Dim bytKey_save As Byte
Dim bytKey_enc As Byte                            '=1 if key enc   is pressed
Dim bytKey_mode As Byte                           '=1 if key mode  is pressed
Dim bytKey_store As Byte                          '=1 if key store is pressed
Dim bytKey_reset As Byte                          '=1 if key reset is pressed
Dim bytKey_tune As Byte                           '=1 if key tune  is pressed
Dim bytKey_lock As Byte                           '=1 if key lock  is pressed

'EEPROM data ------------------------------------------------------------------
Dim wrdEEPROM As Word                             'Word variable to/from EEPROM
Dim bytWrdEEP(2) As Byte At wrdEEPROM Overlay
Dim intEEPROM As Integer                          'Integer variable to/from EEPROM
Dim bytIntEEP(2) As Byte At intEEPROM Overlay
'Used in EEPROM data setup
Dim bytData As Byte                               'EEPROM variable for bytType = 1 (Byte)
Dim bytDataRef As Byte                            'Reference variable for bytType = 1 (Byte)
Dim bytDataMin As Byte                            'EEPROM variable min value for bytType = 1 (Byte)
Dim bytDataMax As Byte                            'EEPROM variable max value for bytType = 1 (Byte)
Dim bytDataInc As Byte                            'Data increment value
Dim bytDataDec As Byte                            'Data decrement value
Dim wrdData As Word                               'EEPROM variable for bytType = 2 (Word)
Dim wrdDataRef As Word                            'Reference variable for bytType = 2 (Word)
Dim wrdDataMin As Word                            'EEPROM variable min value for bytType = 2 (Word)
Dim wrdDataMax As Word                            'EEPROM variable max value for bytType = 2 (Word)
Dim intData As Integer                            'EEPROM variable for bytType = 3 (Integer)
Dim intDataRef AS Integer                         'Reference variable for bytType = 3(Integer)
Dim intDataMin As Integer                         'EEPROM variable min value for bytType = 3 (Integer)
Dim intDataMax As Integer                         'EEPROM variable max value for bytType = 3 (Integer)
Dim strUnit As String * 4                         'Variable unit, e.g. dBm

'Variables Encoder ------------------------------------------------------------
Dim bytEncCtrl As Byte                            'Contains encoder control bits
bitLC_changed Alias bytEncCtrl.0                  '=1: L/C or Hi/Lo pass changed
bitC_reset Alias bytEncCtrl.1                     'C manually resetted
bitL_reset Alias bytEncCtrl.2                     'L manually resetted
bitHL_pass Alias bytEncCtrl.3                     'High pass(1), low pass(0)
bitHL_old Alias bytEncCtrl.4                      'High/Low pass before change
Dim bytEnc_Incr As Byte                           '=2: Increment, =1: Decrement, =0: ./.
Dim bytEnc_old As Byte                            'Encoder value before change
Dim bytEnc_new As Byte                            'Encoder value after change
Dim bytEnc_L As Byte                              'Encoder value L
Dim bytEnc_C As Byte                              'Encoder value C
Dim bytEncL_old As Byte                           'Encoder L value before change
Dim bytEncC_old As Byte                           'Encoder C value before change
Dim bytEnc_E As Byte                              'Encoder value EEPROM address
Dim bytLC_Active As Byte                          '1="L", 2="C", 3="High-/low pass"
Const DoubleStep = 1                              '=1: intermediate position detected

'Coupler data -----------------------------------------------------------------
Dim bytCoupCal As Byte                            '=1: Coupler calibrated, =0 else
Dim bytADCRep As Byte                             'Repetitive ADC runs for mean value
Dim bytPEPhold As Byte                            'PEP peak sample & hold time (sec)
Dim wrdCoupFac As Word                            'Coupling factor (dB x 10)
Dim wrdAmpAtt As Word                             'Log amp attenuator (dB x 10)
Dim wrdCalFwdADCLo As Word                        'Forward ADC counts at low  cal. point
Dim wrdCalRevADCLo As Word                        'Reverse ADC counts at low  cal. point
Dim wrdCalFwdADCHi As Word                        'Forward ADC counts at high cal. point
Dim wrdCalRevADCHi As Word                        'Reverse ADC counts at high cal. point
Dim intCaldBmLo As Integer                        'Low  dBm calibration point (dBm x 10)
Dim intCaldBmHi As Integer                        'High dBm calibration point (dBm x 10)
Dim intMindBm As Integer                          'Lower detection limit (dBm x 10)
Dim intMaxdBm As Integer                          'Upper detection limit (dBm x 10)
Dim sngMindBm As Single                           'Lower detection limit (dBm)
Dim sngMaxdBm As Single                           'Upper detection limit (dBm)
Dim sngCoupAtt As Single                          'Total coupler attenuation (dB)

'Variables Counter ------------------------------------------------------------
Dim bytCntCtrl As Byte                            'Contains counter control bits
bitLED_on Alias bytCntCtrl.0                      'Toggle for LED flashing with Timer2
bitCounterDone Alias bytCntCtrl.1                 '=1: Frequency available
bitCounterNull Alias bytCntCtrl.2                 '=1: Null counts in gate time
Dim bytTimer0_cnt As Byte                         'Timer0-Overflow cycles for Counter
Dim wrdCountLo As Word                            'Low word Counter
Dim wrdCountHi As Word                            'High word Counter
Dim wrdCountSav As Word
Dim wrdCountDif As Word
Dim wrdTRXfreqCnt As Word                         'TX freq. in kHz measured with counter

'Variables ADC ----------------------------------------------------------------
Dim bytADCCtrl As Byte                            'Contains ADC control bits
bitADCdone Alias bytADCCtrl.0                     '1= all ADC channels complete
Dim bytADCRun As Byte                             'Current ADC run for mean value
Dim bytADMUX As Byte                              'ADMUX register value
Dim bytADChannel As Byte                          '0=FWD, 1=REV
Dim bytADCValIdx As Byte                          'Current ADC value index
Dim bytADCOvl As Byte                             'ADC overflow, =1: FWD, =2: REV
Dim wrdADC As Word                                'ADC conversion result
Dim bytADCLo As Byte At wrdADC Overlay            '... low byte
Dim bytADCHi As Byte At wrdADC + 1 Overlay        '... high byte
Dim wrdADCVal(2) As Word                          'ADC counts, 1=Forward, 2=Reverse
Dim wrdADCMean(2) As Word                         'ADC mean counts, 1=Forward, 2=Reverse
Const bytADCUsed = 1                              'Nmbr of used ADC minus 1

'SWR & power Variables --------------------------------------------------------
Dim bytPwrCtrl As Byte                            'Contains power&SWR control bits
bitADCMean Alias bytPwrCtrl.0                     '=1: Power accumulation done
bitShowAvg Alias bytPwrCtrl.1                     '=0: Show PEP, =1: show avg. Power
Dim bytSWRAutoMin As Byte                         'Min SWR for Auto tuning (SWR*100)
Dim bytPWRmax As Byte                             'Max HF power for ATU tuning (W)
Dim bytPWRerr As Byte                             'Error flag ADC calibration
Dim wrdADCPEP As Word                             'Current max ADC PEP value
Dim wrdPEP As Word                                'Final max ADC PEP value
Dim wrdADCCal As Word
Dim sngFwddBm As Single                           'Forward power (dBm)
Dim sngRevdBm As Single                           'Reverse power (dBm)
Dim sngFwdPwr As Single                           'Forward power (W)
Dim sngRevPwr As Single                           'Reverse power (W)
Dim sngPWRmax As Single                           'Max HF power for ATU tuning (W)
Dim sngFwdGrd As Single                           'Cal line gradient forward
Dim sngFwdOrd As Single                           'Cal line ordinate forward
Dim sngRevGrd As Single                           'Cal line gradient reverse
Dim sngRevOrd As Single                           'Cal line ordinate reverse
Dim sngADCFwd As Single                           'ADC mean forward
Dim sngADCRev As Single                           'ADC mean reverse
Dim sngSWR As Single                              'SWR
Dim sngSWRBarCal As Single                        'SWR bar scale factor
Const sngSWRBarMax = 3                            'Max SWR for bargraph

'Variables Auto tune ----------------------------------------------------------
Dim bytPass As Byte                               'Pass# in auto tune, 1st(0)/2nd(1) pass
Dim bytRefine As Byte                             '=1: Refine step size
Dim bytRLPeakFound As Byte
Dim bytCMax As Byte                               'Current ATU C at return loss maximum
Dim bytLMax As Byte                               'Current ATU L at return loss maximum
Dim bytCMaxOld As Byte                            'Old     ATU C at return loss maximum
Dim bytLMaxOld As Byte                            'Old     ATU L at return loss maximum
Dim bytStepMaxC As Byte                           'Max step width C
Dim bytStepMaxL As Byte                           'Max step width L
Dim wrdEnc_L As Word                              'Encoder value L
Dim wrdEnc_C As Word                              'Encoder value C
Dim bytStepC As Byte                              'Auto tune max step width C
Dim bytStepL As Byte                              'Auto tune max step width L
Dim bytStartC As Byte                             'Auto tune start with C(0...255)
Dim bytStartL As Byte                             'Auto tune start with L(0...255)
Dim bytEndC As Byte                               'Auto tune end with C(0...255)
Dim bytEndL As Byte                               'Auto tune end with L(0...255)
Dim bytPassBand As Byte                           'Auto tune start with lo(0), hi(1) pass
Dim bytEnc_LStart As Byte                         'Encoder start value for L
Dim bytEnc_CStart As Byte                         'Encoder start value for C
Dim bytEnc_LStop As Byte                          'Encoder stop  value for L
Dim bytEnc_CStop As Byte                          'Encoder stop  value for C
Dim bytTuneStepL As Byte                          '+/- Step width for auto tune L variation
Dim bytTuneStepC As Byte                          '+/- Step width for auto tune C variation
Dim wrdRun As Word
Dim sngRL As Single                               'Current return loss (dB)
Dim sngRLMax As Single                            'Current max return loss (dB)
Dim sngRLMaxOld As Single                         'Old     max return loss (dB)
Dim sngRLBest As Single                           'Best RL (dB) to be reached
Dim sngRLPeak As Single                           'Identified RL peak
Dim sngRLFind As Single                           'RL threshhold (dB) to find RL peak

'Variables L/C Bank -----------------------------------------------------------
Dim wrdDelta_C As Word                            'Lowest Change of C in 1/100 pF (3.13 pF)
'                                                   Max. value: 255*313=79,815 (798.15 pF)
Dim wrdDelta_L As Word                            'Lowest Change of L in 1/100 uH (0.13 uH)
'                                                   Max. value: 255*13=3,315 (33.15 uH)
Dim wrdBias_C As Word                             'Bias capacity for all C's out (1/100 pF)
Dim wrdBias_L As Word                             'Bias inductivity for all L's out (1/100 uH)
Dim bytLCseries As Byte                           '=0: L/C banks are not wired in series in Hi/Lopass switch
'                                                   =1: L/C banks are wired in series

'Variables RS485 communication ------------------------------------------------
Dim bytComCtrl As Byte                            'Contains communication control bits
bitTRXCom Alias bytComCtrl.0                      '=1: TRX communication is enabled
bitComBusy Alias bytComCtrl.1                     '=1: ATU/TRX communication active
Const bytConAdr = &H00                            'ATU Controller  COM address
Const bytATUAdr = &H01                            'ATU remote unit COM address
Const bytTRXAdr = &H02                            'TRX             COM address
Const bytATUCall = &HAA                           'Hex code for ATU relay switching
Const bytFreqCall = &HFA                          'Hex code for TRX freq call & response
Const bytDummy = &HFF                             'Dummy byte
Const bytFrameLen = 7                             'Length of protocol frame (Bytes)
Const bytFrameLenminus1 = bytFrameLen - 1
Dim bytComErr As Byte                             'communication error
Dim bytCRC8 As Byte                               'Crc8 checksum
Dim bytBuffCount As Byte                          'Buffer byte counter
Dim bytBuffInp(bytFrameLen) As Byte               'Input Buffer
Dim bytBuffOut(bytFrameLen) As Byte               'Output Buffer
Dim wrdTRXfreqCom As Word                         'TRX frequency in kHz from TRX com
Dim bytTRXfreqCom(2) As Byte At wrdTRXfreqCom Overlay

'Amateur band data ------------------------------------------------------------
Const bytBand_Num = 11                            'Number of bands
Dim bytBand_Set As Byte                           'Index of band in auto tune setup
Dim bytBand_Cur As Byte                           'Index of current band
Dim bytBand_Data(bytBand_Num) As Byte             '=1: L/C data stored in band
Dim bytBand_Width(bytBand_Num) As Byte            'Width of band segments in kHz
Dim bytBand_Meter(bytBand_Num) As Byte            'Band wavelength (m)
Dim wrdBand_Lo(bytBand_Num) As Word               'Lower band limits 160-6m (11 bands)
Dim wrdBand_Hi(bytBand_Num) As Word               'Upper band limits, both in kHz
Dim wrdBand_Addr(bytBand_Num) As Word             'Start addresses EEPROM band segments
Dim wrdBand_FreqLo As Word                        'Band lower freq. limit
Dim wrdBand_FreqHi As Word                        'Band upper freq. limit

'Variables Program control ----------------------------------------------------
Dim bytPrgCtrl As Byte                            'Contais program control bits
bitLock Alias bytPrgCtrl.0                        '1=Lock, 0=Unlock encoder
bitTXOn Alias bytPrgCtrl.1                        '=1: TX on, =0: TX off (no count rate)
bitMute Alias bytPrgCtrl.2                        '=1: mute TRX for SendRemote
bitDelay Alias bytPrgCtrl.3                       '=1: delay mute for SendRemote
bitRemoteSnd Alias bytPrgCtrl.4                   '=1: Send to Remote Unit
bitFindBand Alias bytPrgCtrl.5                    '=1: Call Sub FindBand
Const On = 1
Const Off = 0
Const Hi = 1
Const Lo = 0

Dim bytPwr_high As Byte                           'Set by CheckTXMute at program start
'                                                  '=0: TX muted power OK for ATU relay
'                                                  '=1: TX mute not checked
'                                                  '=2: Power too high for ATU relay
Dim bytSeg_found As Byte                          'Flag FindBand = 0, 1 , 2
Dim byt8Coils As Byte                             '=1: 8 coils, =0: 7 coils (DL3LAC tuner)
Dim bytLNonInv As Byte                            '=1: L bank logic inverted (relays on=L shortened)
Dim bytSWLopt As Byte                             '=1: SWL option, =0: TRX option
Dim bytTXDelay As Byte                            'Delay for TX mute (10 msec)
Dim bytSetup As Byte                              'Setup & calib menu index
Dim bytSetupAct As Byte                           '=0: Prepare setup, =1, 2: setup data
Dim bytUseTRXFreq As Byte                         '=0: Use counter measured freq. in TX mode
'                                                  '=1: Use TRX-COM freq in RX & TX mode
Dim bytMode As Byte                               'Index of bytMode_state
Dim bytCalManual As Byte                          '=0: Calib with TX, =1: by hand
Dim bytTuneMan As Byte                            'Tune key action in man. mode
Dim bytCursorFirst As Byte                        'First LCD cursor position
Dim bytMode_state(2) As Byte
bytMode_state(1) = "A"                            'Automatic
bytMode_state(2) = "M"                            'Manual
Dim strDis As String * 20                         'Text to be dispayed on LCD
Dim bytDis(20) As Byte At strDis Overlay          'the same as byte array
Dim wrdTRXfreq As Word                            'Counter or Com transmitted TRX freq
Dim wrdTRXfreqSav As Word                         'Saved TRX counter freq

'Common Variables, multiply used ----------------------------------------------
Dim bytTmp0 As Byte
Dim bytTmp1 As Byte
Dim bytTmp2 As Byte
Dim bytTmp3 As Byte
Dim bytTmp4 As Byte
Dim wrdTmp0 As Word
Dim wrdTmp1 As Word
Dim wrdTmp2 As Word
Dim wrdTmp4 As Word
Dim intTmp0 As Integer
Dim intTmp1 As Integer
Dim intTmp2 As Integer
Dim intTmp3 As Integer
Dim intTmp4 As Integer
Dim sngTmp0 As Single
Dim sngTmp1 As Single
Dim sngTmp2 As Single
Dim dwdTmp0 As Dword
Dim dwdTmp1 As Dword
Dim strTmp1 As String * 1
Dim strTmp10 As String * 10
Dim bytTmp10(11) As Byte At strTmp10 Overlay


'Configure PortA (ADC @ A.0...A.3 & LED @ Port A.4...A.7) ---------------------
'PA.0 to PA.3 are used as ADC
'ADC0    Forward
'ADC1    Reverse
'ADC2    Future use
'ADC3    Future use
DDRA = &B11110000                                 'PA0...3 ADC input/PA4...7 output
PORTA = &B00001100                                'Pullup PA2 & 3 (not used yet)
Led_lock Alias PortA.4                            'LED Lock
Led_SWR Alias PortA.5                             'LED SWR>3
Led_auto Alias PortA.6                            'LED Automatic
Led_man Alias PortA.7                             'LED Manual

'Configure PortB (TRX Com enable jumper, counter, LED & LCD) ------------------
DDRB.0 = 0                                        'PB.0=Input
PortB.0 = 1                                       'Pullup PB.0
bitTRXEnab Alias PINB.0                           'Jumper in: enable TRX Com

'PortB.1 (T1) is Counter input
'Special 20x4 Text LCD EA DIP204-4 LCD with KS00783 controller (4 bit mode)
LCD_RS Alias PortB.2                              'LCD pin RS
LCD_E Alias PortB.3                               'LCD pin E
LCD_D7 Alias PortB.4                              'LCD pin D7
LCD_D6 Alias PortB.5                              'LCD pin D6
LCD_D5 Alias PortB.6                              'LCD pin D5
LCD_D4 Alias PortB.7                              'LCD pin D4
Config Lcdpin = Pin , Db7 = LCD_D7 , Db6 = LCD_D6 , Db5 = LCD_D5 , Db4 = LCD_D4
Config Lcdpin = Pin , E = LCD_E , Rs = LCD_RS
Config Lcdbus = 4
#If HD44780 = 1                                   'HD44780 LCD is used
   Config Lcd = 20 * 4
#Else
   Config Lcd = 20 * 4A , Chipset = KS077         'Special for EA DIP204
#EndIf

'LCD ShowSWR calibration -----------------------------------------------------
'20 char display, 1 character = 5 dots
Dim strBar(5) As String * 1                       'Bars 1...5 columns width
Const bytChar = 20                                'Number of display characters
Const bytDots = 5                                 'bar dots per character
Const bytBarLen = 15                              'Bar length (characters)
Const bytRowPwr = 3                               'Row for power display
Const bytRowSWR = 4                               'Row for SWR display

'Configure PortC (keys) -------------------------------------------------------
DdrC = &B00000000                                 'PortC = Key input
PortC = &B11111111                                'activate Pullup
Key_port Alias PinC                               'Input-Port Keys
Const Key_enc = 2                                 'Encoder key @ PINC.2
Const Key_mode = 3                                'Mode key @ PINC.3
Const Key_store = 4                               'Store key @ PINC.4
Const Key_reset = 5                               'Reset key @ PINC.5
Const Key_tune = 6                                'Tune key @ PINC.6
Const Key_lock = 7                                'Lock/Unlock key @ PinC.7

'Configure PortD (RS485-Interface, TRX control & encoder) ---------------------
Ddrd = &B00011100                                 'D.0   RS485 RXD
'                                                 'D.1   RS485 TXD
'                                                 'D.2   RS485 RE/DE output
'                                                 'D.3   TX PTT output
'                                                 'D.4   TX mute output
'                                                 'D.5   Disable remote input
'                                                 'D.6   Encoder B input
'                                                 'D.7   Encoder A input
PORTD = &B11100000                                'Pullup D.5 to D.7
bitRS485Send Alias Portd.2                        '=1:Transmit, =0: Receive
bitTXPTT Alias PortD.3                            'TX PTT relay
bitTXmute Alias PortD.4                           'TX mute voltage & Tune LED
bitRemote Alias PIND.5                            'Jumper in: Remote disabled
bitEnc_a Alias PIND.6                             'Encoder B @ PIND.6
bitEnc_b Alias PIND.7                             'Encoder A @ PIND.7

'Subroutines & Functions ------------------------------------------------------
'Display subs
Declare Sub LCDInit
Declare Sub LCDWriteNibble(byVal bytLCDChar As Byte)
Declare Sub LCDWriteInstruction(byVal bytLCDChar As Byte)
Declare Sub DisplayLC(byVal bytRow As Byte , byVal bytEncEnab As Byte)
Declare Sub DisplayTune(byVal bytRow As Byte)
Declare Function CalcLC(byVal bytEnc As Byte , byVal wrdDelta As Word , byVal wrdBias As Word , byVal bytIsL As Byte) As Dword
Declare Sub DisplayFreq(byVal bytRow As Byte)
Declare Sub ClearLCDLine(byVal bytRow As Byte , byVal bytChars As Byte)
Declare Sub SetValue(byVal bytRow As Byte , byVal bytType As Byte , byVal wrdScale As Word , byVal bytComp As Byte)
Declare Sub SetupErr(byVal bytRow As Byte , byVal bytErr As Byte)
'ADC, SWR & power subs
Declare Sub GetADCData
Declare Function ReturnLoss(byVal bytRep As Byte) As Single
Declare Sub TwoPointCal(byVal bytOpt As Byte)
Declare Sub CalibMenu
Declare Sub TuneSetupMenu
Declare Sub CalCouplerTX(byVal wrdAdd1 As Word , byVal wrdAdd2 As Word)
Declare Sub CalCouplerMan(byVal wrdAdd1 As Word , byVal wrdAdd2 As Word)
Declare Sub CheckTXMute
Declare Sub CalcFwdPower
Declare Sub CalcSWRPower
Declare Sub GetPower
Declare Sub ShowSWRPower(byVal bytRowS As Byte , byVal bytRowP As Byte)
Declare Function dBm2Watt(byVal sngdBm As Single) As Single
Declare Sub AutoTune
Declare Sub NewBoundaries(byVal bytOpt As Byte , byVal bytMax As Byte , byVal bytStep As Byte)
Declare Sub ShiftSearch(byVal bytOpt As Byte , byVal bytMax As Byte , byVal bytMaxOld As Byte , byVal bytStep As Byte)
Declare Sub HalfStep
'ATU control subs
Declare Sub IsTXOn
Declare Sub TXMuteDelay
Declare Sub ChristanATU
'Communication subs
Declare Function IsComBusy() As Byte
Declare Sub SendRemote(byVal bytRow As Byte)
Declare Sub GetTRXFreq(byVal bytRow As Byte)
'EEPROM subs
Declare Sub CheckEEPROM
Declare Function EEPROMAdr(byval bytEncoder As Byte) As Word
Declare Sub StoreLCData
Declare Sub WriteBandSegment(byVal bytFrom As Byte , byVal bytTo As Byte , byVal bytAll As Byte , byVal bytMsg As Byte)
Declare Sub ReadBandSegment(byVal wrdStartAddr As Word , byVal bytFrom As Byte , byVal bytTo As Byte)
Declare Sub FindBand
Declare Sub EEPROMWriteWrd(byRef wrdWord As Word)
Declare Sub EEPROMReadWrd(byRef wrdWord As Word)
Declare Sub EEPROMWriteInt(byRef intVal As Integer)
Declare Sub EEPROMReadInt(byRef intVal As Integer)
Declare Function AutoTuneAdr(byVal bytOpt As Byte , byVal bytBand As Byte) As Word
Declare Function SegCenterFreq() As Word

'Configure Timer0-Interrupt ---------------------------------------------------
'for counter gate (Timer0 = 8bit, 2^8=256 counts)
'Gate time: Timer0 counts * Prescale * bytTimer0_runs / crystal frequency
'Possible Prescales: 8, 64, 256, 1024, here 1024
'For 18.432 MHz xtal:
'Timer0 overflow counts are reduced by PresetTimer0 = 64 to 256-64=192
'Overflow time: 192 * 1024 / 18,432,000 = 10.67 msec (93.75 Hz)
'Gate time: 192 * 1024 * 12 / 18,432,000 = 128 msec (crystal 18.432 MHz)
'bytTimer0_runs = 12 repetitive overflows
'Measured frequency is to be divided by 16 = 2^4
Config Timer0 = Timer , Prescale = 1024
Const Presettimer0 = 64
Const bytTimer0_runs = 12                         'Number overflows for gate time
On Timer0 Timer0_isr                              'On Overflow go to Timer0_isr

'Configure Timer1-Interrupt ---------------------------------------------------
'for Counter (Timer1 = 16bit, 2^16=65,536 counts)
Config Timer1 = Counter , Edge = Rising
On Timer1 Timer1_isr                              'On Overflow go to Timer1_isr

'Configure Timer2-Interrupt ---------------------------------------------------
'for keys and encoder (Timer2 = 8bit, 2^8=256 counts)
'Overflow time: Overflow-Counts * Prescale / crystal frequency
'Possible Prescales: 8, 64, 256, 1024, here 1024
'Timer2 overflow counts are reduced by PresetTimer2 = 106 to 256-106=150
'= 150 * 1024 / 18,432,000 = 8.3 msec = ~ 120 Hz (crystal 18.432 MHz)
Config Timer2 = Timer , Prescale = 1024
Const Presettimer2 = 106                          'Initialize Timer2 (shorten time, >0)
Dim bytTimer2_LED As Byte                         'Timer2 overflow cycles for LED flashing
Dim bytTimeout_cnt As Byte                        'Counter for communication time out
Const bytTimeout_runs = 90                        'Timer counts for  timeout ~0.76sec
Dim bytCallTRX_cnt As Byte                        'Counter for TRX call
Const bytCallTRX_runs = 50                        'Timer counts for TRX call ~0.4sec
Dim wrdTimer2_PEP As Word                         'Timer2 counter for PEP peak hold
Dim wrdTimer2_PEPhold As Word                     'Timer2 counts for PEP peak hold
On Timer2 Timer2_isr                              'On Overflow go to Timer2_isr

'Configure ADC in single running mode (ATmega1284P) ---------------------------
'Reference = internal 2.56V
'ADC data right adjusted
'Start with ADC channel 0
'Enable ADC
'ADC clock prescaler 128 -> ADC clock 144kHz (xtal 18.432MHz)
'ADC not yet started
'Conversion completion interrupt activated
bytADMUX = &B11000000
Admux = bytADMUX                                  'Reference = Int. 2.56V, ADLAR=0, ADC0
'                                                  ADC Multiplexer Selection Register
'                                                  Bit 7 , 6 : REFS1 , REFS0
'                                                  00: Reference = AREF
'                                                  01: Reference = AVCC
'                                                  10: Int. 1.10V Reference
'                                                  11: Int. 2.56V Reference
'                                                  Bit 5, ADLAR=0: data right adjusted
'                                                              =1: data left adjusted
'                                                  Bit 4-0, MUX4...MUX0
'                                                  =0000: ADC0 is input
'                                                  =0001: ADC1 is input
'                                                  =0010: ADC2 is input
'                                                  =0011: ADC3 is input
'                                                  =0100: ADC4 is input
'                                                  =0101: ADC5 is input
'                                                  =0110: ADC6 is input
'                                                  =0111: ADC7 is input
ADCSRA = &B10001111                               'Enable ADC, single mode, div. factor 128
'                                                  ADC Control & Status Register A
'                                                  Bit 7, ADEN = 1: enable ADC
'                                                  Bit 6, ADSC = Start ADC
'                                                  Bit 5, ADATE = 1: Auto trigger enabled
'                                                  Bit 4, ADIF = 1 when conversion completed
'                                                  Bit 3, ADIE = 1: Conversion completion interrupt activated
'                                                  Bit 2-0, ADPS2...ADPS0, ADC prescaler
'                                                  =000: Div. factor 2
'                                                  =001: Div. factor 2
'                                                  =010: Div. factor 4
'                                                  =011: Div. factor 8
'                                                  =100: Div. factor 16
'                                                  =101: Div. factor 32
'                                                  =110: Div. factor 64
'                                                  =111: Div. factor 128
ADCSRB = &B00000000                               'ADC Control & Status Register B
'                                                  Bit 7...3: Reserved, Set 00000
'                                                  Bit 2-0, ADTS2...ADTS0, Auto trigger source
'                                                  =000: Free running mode
'                                                  =001: Analog comparator
'                                                  =010: Ext. Interrupt Request 0
'                                                  =011: Timer/Counter0 Compare match
'                                                  =100: Timer/Counter0 Overflow
'                                                  =101: Timer/Counter1 Compare match
'                                                  =110: Timer/Counter1 Overflow
'                                                  =111: Timer/Counter1 Capture event
DIDR0 = &B11111100                                'Digital Input Disable Register 0
'                                                  Any bit = 0: corresponding ADC input enabled
'                                                  Any bit = 1: corresponding ADC input disabled
'                                                  No reaction on any setting!?

On ADC ADC_isr                                    'ADC conversion complete interrupt

'Enable UART receive interrupt service routine --------------------------------
On URXC RXD_isr                                   'Go to RXD_isr if character is received
Enable URXC                                       'Enable Character receive interrupt

'Prepare special characters for bar graph--------------------------------------
#If HD44780 = 1                                   'HD44780 LCD is used
   'User defined display bars for HD44780 LCD
   Deflcdchar 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0   'not used
   Deflcdchar 1 , 16 , 16 , 16 , 16 , 16 , 16 , 16 , 16       'chr(1): 1 bar
   Deflcdchar 2 , 24 , 24 , 24 , 24 , 24 , 24 , 24 , 24       'chr(2): 2 bars
   Deflcdchar 3 , 28 , 28 , 28 , 28 , 28 , 28 , 28 , 28       'chr(3): 3 bars
   Deflcdchar 4 , 30 , 30 , 30 , 30 , 30 , 30 , 30 , 30       'chr(4): 4 bars
   Deflcdchar 5 , 31 , 31 , 31 , 31 , 31 , 31 , 31 , 31       'chr(5): 5 bars (full)
   Cls
   For bytTmp0 = 1 To 5                           'User defined bars to strBar(1...5)
      strBar(bytTmp0) = chr(bytTmp0)
   Next bytTmp0
#Else                                             'EA DIP203/204 LCD is used
   'Bars for EA DIP203-4 & EA DIP204-4 LCD, see data sheet character table
   strBar(1) = Chr(218)                           '1 bar
   strBar(2) = Chr(217)                           '2 bars
   strBar(3) = Chr(216)                           '3 bars
   strBar(4) = Chr(215)                           '4 bars
   strBar(5) = Chr(214)                           '5 bars (full)
   Call LCDInit                                   'Special init for DIP204/203 LCD
#EndIf

'Initialize data --------------------------------------------------------------
$Include ATU_Contoller_EEPROM_Data_300.bas        'Initialize band data

Start:                                            'Reentry after EEPROM init

bytKey_ct0 = 0                                    'Initialize key data
bytKey_ct1 = 0
Decr bytKey_ct0
bytKey_ct1 = bytKey_ct0
bytKey_state = 0
bytKey_press = 0

'Initial setting of Remote Unit (Relay not set):
'- Low pass (0)
'- All L shortened (Encoder value 000)
'- All C open (Encoder value 000)

bytEnc_new = 0                                    'Initialize encoder data
bytEnc_old = 0
bytEnc_L = 0
bytEnc_C = 0
bytEnc_E = 0
bytEncL_old = 0
bytEncC_old = 0
bytLC_Active = 0
bytBand_Cur = 0
bitHL_old = Lo
bytPwr_high = 1
bytTuneMan = 0                                    'Tune key not preesed in man. mode

sngADCFwd = 0                                     'Initialize ADC data
sngADCRev = 0
bytADCRun = 0
wrdADCMean(1) = 0
wrdADCMean(2) = 0
wrdTimer2_PEP = 0                                 'Timer2 counter for PEP peak hold
sngTmp0 = bytPEPhold                              'PEP hold time (sec x 10)
sngTmp0 = 0.1 * sngTmp0                           'PEP hold time (sec)
sngTmp0 = sngTmp0 / 0.00839                       'PEP hold time Timer2 cycles
wrdTimer2_PEPhold = sngTmp0                       'Timer2 counts for PEP peak hold

sngSWRBarCal = bytBarLen * bytDots                'SWR bar dots (5 per character)
sngTmp0 = sngSWRBarMax - 1
sngSWRBarCal = sngSWRBarCal / sngTmp0             'SWR bar calibration, SWR=1 suppressed

bitLock = Off                                     'Encoder is enabled
bitHL_pass = Lo                                   'Low pass
bytTimer0_cnt = 0                                 'Timer2 overlow runs for LED flash
bytCallTRX_cnt = 0                                'Timer2 overlow runs for TRX call
bytBuffCount = 0                                  'RS485 Input buffer counter
bitComBusy = 0                                    'ATU/TRX communication not busy

Cls
Cursor Off
Locate 1 , 1
Lcd txtVers ; " Hello"
Wait 1

'Enable interrupts ------------------------------------------------------------
Enable Timer2                                     'keys and encoder timer
SREG.7 = 1                                        'Enable all interrupts
Timer2 = Presettimer2                             'initialize Timer2 (shorten time if >0)

Restart:                                          'Entry point after calibration
Cls

'Check EEPROM -----------------------------------------------------------------
'Read EEPROM data if initialized
'Initialize EEPROM if empty (new chip or erased EEPROM)
Call CheckEEPROM


'Check SWL option -------------------------------------------------------------
If bytSWLopt = ON Then                            'SWL option
   Cls
   Locate 1 , 1
   LCD "SWL option is set"
   Locate 2 , 1
   LCD "Coupler not checked!"
   Do
      Call IsTXOn
      If bitTXOn = On Then                        'TX is on
         Locate 3 , 1
         LCD "TX is on, turn off"
      Else
         Locate 4 , 1
         LCD "Remote unit enabled"
         Wait 2
         Cls
         Exit Do
      End If
   Loop
   bytMode = 2                                    'Start with Manual Mode
Else                                              'TRX option
   bytMode = 1                                    'Start with Auto Mode
End If

'Check coupler calibration ----------------------------------------------------
If bytSWLopt = OFF Then                           'TRX option
   If bytCoupCal = 0 Then                         'Coupler not calibrated
      Locate 2 , 1
      LCD "Coupler not calib.  "
      Locate 3 , 1
      LCD "Calib. is entered   "
      Wait 2
   Else                                           'Coupler calibrated
      'Prepare coupler data ---------------------------------------------------
      Call TwoPointCal(1)                         'Forward calibration line
      Call TwoPointCal(2)                         'Reverse calibration line
      bitShowAvg = 0                              '=0: Show PEP, =1: show avg. Power
      sngTmp1 = wrdCoupFac                        'Coupling factor (dB x 10)
      sngTmp1 = 0.1 * sngTmp1                     'Coupling factor (dB)
      sngTmp2 = wrdAmpAtt                         'Log amp attenuator (dB x 10)
      sngTmp2 = 0.1 * sngTmp2                     'Log amp attenuator (dB)
      sngCoupAtt = sngTmp1 + sngTmp2              'Total coupler attenuation (dB)
      sngMindBm = intMindBm
      sngMindBm = 0.1 * sngMindBm                 'Lower detection limit (dBm)
      sngMaxdBm = intMaxdBm
      sngMaxdBm = 0.1 * sngMaxdBm                 'Upper detection limit (dBm)
   End If
End If

'Check remote unit ------------------------------------------------------------
Cls
Locate 1 , 1
Lcd "Remote Unit OK?"
Locate 2 , 1
Waitms 500
If bitRemote = On Then                            'Remote disable jumper is not in
   If bytLNonInv = 1 Then                         'Encoder L value not to be inverted
      bytEnc_L = 0                                'Encoder value L=0, all relays off
   Else                                           'Invert encoder value L
      bytEnc_L = &B0111_1111                      'Inverted encoder value L, relays 0...6=On
   End If
   bytEnc_C = 0                                   'Encoder value C
   bitHL_pass = Lo                                'High pass(Hi=1), Low pass(Lo=0)
   bytTmp2 = 0
   For bytTmp0 = 1 To 3                           'Try it 3 times
      Call SendRemote(2)                          'Send dummy message
      If bytComErr = 0 Then                       'Response = ACK, remote unit is ready            '
         bytTmp2 = 1
         Exit For
      End If
      Waitms 100
   Next bytTmp0
   Call ClearLCDLine(2 , bytChar)
   If bytTmp2 = 1 Then
      Lcd "... is OK!"
   Else
      Lcd "... is not OK!"
      Stop                                        'Program stopped
   End If
Else
   Lcd "... is disabled     "
End If

'Check TRX communication unit -------------------------------------------------
Locate 3 , 1
Lcd "TRX Com OK?     "
Locate 4 , 1
Waitms 500
bitTRXCom = Off                                   'TRX com is not active
If bitTRXEnab = 0 Then                            'TRX Com enable jumper is in
   bytTmp2 = 0
   For bytTmp0 = 1 To 3                           'Try it 3 times
      Call GetTRXFreq(4)                          'Send frequency request to TRX
      If bytComErr = 0 Then                       'Freq received, TRX unit is ready            '
         bytTmp2 = 1
         Exit For
      End If
      Waitms 100
   Next bytTmp0
   Call ClearLCDLine(4 , bytChar)
   If bytTmp2 = 1 Then
      Lcd "... is OK!"
      bitTRXCom = On                              'TRX com is active
   Else
      Lcd "... is not OK!"
      Wait 1
   End If
Else
   Lcd "... is disabled     "
End If
If bitTRXCom = Off Then                           'TRX com is not active
   If bytUseTRXFreq = 1 Then                      'Use TRX Com freq if TX is on was set
      bytUseTRXFreq = 0                           'Use counter freq if TX is on
   End If
End If
Wait 1
Cls


'Get RX frequency and set relays in remote unit for TX mute check -------------
'Introduced in V3.22
'Only available if TRX com is active
If bitTRXCom = On Then                            'TRX com is active
   Locate 1 , 1
   LCD "Set tuner > TRX freq"
   Do
      Call IsTXOn
      If bitTXOn = On Then                        'TX is on
         Locate 2 , 1
         LCD "TX is on, turn off"
         Waitms 500
      Else
         Call ClearLCDLine(2 , bytChar)
         Exit Do
      End If
   Loop

   bytTmp4 = 0
   Locate 2 , 1
   LCD "TRX freq. "
   Call GetTRXFreq(4)                             'Send TRX frequency request
   wrdTRXfreq = wrdTRXfreqCom
   If wrdTRXfreq > 0 Then                         'TRX freq was sent
      strTmp10 = Str(wrdTRXfreq)
      strTmp10 = Format(strTmp10 , " 0.000")
      Locate 2 , 13
      LCD strTmp10
      Call FindBand                               'Get Band segment from EEPROM
      bytEncL_old = bytEnc_L                      'Save new ATU data
      bytEncC_old = bytEnc_C
      bitHL_old = bitHL_pass
      bitLC_changed = 1
      If bytSeg_found = 2 Then                    'Band was found
         Call SendRemote(3)                       'Send to remote unit and get response
         If bytComErr = 0 Then                    'Response = ACK, remote unit did it
            Locate 4 , 1
            LCD "Tuner set > " ; strTmp10
         Else
            bytTmp4 = 1                           'Remote unit error
            Locate 3 , 1
            LCD "Remote unit error " ; bytComErr
         End If
      Else
         bytTmp4 = 2                              'Findband error
         Locate 3 , 1
         LCD "Freq. band not found"
      End If
   Else                                           'TRX freq was not sent
      bytTmp4 = 3
      Locate 2 , 13
      LCD "--.---  "
   End If
   If bytTmp4 = 0 Then
      Wait 3
   Else
      Do
         Locate 4 , 1
         LCD "Not set->press Reset"
         Waitms 500
         If bytKey_press.Key_reset = 1 Then       'Reset key pressed
            bytKey_press.Key_reset = 0
            Exit Do
         End If
      Loop
   End If
   Cls
End If


'Check TX mute function if coupler is calibrated ------------------------------
'Wait until muted Power is OK or Reset key is pressed (Cancel)
'Result  bytPwr_high = 0: Muted TX power is  checked and OK
'                    = 1: Muted TX power not checked
If bytCoupCal = 1 Then                            'Coupler is calibrated
   If bytSWLopt = OFF Then                        'TRX option
      bytADChannel = 0                            'First ADC channel ADC0
      bitADCdone = 0
      Locate 1 , 1
      LCD "Check TX mute ..."
      Locate 3 , 1
      LCD "Cancel ->press Reset"
      Do
         Call IsTXOn
         If bitTXOn = On Then                     'TX is on
            Call CheckTXMute                      'Check TX muted power
            If bytPwr_high = 0 Then               'Muted power is OK
               Call ClearLCDLine(3 , bytChar)
               Locate 4 , 1
               LCD "...Check done and OK"
               Exit Do
            End If
         Else                                     'TX is off
            Locate 2 , 1
            LCD "Turn on TX please   "
         End If
         If bytKey_press.Key_reset = 1 Then       'Reset key pressed: Cancel
            Call ClearLCDLine(3 , bytChar)
            Locate 4 , 1
            LCD "...Check cancelled"
            bytKey_press.Key_reset = 0
            bytPwr_high = 1                       'TX mute not checked
            Exit Do
         End If
      Loop
      Wait 2
      Cls
   End If
Else                                              'Coupler is not calibrated
   If bytSWLopt = OFF Then                        'TRX option
      bytMode = 4                                 'Enter calib menu
      Call CalibMenu
   End If
End If


Do                                                'Main program ***************

   'Check keys, any key pressed? ----------------------------------------------
   'bytKey_press returned from Timer2_isr, flag bits 1...5 = keys 1...5
   'Key_enc   = #2                                'Encoder key @ PINC.2
   'Key_mode  = #3                                'Mode    key @ PINC.3
   'Key_store = #4                                'Store   key @ PINC.4
   'Key_reset = #5                                'Reset   key @ PINC.5
   'Key_tune  = #6                                'Tune    key @ PINC.6
   'Key_lock  = #7                                'Lock    key @ PinC.7
   'Just pressed detected in Timer2_isr: pressed status flag bit = 1 bytKey_press.x
   '2 key flags are issued from Timer2_isr:
   '1   bytKey_press.x = 1: key x changed from open to pressed,
   '    Used for normal key pressed action
   '    After evaluating bytKey_press.x flag is reset just below.
   '    Short/long pressed status flag is reset in function KeyPressedLong
   '2   bytKey_state.x = 1: while key is pressed, = 0: Key is released
   '    bytKey_state.x flag is used for autorepeat
   'Key action is stored in the variables:
   'bytKey_enc    = 0: Key encoder was not pressed
   '              = 1: Key encoder was pressed short
   'bytKey_mode   = 0: Key mode was not pressed
   '              = 1: Key mode was pressed short
   'bytKey_store  = 0: Key store was not pressed
   '              = 1: Key store was pressed short
   'bytKey_reset  = 0: Key reset was not pressed
   '              = 1: Key reset was pressed short
   'bytKey_tune   = 0: Key tune was not pressed
   '              = 1: Key tune was pressed short
   'bytKey_lock   = 0: Key lock was not pressed
   '              = 1: Key lock was pressed short
   If bytKey_press > 1 Then                       'Any key was pressed
      'Check encoder key
      bytKey_enc = bytKey_press.Key_enc           '=1: Key enc pressed
      bytKey_press.Key_enc = 0                    'Reset Key enc pressed flag
      'Check mode key ---------------------------------------------------------
      bytKey_mode = bytKey_press.Key_mode         '=1: Key mode pressed
      bytKey_press.Key_mode = 0                   'Reset Key mode pressed flag
      'Check store key --------------------------------------------------------
      bytKey_store = bytKey_press.Key_store       '=1: Key store pressed
      bytKey_press.Key_store = 0                  'Reset Key store pressed flag
      'Check reset key --------------------------------------------------------
      bytKey_reset = bytKey_press.Key_reset       '=1: Key reset pressed
      bytKey_press.Key_reset = 0                  'Reset Key reset pressed flag
      'Check tune key
      bytKey_tune = bytKey_press.Key_tune         '=1: Key tune pressed
      bytKey_press.Key_tune = 0                   'Reset Key tune pressed flag
      'Check lock key ---------------------------------------------------------
      bytKey_lock = bytKey_press.Key_lock         '=1: Key lock pressed
      bytKey_press.Key_lock = 0                   'Reset Key lock pressed flag
   Else                                           'No key was pressed
      bytKey_enc = 0                              'Reset all key flags
      bytKey_mode = 0
      bytKey_store = 0
      bytKey_reset = 0
      bytKey_tune = 0
      bytKey_lock = 0
   End If

   'Check Mode Key, prepare data for different modes --------------------------
   'Mode    switched consecutively by Mode key:
   '        (1) Automatic: find data in EEPROM and set data
   '        (2) Manual: set data by Encoder
   '        (3) Setup mode
   '        (4) Calibrate coupler log amps
   '        (5) Setup auto tune data
   If bytKey_mode = 1 Then                        'Mode key pressed
      Incr bytMode                                'Next Mode                             '
      Select Case bytMode
         Case Is <= 2                             'Auto or man mode
            ADCSRA.ADSC = 1                       'Start ADC
         Case 3                                   'Enter setup mode
            If bytSWLopt = OFF Then               'TRX option
               If bytTuneMan = 1 Then             'permanent mute was set in man mode
                  bitTXmute = Off                 'TX-Mute voltage off
                  bitTXPTT = Off                  'TX-PTT relay off
                  bytTuneMan = 0
               End If
            End If
            bytSetupAct = 0                       'Initialize setup mode
            bytSetup = 1                          'First setup item
         Case 4                                   'Calibrate coupler log amps
            If bytSWLopt = OFF Then               'TRX option
               Call CalibMenu                     'Show calibration menu
            Else                                  'SWL option
               Cls
               Locate 1 , 1
               LCD "SWL option is set"
               Locate 2 , 1
               LCD "No calib. needed"
               Locate 4 , 1
               LCD "Returned to man mode"
               Wait 4
               Cls
               bytmode = 2
            End If
         Case 5                                   'Setup auto tune data
            Call TuneSetupMenu                    'Prepare menu 5, setup auto tune
         Case 6                                   'End of mode selection
            Cls
            If bytSWLopt = ON Then                'SWL option
               bytMode = 2                        'Back to manual mode
            Else                                  'TRX option
               If bytCoupCal = 1 Then             'Cal OK, back to mode 1
                  wrdAddr = 17
                  ReadEEPROM bytADCRep , wrdAddr  'ADC mean repetition
                  bitADCdone = 0
                  bytADCRun = 0
                  bytMode = 1                     'Back to automatic mode
               Else                               'Cal not OK
                  wrdSeg_FreqHi = 0               'Force FindBand to get segment again
                  Goto Restart
               End If
            End If
      End Select
   End If

   'Handle different modes ----------------------------------------------------
   'Mode 1 & 2: Get frequency, band segment and show frequency
   If bytMode <= 2 Then                           'Automatic or manual mode
      'Request frequency from TRX if bus is free (ATU com is active) -------------
      If bitTRXCom = On Then                      'TRX com is active
         If IsComBusy() = 0 Then                  'Bus is not busy
            'Check wait time for TRX call
            If bytCallTRX_cnt > bytCallTRX_runs Then       'Wait time elapsed
               Call GetTRXFreq(4)                 'Send TRX frequency request
               bytCallTRX_cnt = 0                 'Reset Call timer counter
            End If
         End If
      End If

      'Get TRX frequency (counter measured or TRX Com transferred)
      'If TRX Com is disabled frequency counter is used anyway
      'If TRX Com is enabled frequency counter is used if bytUseTRXFreq = 0
      'Counter is enabled only if TX is on, then frequency is measured
      'After that counter is disabled again
      wrdTRXfreq = 0
      wrdTRXfreqSav = 0
      Call IsTXOn
      If bitTRXCom = Off Or bytUseTRXFreq = 0 Then       'Counter frequency if TX is on
         If bitTXOn = ON Then                     'Tx is on, measure frequency
            bitCounterDone = 0
            bitCounterNull = 0
            Enable Timer0                         'Enable counter timer0 interrupts
            Start Timer0
            Do
               NOP
            Loop Until bitCounterDone = 1         'Wait for counter result
            Stop Timer0
            Disable Timer0                        'Disable counter timer0 interrupts
            If bitCounterNull = 0 Then            'No null counts detected
               wrdTRXfreq = wrdTRXfreqCnt         'Last counter measured frequency
               wrdTRXfreqSav = wrdTRXfreqCnt      'Save freq for TRX off
            Else                                  'Null counts detected
               wrdTRXfreq = wrdTRXfreqSav
            End If
         Else
            If bitTRXCom = On Then                'TRX com is active
               wrdTRXfreq = wrdTRXfreqCom         'TRX freq transferred by Com
            Else                                  'TRX com is not active
               wrdTRXfreq = wrdTRXfreqSav         'Saved TRX frequency
            End If
         End If
      Else                                        'Use TRX freq transferred by Com
         wrdTRXfreq = wrdTRXfreqCom               'TRX freq transferred by Com
      End If

      Call DisplayFreq(2)                         'Show TX freq & segment freq

      'Check if Sub FindBand is to be called to get ATU data from EEPROM
      bitFindBand = 1                             'Call Sub FindBand
      If wrdTRXfreq = 0 Then                      'No TRX freq available
         bytSeg_found = 0                         'No band segment available
         bitFindBand = 0                          'Nothing to do in Sub FindBand
      Else                                        'Frequency available
         If bitLock = On Then                     'Lock active
            bytSeg_found = 1                      'Freq in the current band segment
            bitFindBand = 0                       'Nothing to do in Sub FindBand
         End If
         If wrdTRXfreq >= wrdSeg_FreqLo And wrdTRXfreq < wrdSeg_FreqHi Then
            If bytBand_Cur > 0 Then
               bytSeg_found = 1                   'Freq in the current band segment
               bitFindBand = 0                    'Nothing to do in Sub FindBand
            End If
         End If
      End If
      If bitFindBand = 1 Then
         Call FindBand                            'get Band segment from EEPROM
      End If
      If bytSeg_found = 2 Then                    'New band segment found
         bytEncL_old = bytEnc_L                   'Save new ATU data
         bytEncC_old = bytEnc_C
         bitHL_old = bitHL_pass
         bitLC_changed = 1
      End If

      'Show power & SWR for TRX option
      If bitTXOn = ON Then                        'TX is on
         ADCSRA.ADSC = 1                          'Start ADC
         If bitADCdone = 1 Then                   'ADC accumulation complete
            Call GetADCData                       'Handle ADC values PEP & Avg.
            If bitADCMean = 1 Then                'ADC mean data available
               Call CalcSWRPower                  'Calculate SWR & power
               Call ShowSWRPower(bytRowSWR , bytRowPwr)       'Show SWR & power
               bytADCRun = 0                      'Initialize next ADC run
               sngADCFwd = 0
               sngADCRev = 0
               wrdADCMean(1) = 0
               wrdADCMean(2) = 0
            End If
         End If
         If sngSWR > 3 Then                       'SWR>3
            Led_SWR = bitLED_on                   'SWR LED flashing
         Else
            Led_SWR = Off
         End If
      Else                                        'TX is off
         Led_SWR = Off
         Locate bytRowPwr , 1
         If bytSWLopt = ON And bytPwr_high = 0 Then
            LCD "SWL option active   "
         Else
            LCD "No coupler signal   "
            Locate bytRowSWR , 1
            If bytMode = 2 And bytSWLopt = Off Then       'Manual mode, not SWL
               Locate bytRowSWR , 1
               LCD "No manual  tuning   "
            Else
               LCD "                    "
            End If
         End If
       End If                                     'End If bitTXOn = ON
   End If                                         'End If bytMode <= 2

   Select Case bytMode
      '************************************************************************
      'Mode 1: Automatic Mode
      '************************************************************************
      Case 1
         Led_auto = On                            'Auto LED on
         Led_man = Off                            'Man LED off
         Call DisplayLC(1 , 0)                    'Show L, C, Hi/Lo pass

         If bytKey_store = 1 Then                 'Store key pressed
            Call StoreLCData                      'Store L/C data to EEPROM
         End If

         If bytKey_Reset = 1 Then                 'Reset key pressed
            Toggle bitShowAvg                     ' Change PEP/AVG display
         End If

         If bytKey_lock = 1 Then                  'Lock key pressed
            Toggle bitLock
            Led_lock = bitLock                    'Lock LED on/off
         End If

         If bytSWLopt = OFF Then                  'TRX option only
            If bytKey_tune = 1 Then               'Tune key pressed
               Call AutoTune                      'Start auto tuning
            End If
         End If

      '************************************************************************
      'Mode 2: Manual Mode
      '************************************************************************
      Case 2
         Led_auto = Off                           'Auto LED off
         Led_man = On                             'Man LED on

         If bitTXOn = OFF And bytSWLopt = OFF Then       'TX is off and TRX option
            Call DisplayLC(1 , 0)                 'Show L, C, Hi/Lo pass only
         Else                                     'TX is on or SWL option or both
            bytEncL_old = bytEnc_L                'Save old ATU data
            bytEncC_old = bytEnc_C
            bitHL_old = bitHL_pass
            Call DisplayLC(1 , 1)                 'Set/show L, C, Hi/Lo pass
            If byt8Coils = 0 Then                 'Only 7 Coils (Christian ATU)
               Call ChristanATU
            End If

            If bytKey_store = 1 Then              'Store key pressed
               Call StoreLCData                   'Store L/C data to EEPROM
            End If

            If bytKey_reset = 1 Then              'Reset key pressed
               Select Case bytLC_Active
                  Case 1                          'L is active
                     bitL_reset = 1               'Reset Encoder L
                  Case 2                          'C is active
                     bitC_reset = 1               'Reset Encoder C
               End Select
            End If

            If bytKey_lock = 1 Then               'Lock key pressed
               Toggle bitLock
               Led_lock = bitLock                 'Lock LED on/off
            End If

            'Encoder Key: switch L / C / Hi/lo pass (bytLC_Active = 1, 2, 3)
            If bitLock = Off Then                 'Encoder not locked
               If bytKey_enc = 0 And bytKey_enc_old = 0 Then       'Encoder not pressed
                  bytKey_enc_old = 1
                  Incr bytLC_Active
                  If bytLC_Active = 4 Then bytLC_Active = 1
               End If
               If bytKey_enc = 1 And bytKey_enc_old = 1 Then       'Encoder pressed
                  bytKey_enc_old = 0
               End If
            End If
            'bytTuneMan=0  TX permanent mute not set in man. mode
            'bytTuneMan=1  Tune key was pressed
            '              TX permanent mute to allow ATU setting
            '              holds until tune key was pressed again
            If bytSWLopt = OFF Then               'TRX option only
               If bytKey_tune = 1 Then            'Tune key pressed
                  Select Case bytTuneMan
                     Case 0                       'TX mute was off
                        bitTXmute = On            'TX-Mute voltage on
                        bitTXPTT = On             'TX-PTT relay on
                        bytTuneMan = 1            'Permanent mute, man mode
                     Case Else                    'TX mute was permanent on
                        bitTXmute = Off           'TX-Mute voltage off
                        bitTXPTT = Off            'TX-PTT relay off
                        bytTuneMan = 0            'Permanent mute off
                  End Select
               End If
            End If
         End If                                   'End of If bitTXOn = OFF And bytSWLopt = OFF

      '************************************************************************
      'Mode 3: Setup EEPROM data
      '************************************************************************
      Case 3
         'Controlled by bytSetupAct
         'bytSetupAct = 0: read and show EEPROM data
         '            = 1: change EEPROM data
         If bytSetupAct = 0 Then                  'Enter Setup mode
            Led_auto = Off                        'Auto LED off
            Led_SWR = Off                         'SWR LED on
            Led_man = Off                         'Man LED off
            strTmp10 = Str(bytSetup)
            strTmp10 = Format(strTmp10 , " 0")
            Locate 1 , 1
            LCD "Setup EEPROM   " ; strTmp10 ; "/" ; "22  "
            Call ClearLCDLine(3 , bytChar)
            Locate 4 , 1
            LCD "ESC Sto  Ret Next   "
            Locate 2 , 1
         End If

         Select Case bytSetup
            Case 1                                'SWL option *****************
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 4
                  ReadEeprom bytSWLopt , wrdAddr
                  bytData = bytSWLopt
                  bytDataMin = 0
                  bytDataMax = 1
                  bytDataInc = 1
                  bytDataDec = 1
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "SWL option (1):"
                  bytCursorFirst = 15
                  bytSetupAct = 1
               Else
                  wrdAddr = 4
                  Call SetValue(2 , 1 , 1 , 0)    'Row 2, Byte, Multiplier 1
                  wrdAddr = 4
                  ReadEeprom bytSWLopt , wrdAddr  'Read after change
               End If

            Case 2                                'byt8Coils: 8 or 7 coils ****
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 5
                  ReadEeprom byt8Coils , wrdAddr  '7 or 8 coils version
                  bytData = byt8Coils
                  bytDataMin = 0
                  bytDataMax = 1
                  bytDataInc = 1
                  bytDataDec = 1
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Coils 8(1), 7(0):"
                  bytCursorFirst = 17
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 5
                  Call SetValue(2 , 1 , 1 , 0)    'Row 2, Byte, Multiplier 1
                  wrdAddr = 5
                  ReadEeprom byt8Coils , wrdAddr  'Read after Change
               End If

            Case 3                                'L Banks inverted (relays on=L shortened) ****
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 41
                  ReadEeprom bytLNonInv , wrdAddr 'L banks inverted
                  If bytLNonInv > 1 Then bytLNonInv = 1
                  bytData = bytLNonInv
                  bytDataMin = 0
                  bytDataMax = 1
                  bytDataInc = 1
                  bytDataDec = 1
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "L std(1), inv(0):"
                  bytCursorFirst = 17
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 41
                  Call SetValue(2 , 1 , 1 , 0)    'Row 2, Byte, Multiplier 1
                  wrdAddr = 41
                  ReadEeprom bytLNonInv , wrdAddr  'Read after Change
               End If


            Case 4                                'TX mute delay **************
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 6
                  ReadEeprom bytTXDelay , wrdAddr
                  bytData = bytTXDelay
                  bytDataMin = 0                  '0msec
                  bytDataMax = 250                '250msec
                  bytDataInc = 10
                  bytDataDec = 10
                  strUnit = "msec"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "TXmute delay"
                  bytCursorFirst = 14
                  bytSetupAct = 1
               Else
                  wrdAddr = 6
                  Call SetValue(2 , 1 , 1 , 0)    'Row 2, Byte, Multiplier 1
                  wrdAddr = 6
                  ReadEeprom bytTXDelay , wrdAddr 'Read After change
               End If

            Case 5                                'Capacity step width ********
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 7
                  Call EEPROMReadWrd(wrdDELTA_C)
                  wrdData = wrdDELTA_C
                  wrdDataMin = 10                 '0.1pF
                  wrdDataMax = 2000               '20pF
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "pF"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Delta-C:"
                  bytCursorFirst = 10
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 7
                  Call SetValue(2 , 2 , 100 , 0)  'Row 2, Word, Multiplier 100 to 1/100pF
                  wrdAddr = 7
                  Call EEPROMReadWrd(wrdDELTA_C)  'Read after change
               End If

            Case 6                                'Inductivity step width *****
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 9
                  Call EEPROMReadWrd(wrdDELTA_L)
                  wrdData = wrdDELTA_L
                  wrdDataMin = 5                  '0.05uH
                  wrdDataMax = 2000               '20uH
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "uH"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Delta-L:"
                  bytCursorFirst = 10
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 9
                  Call SetValue(2 , 2 , 100 , 0)  'Row 2, Word, Multiplier 100 to 1/100uH
                  wrdAddr = 9
                  Call EEPROMReadWrd(wrdDELTA_L)  'Read after change
               End If

            Case 7                                'Bias capacity **************
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 11
                  Call EEPROMReadWrd(wrdBias_C)
                  wrdData = wrdBias_C
                  wrdDataMin = 10                 '0.1pF
                  wrdDataMax = 4000               '40pF
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "pF"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Bias-C:"
                  bytCursorFirst = 9
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 11
                  Call SetValue(2 , 2 , 100 , 0)  'Row 2, Word, Multiplier 100 to 1/100pF
                  wrdAddr = 11
                  Call EEPROMReadWrd(wrdBias_C)   'Read after change
               End If

            Case 8                                'Bias inductivity ***********
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 13
                  Call EEPROMReadWrd(wrdBias_L)
                  wrdData = wrdBias_L
                  wrdDataMin = 5                  '0.05uH
                  wrdDataMax = 2000               '20uH
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "uH"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Bias-L:"
                  bytCursorFirst = 9
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 13
                  Call SetValue(2 , 2 , 100 , 0)  'Row 2, Word, Multiplier 100 to 1/100uH
                  wrdAddr = 13
                  Call EEPROMReadWrd(wrdBias_L)   'Read after change
               End If

            Case 9                                'L/C series option ***********
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 42
                  ReadEeprom bytLCSeries , wrdAddr
                  If bytLCSeries > 1 Then bytLCSeries = 1
                  bytData = bytLCSeries
                  bytDataMin = 0
                  bytDataMax = 1
                  bytDataInc = 1
                  bytDataDec = 1
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "L/C ser Y(1) N(0):"
                  bytCursorFirst = 18
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 42
                  Call SetValue(2 , 1 , 1 , 0)    'Row 2, Byte, Multiplier 1
                  wrdAddr = 42
                  ReadEeprom bytLCSeries , wrdAddr       'Read after change
               End If

            Case 10                               'Min SWR for AutoTune *******
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 15
                  ReadEeprom bytSWRAutoMin , wrdAddr
                  bytData = bytSWRAutoMin
                  bytDataMin = 101                '0.01
                  bytDataMax = 200                '2.00
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Min Auto SWR:"
                  bytCursorFirst = 15
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 15
                  Call SetValue(2 , 1 , 100 , 0)  'Row 2, Byte, Multiplier 100 to 1/100 SWR
                  wrdAddr = 15
                  ReadEeprom bytSWRAutoMin , wrdAddr       'Read after change
               End If

            Case 11                               'Max pwr for ATU tune *******
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 16
                  ReadEeprom bytPWRmax , wrdAddr
                  bytData = bytPWRmax
                  bytDataMin = 1                  '1W
                  bytDataMax = 50                 '50W
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "W"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Max tune power:"
                  bytCursorFirst = 16
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 16
                  Call SetValue(2 , 1 , 1 , 0)    'Row 2, Byte, Multiplier 1 to W
                  wrdAddr = 16
                  ReadEeprom bytPWRmax , wrdAddr  'Read after change
                  sngPWRmax = bytPWRmax
               End If

            Case 12                               'ADC repetitive runs ********
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 17
                  ReadEeprom bytADCRep , wrdAddr
                  bytData = bytADCRep
                  bytDataMin = 1
                  bytDataMax = 50
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = " "
                  Call ClearLCDLine(2 , bytChar)
                  LCD "ADC runs:"
                  bytCursorFirst = 11
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 17
                  Call SetValue(2 , 1 , 1 , 0)    'Row 2, Byte, Multiplier 1
                  wrdAddr = 17
                  ReadEeprom bytADCRep , wrdAddr  'Read after change
               End If

            Case 13                               'PEP sample&hold time *******
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 39
                  ReadEeprom bytPEPhold , wrdAddr
                  bytData = bytPEPhold
                  bytDataMin = 1                  '0.1sec
                  bytDataMax = 100                '10sec
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "sec"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "PEP hold:"
                  bytCursorFirst = 11
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 39
                  Call SetValue(2 , 1 , 10 , 0)   'Row 2, Byte, Multiplier 10 to 1/10 sec
                  wrdAddr = 39
                  ReadEeprom bytPEPhold , wrdAddr 'Read after change
               End If

            Case 14                               'Coupling factor ************
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 19
                  Call EEPROMReadWrd(wrdCoupFac)
                  wrdData = wrdCoupFac
                  wrdDataMin = 100                '10dB
                  wrdDataMax = 400                '40dB
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "dB"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Coupl. fac:"
                  bytCursorFirst = 13
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 19
                  Call SetValue(2 , 2 , 10 , 0)   'Row 2, Word, Multiplier 10: dB to 1/10dB
                  wrdAddr = 19
                  Call EEPROMReadWrd(wrdCoupFac)  'Read after change
               End If

            Case 15                               'Log amp attenuator *********
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 21
                  Call EEPROMReadWrd(wrdAmpAtt)
                  wrdData = wrdAmpAtt
                  wrdDataMin = 0                  '0dB
                  wrdDataMax = 500                '50dB
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "dB"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Log amp att:"
                  bytCursorFirst = 14
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 21
                  Call SetValue(2 , 2 , 10 , 0)   'Row 2, Word, Multiplier 10: dB to 1/10dB
                  wrdAddr = 21
                  Call EEPROMReadWrd(wrdAmpAtt)   'Read after change
               End If

            Case 16                               'Low log amp calib level ****
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 23
                  Call EEPROMReadInt(intCaldBmLo)
                  intData = intCaldBmLo
                  intDataMin = -900               '-90dBm
                  intDataMax = -400               '-40dBm
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "dBm"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Calib lo:"
                  bytCursorFirst = 11
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 23
                  Call SetValue(2 , 3 , 10 , 0)   'Row 2, Integer, Multiplier 10: dBm to 1/10dBm
                  wrdAddr = 23
                  Call EEPROMReadInt(intCaldBmLo) 'Read after change
               End If

            Case 17                               'High log amp calib level ****
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 25
                  Call EEPROMReadInt(intCaldBmHi)
                  intData = intCaldBmHi
                  intDataMin = -200               '-20dBm
                  intDataMax = 300                '+30dBm
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "dBm"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Calib hi:"
                  bytCursorFirst = 11
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 25
                  Call SetValue(2 , 3 , 10 , 0)   'Row 2, Integer, Multiplier 10: dBm to 1/10dBm
                  wrdAddr = 25
                  Call EEPROMReadInt(intCaldBmHi) 'Read after change
               End If

            Case 18                               'Low log amp detection limit
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 35
                  Call EEPROMReadInt(intMinDbm)
                  intData = intMinDbm
                  intDataMin = -900               '-90dBm
                  intDataMax = -400               '-40dBm
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "dBm"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Limit lo:"
                  bytCursorFirst = 11
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 35
                  Call SetValue(2 , 3 , 10 , 0)   'Row 2, Integer, Multiplier 10: dBm to 1/10dBm
                  wrdAddr = 35
                  Call EEPROMReadInt(intMinDbm)   'Read after change
               End If

            Case 19                               'High log amp detection limit
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 37
                  Call EEPROMReadInt(intMaxDbm)
                  intData = intMaxDbm
                  intDataMin = -200               '-20dBm
                  intDataMax = 300                '+30dBm
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = "dBm"
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Limit hi:"
                  bytCursorFirst = 11
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 37
                  Call SetValue(2 , 3 , 10 , 0)   'Row 2, Integer, Multiplier 10: dBm to 1/10dBm
                  wrdAddr = 37
                  Call EEPROMReadInt(intMaxDbm)   'Read after change
               End If

            Case 20                               'Use TRX-COM frequency in RX & TX mode
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = 40
                  ReadEEPROM bytUseTRXFreq , wrdAddr
                  bytData = bytUseTRXFreq
                  bytDataMin = 0
                  If bitTRXCom = On Then          'TRX-COM is active, can be changed
                     bytDataMax = 1
                     bytDataInc = 1
                     bytDataDec = 1
                  Else                            'TRX-COM disabled, not to be changed
                     bytDataMax = 0
                     bytDataInc = 0
                     bytDataDec = 0
                     If bytUseTRXFreq = 1 Then    'Change to 0
                        bytUseTRXFreq = 0
                        wrdAddr = 40
                        WriteEEPROM bytUseTRXFreq , wrdAddr
                     End If
                     Locate 3 , 1
                     LCD "TRX-COM not active  "
                  End If
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Use TRX freq (1)"
                  bytCursorFirst = 17
                  bytSetupAct = 1
               Else                               'Change EEPROM data
                  wrdAddr = 40
                  Call SetValue(2 , 1 , 1 , 0)    'Row 2, Byte, Multiplier 1
                  wrdAddr = 40
                  ReadEEPROM bytUseTRXFreq , wrdAddr       'Read after change
               End If

            Case 21                               'Show Band segments *********
               If bytSetupAct = 0 Then
                  Locate 2 , 1
                  LCD "Addr.      ->       "
                  bytEnc_E = 1
                  bytSetupAct = 1
                  Locate 4 , 1
                  LCD "ESC ---  Ret Next   "
               Else
                  If bytEnc_Incr > 0 Then         'Encoder changed
                     Select Case bytEnc_Incr
                        Case 1                    'Decrement
                           Decr bytEnc_E
                           If bytEnc_E < 1 Then bytEnc_E = 1
                        Case 2                    'Increment
                           Incr bytEnc_E
                           If bytEnc_E > bytSeg_Bands Then bytEnc_E = bytSeg_Bands
                     End Select
                  End If
                  Disable Timer2                  'Stop encoder interrupts

                  wrdEEPROM_Addr = EEPROMAdr(bytEnc_E)       'Calculate EEPROM address
                  'Read Band segment data from EEPROM
                  Call ReadBandSegment(wrdEEPROM_Addr , 1 , 7)
                  bytEnc_L = bytSeg_L             'Data for Sub DisplayLC from EEPROM
                  bytEnc_C = bytSeg_C
                  bitHL_pass = bytSeg_HL
                  strTmp10 = Str(wrdEEPROM_Addr)
                  strTmp10 = Format(strTmp10 , "0000")
                  Locate 2 , 7
                  LCD strTmp10
                  wrdSeg_freqCe = SegCenterFreq()
                  strTmp10 = Str(wrdSeg_freqCe)
                  strTmp10 = Format(strTmp10 , " 0.000")
                  Locate 2 , 15
                  LCD strTmp10                    'Show EEPROM Address & freq.
                  Call DisplayLC(3 , 0)           'Show EEPROM Segment data

                  bytEnc_Incr = 0                 'Reset Encoder
                  Enable Timer2                   'Start encoder again

               End If

            Case 22                               'Initialize EEPROM **********
               Led_auto = bitLED_on               'LED Prog + Auto + Man flashing
               Led_SWR = bitLED_on
               Led_man = bitLED_on
               If bytSetupAct = 0 Then
                  Led_auto = bitLED_on            'LED SWR + Auto + Man flashing
                  Led_SWR = bitLED_on
                  Led_man = bitLED_on
                  Locate 2 , 1
                  Lcd "Init EEPROM?        "
                  Locate 4 , 1
                  LCD "ESC ---  Ret All Seg"
                  Locate 4 , 15
                  bytSetupAct = 1
               Else
                  If bytKey_state.Key_lock = 1 Or bytKey_state.Key_tune = 1 Then
                     'Lock key was pressed: 'All': init all EEPROM addresses
                     'Set EEPROM Address 1 and 2 = 255
                     'Tune key was pressed: 'Seg': init only EEPROM band segments
                     'Set EEPROM Address 1 = 255 and 2 = 1
                     bytTmp2 = 255
                     WriteEeprom bytTmp2 , 1
                     If bytKey_state.Key_lock = 1 Then       'Init All
                        WriteEeprom bytTmp2 , 2
                     Else                         'Init Seg
                        bytTmp2 = 1
                        WriteEeprom bytTmp2 , 2
                     End If
                     GoTo Start                   'Start program again
                  End If
               End If

         End Select                               'End of setup ***************

         If bytKey_lock = 1 Then                  'Lock key pressed (Next item)
            Incr bytSetup
            If bytSetup > 22 Then                 'Was last setup item
               If bytCoupCal = 0 Then             'Coupler not calibrated
                  bytMode = 4                     'Enter calibration
                  bytSetup = 1
               Else                               'Coupler calibrated
                  bytMode = 5                     'Enter setup auto tune data
                  bytSetup = 1
               End If
            End if
            bytSetupAct = 0                       'Initialize setup
         End If

         If bytKey_reset = 1 Then                 'Reset key pressed (Return)
            If bytSetup > 1 Then
               Decr bytSetup                      'Return to previous setup item
               bytSetupAct = 0
            End If
         End If

      '************************************************************************
      'Mode 4: Calibrate coupler log amps
      '                  bytSetupAct =0: Select option
      '                                  "With TX" (bytCalManual = 0)
      '                                  "By hand" (bytCalManual = 1)
      '                              =1: Continue with selected option
      '                              =9: Exit Calibration
      'bytCalManual = 0: Measure coupler with TX
      '                  bytSetup    =1: FWD, lower calibration level
      '                  bytSetup    =2: REV, lower calibration level
      '                  bytSetup    =3: FWD, upper calibration level
      '                  bytSetup    =4: REV, upper calibration level
      '                  bytSetupAct =1: Read & show calibration level hi/lo
      '                              =2: Wait for go for measurement
      '                              =3: Read old ADC value, measure new ADC value
      '                              =4: ADC counts measured, wait for store
      '                              =9: Exit calibration
      'bytCalManual = 1: Enter calibration data by hand
      '                  bytSetup    =1: FWD, lower calibration level
      '                  bytSetup    =2: REV, lower calibration level
      '                  bytSetup    =3: FWD, upper calibration level
      '                  bytSetup    =4: REV, upper calibration level
      '                  bytSetupAct =1: Read & show calibration level hi/lo
      '                              =2: Change ADC data, store
      '************************************************************************
      Case 4
         'Select calibration opion with TX / by hand
         If bytSetupAct = 0 Then
            bytCalManual = 0
            Do
               If bytEnc_Incr > 0 Then            'Encoder turned
                  If bytCalManual = 1 Then
                     bytCalManual = 0
                     Locate 3 , 1
                     LCD "With TX " ; Chr(223) ; "Encoder key"
                  Else
                     If bytCalManual = 0 Then
                        bytCalManual = 1
                        Locate 3 , 1
                        LCD "By hand " ; Chr(223) ; "Encoder key"
                     End If
                  End If
                  bytEnc_Incr = 0                 'Reset Encoder
               End If
               'Exit loop if selection is done (encoder key pressed)
               If bytKey_press.Key_enc = 1 Then   'Encoder key pressed
                  bytKey_press.Key_enc = 0        'Reset Key pressed flag
                  Call ClearLCDLine(2 , bytChar)
                  Call ClearLCDLine(3 , bytChar)
                  If bytCalManual = 0 Then        'Calibration with TX
                     bytADCRep = 50               '50 ADC measurements for cal
                     wrdAddr = 19
                     Call EEPROMReadWrd(wrdCoupFac)       'Coupling factor (dB x 10)
                     Call EEPROMReadWrd(wrdAmpAtt)       'Log amp attenuators (dB x 10)
                     Locate 2 , 1
                  Else                            'Calibration by hand
                     Locate 4 , 1
                     LCD "ESC Sto Ret  Next   "
                     Locate 2 , 1
                  End If
                  bytSetup = 1
                  bytSetupAct = 1                 'Continue calibration
                  Exit Do
               End If
               'Exit loop if mode key is pressed (ESC)
               If bytKey_press.Key_mode = 1 Then  'Key mode pressed (ESC)
                  bytKey_press.Key_mode = 0       'Reset Key mode pressed flag
                  bytSetupAct = 9                 'Exit calibration
                  Exit Do
               End If
            Loop
         End If

         If bytSetupAct = 9 Then                  'Exit calibration
            Call TuneSetupMenu                    'Prepare menu 5, setup auto tune
            bytMode = 5
         Else                                     'Continue calibration
            If bytCalManual = 0 Then              'Calibration with TX --------
               Select Case bytSetup
                  Case 1                          'FWD, lower calibration level
                     Call CalCouplerTX(23 , 27)
                  Case 2                          'REV, lower calibration level
                     Call CalCouplerTX(23 , 29)
                  Case 3                          'FWD, upper calibration level
                     Call CalCouplerTX(25 , 31)
                  Case 4                          'REV, upper calibration level
                     Call CalCouplerTX(25 , 33)
               End Select
            Else                                  'Calibration by hand --------
               Select Case bytSetup
                  Case 1                          'FWD, lower calibration level
                     Call CalCouplerMan(23 , 27)
                  Case 2                          'REV, lower calibration level
                     Call CalCouplerMan(23 , 29)
                  Case 3                          'FWD, upper calibration level
                     Call CalCouplerMan(25 , 31)
                  Case 4                          'REV, upper calibration level
                     Call CalCouplerMan(25 , 33)
               End Select
            End if

            If bytSetup > 4 Then                  'Was last calibration
               Call ClearLCDLine(4 , bytChar)
               Call ClearLCDLine(2 , bytChar)
               wrdAddr = wrdCal_FirstAddr
               Call EEPROMReadWrd(wrdCalFwdADCLo) 'Forward ADC counts @ lo cal point
               Call EEPROMReadWrd(wrdCalRevADCLo) 'Reverse ADC counts @ lo cal point
               Call EEPROMReadWrd(wrdCalFwdADCHi) 'Forward ADC counts @ hi cal point
               Call EEPROMReadWrd(wrdCalRevADCHi) 'Reverse ADC counts @ hi cal point
               bytTmp0 = 0
               If wrdCalFwdADCLo > 0 Then Incr bytTmp0
               If wrdCalRevADCLo > 0 Then Incr bytTmp0
               If wrdCalFwdADCHi > 0 Then Incr bytTmp0
               If wrdCalRevADCHi > 0 Then Incr bytTmp0
               If bytTmp0 = 4 Then                'Calibration complete
                  bytCoupCal = 1
                  Locate 3 , 1
                  LCD "Calibration complete"
               Else
                  bytCoupCal = 0
                  Locate 3 , 1
                  LCD "*Calib data missing*"
               End If
               wrdAddr = 18
               WriteEEPROM bytCoupCal , wrdAddr
               Locate 4 , 1
               LCD "Program is restarted"
               Wait 2
               Cls
               'Program must be restarted to calculate the new calibration lines
               wrdSeg_FreqHi = 0                  'Force FindBand to get segment again
               GoTo Restart
            End If
         End If

      '************************************************************************
      'Mode 5: Setup auto tune data
      '************************************************************************
      Case 5
         bytTmp0 = bytBand_Meter(bytBand_Set)     'Select band 1...11
         strTmp10 = str(bytTmp0)
         strTmp10 = Format(strTmp10 , "  0")
         Locate 1 , 17
         LCD strTmp10 ; "m"
         Select Case bytSetup
            Case 0                                'Select band 1...11
               Locate 2 , 1
               LCD "Select band " ; Chr(223) ; "Encoder"
               Locate 3 , 1
               LCD "OK " ; Chr(223) ; "Encoder key"
               If bytEnc_Incr > 0 Then            'Encoder turned
                  Select Case bytEnc_Incr
                     Case 1                       'Encoder decremented
                        If bytBand_Set > 1 Then
                           Decr bytBand_Set
                        End If
                     Case 2                       'Encoder incremented
                        If bytBand_Set < 11 Then
                           Incr bytBand_Set
                        End If
                  End Select
                  bytEnc_Incr = 0                 'Reset Encoder
                  bytTmp0 = bytBand_Meter(bytBand_Set)
                  strTmp10 = str(bytTmp0)
                  strTmp10 = Format(strTmp10 , "  0")
                  Locate 1 , 17
                  LCD strTmp10 ; "m"
               End If
               If bytKey_enc = 1 Then             'Encoder key pressed
                  bytSetup = 1
                  bytSetupAct = 0
                  Call ClearLCDLine(2 , bytChar)
                  Call ClearLCDLine(3 , bytChar)
               End If

            Case 1                                'Start value for C
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = AutoTuneAdr(4 , bytBand_Set)
                  ReadEeprom bytDataRef , wrdAddr 'Read C end value
                  wrdAddr = AutoTuneAdr(3 , bytBand_Set)
                  ReadEeprom bytData , wrdAddr    'Read C start value
                  bytDataMin = 0
                  bytDataMax = 255
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Start C:"
                  bytCursorFirst = 10
                  bytSetupAct = 1
                  Call ClearLCDLine(3 , bytChar)
               Else                               'Change EEPROM data
                  wrdAddr = AutoTuneAdr(3 , bytBand_Set)
                  Call SetValue(2 , 1 , 1 , 1)    'Row 2, Byte, Multiplier 1
                  dwdTmp0 = CalcLC(bytData , wrdDelta_C , wrdBias_C, 0)
                  dwdTmp0 = dwdTmp0 / 100         'C in pF
                  strTmp10 = Str(dwdTmp0)
                  strTmp10 = Format(strTmp10 , "  0")
                  Locate 2 , 16
                  LCD strTmp10 ; "pF"
                  If bytPass = 1 Then             'Start >= End
                     Call SetupErr(3 , bytPass)
                  End If
               End If

            Case 2                                'End value for C
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = AutoTuneAdr(3 , bytBand_Set)
                  ReadEeprom bytDataRef , wrdAddr 'Read C start value
                  wrdAddr = AutoTuneAdr(4 , bytBand_Set)
                  ReadEeprom bytData , wrdAddr    'Read C end value
                  bytDataMin = 0
                  bytDataMax = 255
                  bytDataInc = 1
                  bytDataDec = 10
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "End C:"
                  bytCursorFirst = 10
                  bytSetupAct = 1
                  Call ClearLCDLine(3 , bytChar)
               Else                               'Change EEPROM data
                  wrdAddr = AutoTuneAdr(4 , bytBand_Set)
                  Call SetValue(2 , 1 , 1 , 2)    'Row 2, Byte, Multiplier 1
                  dwdTmp0 = CalcLC(bytData , wrdDelta_C , wrdBias_C, 0)
                  dwdTmp0 = dwdTmp0 / 100         'C in pF
                  strTmp10 = Str(dwdTmp0)
                  strTmp10 = Format(strTmp10 , "  0")
                  Locate 2 , 16
                  LCD strTmp10 ; "pF"
                  If bytPass = 1 Then             'End <= Start
                      Call SetupErr(3 , bytPass)
                  End If
               End If

            Case 3                                'Start value for L
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = AutoTuneAdr(6 , bytBand_Set)
                  ReadEeprom bytDataRef , wrdAddr 'Read L end value
                  wrdAddr = AutoTuneAdr(5 , bytBand_Set)
                  ReadEeprom bytData , wrdAddr    'Read L start value
                  bytDataMin = 0
                  If byt8Coils = 0 Then           'Christian tuner, only 7 coils
                     bytDataMax = 127
                  Else
                     bytDataMax = 255             '8 coils
                  End If
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Start L:"
                  bytCursorFirst = 10
                  bytSetupAct = 1
                  Call ClearLCDLine(3 , bytChar)
               Else                               'Change EEPROM data
                  wrdAddr = AutoTuneAdr(5 , bytBand_Set)
                  Call SetValue(2 , 1 , 1 , 1)    'Row 2, Byte, Multiplier 1
                  dwdTmp0 = CalcLC(bytData , wrdDelta_L , wrdBias_L, 1)
                  dwdTmp0 = dwdTmp0 / 10          'Show 1 decimal
                  strTmp10 = Str(dwdTmp0)
                  strTmp10 = Format(strTmp10 , "  0.0")
                  Locate 2 , 14
                  LCD strTmp10 ; "uH"
                  If bytPass = 1 Then             'Start >= End
                     Call SetupErr(3 , bytPass)
                  End If
               End If

            Case 4                                'End value for L
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = AutoTuneAdr(5 , bytBand_Set)
                  ReadEeprom bytDataRef , wrdAddr 'Read L start value
                  wrdAddr = AutoTuneAdr(6 , bytBand_Set)
                  ReadEeprom bytData , wrdAddr    'Read L end value
                  bytDataMin = 0
                  If byt8Coils = 0 Then           'Christian tuner, only 7 coils
                     bytDataMax = 127
                  Else
                     bytDataMax = 255             '8 coils
                  End If
                  bytDataInc = 1
                  bytDataDec = 10
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "End L:"
                  bytCursorFirst = 10
                  bytSetupAct = 1
                  Call ClearLCDLine(3 , bytChar)
               Else                               'Change EEPROM data
                  wrdAddr = AutoTuneAdr(6 , bytBand_Set)
                  Call SetValue(2 , 1 , 1 , 2)    'Row 2, Byte, Multiplier 1
                  dwdTmp0 = CalcLC(bytData , wrdDelta_L , wrdBias_L, 1)
                  dwdTmp0 = dwdTmp0 / 10          'Show 1 decimal
                  strTmp10 = Str(dwdTmp0)
                  strTmp10 = Format(strTmp10 , "  0.0")
                  Locate 2 , 14
                  LCD strTmp10 ; "uH"
                  If bytPass = 1 Then             'End <= Start
                      Call SetupErr(3 , bytPass)
                  End If
               End If

            Case 5                                'Maximal step sizeC
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = AutoTuneAdr(0 , bytBand_Set)
                  ReadEeprom bytData , wrdAddr
                  bytDataMin = 1
                  bytDataMax = 30
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Max step size C:"
                  bytCursorFirst = 18
                  bytSetupAct = 1
                  Call ClearLCDLine(3 , bytChar)
               Else                               'Change EEPROM data
                  wrdAddr = AutoTuneAdr(0 , bytBand_Set)
                  Call SetValue(2 , 1 , 1 , 0)    'Row 2, Byte, Multiplier 1
               End If

            Case 6                                'Maximal step size L
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = AutoTuneAdr(1 , bytBand_Set)
                  ReadEeprom bytData , wrdAddr
                  bytDataMin = 1
                  bytDataMax = 30
                  bytDataInc = 10
                  bytDataDec = 1
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Max step size L:"
                  bytCursorFirst = 18
                  bytSetupAct = 1
                  Call ClearLCDLine(3 , bytChar)
               Else                               'Change EEPROM data
                  wrdAddr = AutoTuneAdr(1 , bytBand_Set)
                  Call SetValue(2 , 1 , 1 , 0)    'Row 2, Byte, Multiplier 1
               End If

            Case 7                                'Start with Hi/Lopass
               If bytSetupAct = 0 Then            'Read & show EEPROM data
                  wrdAddr = AutoTuneAdr(2 , bytBand_Set)
                  ReadEeprom bytData , wrdAddr
                  bytDataMin = 0
                  bytDataMax = 1
                  bytDataInc = 1
                  bytDataDec = 1
                  strUnit = ""
                  Call ClearLCDLine(2 , bytChar)
                  LCD "Start Lo(0),Hi(1):"
                  bytCursorFirst = 18
                  bytSetupAct = 1
                  Call ClearLCDLine(3 , bytChar)
               Else                               'Change EEPROM data
                  wrdAddr = AutoTuneAdr(2 , bytBand_Set)
                  Call SetValue(2 , 1 , 1 , 0)    'Row 2, Byte, Multiplier 1
               End If

            Case 8                                'Last item for this band
               If bytBand_Set < bytBand_Num Then
                  Incr bytBand_Set
                  bytSetup = 0
                  bytSetupAct = 0
               End If
         End Select

         If bytKey_lock = 1 Then                  'Lock key pressed (Next item)
            Incr bytSetup
            If bytSetup > 7 Then                  'Last item for this band
               If bytBand_Set < bytBand_Num Then
                  Incr bytBand_Set                'Next band
                  bytSetup = 0
               Else
                  bytMode = 1                     'Start automatic mode again
                  wrdSeg_FreqHi = 0               'Force FindBand to get segment again
                  Cls
                  GoTo Restart
               End If
            End If
            Call ClearLCDLine(3 , bytChar)
            bytSetupAct = 0
         End If
         If bytKey_reset = 1 Then                 'Reset key pressed (Return)
            If bytSetup > 1 Then
               Decr bytSetup                      'Previous setup item
            Else
               If bytBand_Set > 1 Then
                  Decr bytBand_Set                'Previous band
                  bytSetup = 0
               End If
            End If
            Call ClearLCDLine(3 , bytChar)
            bytSetupAct = 0
         End If

   End Select                                     'End of Mode selection

   '***************************************************************************
   'Send data to Remote unit (automatic or manual mode)
   'Conditions:
   '  - Only if L, C, H/L pass have been changed (bitLC_changed=1).
   '  - If TX is on , only if muted power is measured and sufficiently low
   '    for relay switching (bytPwr_high=0).
   '    While ATU is switched TX power is to be muted.
   '    If muted TX power is
   '    not measured (bytPwr_high=1)or too high (bytPwr_high=2)
   '    ATU is not switched if TX is on (relay protection).
   '  - If TX is off, only for SWL mode or if ATU_TRX Com is active.
   '***************************************************************************
   'Prepare conditions for remote unit send action
   If bytMode <= 2 Then                           'Automatic or manual mode
      If bitRemote = On Then                      'Remote unit is enabled
         If bitLC_changed = 1 Then                'L, C or H/L pass are changed
            If IsComBusy() = 0 Then               'Bus is not busy
               bitRemoteSnd = 1                   'Send command to Remote unit
               If bitTXOn = ON Then               'TX is on
                  If bytSWLopt = ON Then          'SWL  option
                     Locate bytRowPwr , 1
                     LCD "SWL option, TX is on"
                     bitRemoteSnd = 0             'Do not send command to Remote unit
                  Else                            'TRX option
                     'If TX power is to high wait for power reduction
                     Do
                        bitTXmute = On            'TX-Mute voltage on
                        bitTXPTT = On             'TX-PTT relay on
                        If bytTXDelay > 9 Then
                           Call TXMuteDelay       'Wait for TX to power down
                        End If
                        Call GetPower             'Muted TX power(W)
                        bitTXmute = Off           'TX-Mute voltage off
                        bitTXPTT = Off            'TX-PTT relay off
                        If sngFwdPwr < sngPWRmax Then       'TX power OK for ATU switching
                           bitMute = 1            'Flag for mute TX on
                           If bytTuneMan = 1 Then 'Permanent mute is set
                              bitDelay = 0        'Flag for mute TX delay off
                           Else
                              If bytTXDelay > 9 Then
                                 bitDelay = 1     'Flag for mute TX delay on
                              Else
                                 bitDelay = 0     'Flag for mute TX delay off
                              End If
                           End If
                           Exit Do
                        Else                      'Power too high for ATU switching
                           Call ClearLCDline(bytRowPwr , bytChar)
                           LCD "Power " ; Fusing(sngFwdPwr , "#.#") ; "W"
                           Locate bytRowSwr , 1
                           LCD "Please reduce power "
                           Waitms 500
                        End If
                     Loop
                  End If
                  If bitRemoteSnd = 0 Then        'Error detected
                     bytEnc_L = bytEncL_old       'Set old values if error
                     bytEnc_C = bytEncC_old
                     bitHL_pass = bitHL_old
                     Wait 2
                     Call ClearLCDLine(bytRowPwr , bytChar)
                  End If
               Else                               'TX is off
                  bitMute = 0                     'Flag for mute TX off
                  bitDelay = 0                    'Flag for mute TX delay off
                  If bitTRXCom = Off And bytSWLopt = OFF Then       'TRX com & SWL option not active
                     bitRemoteSnd = 0             'Do not send command to Remote unit
                  End If
               End If

               'Conditions complete, send command to remote unit
               If bitRemoteSnd = 1 Then           'Send remote unit command
                  If bitMute = 1 Then             'Mute flag is set
                     bitTXmute = On               'TX-Mute voltage on
                     bitTXPTT = On                'TX-PTT relay on
                  End If
                  If bitDelay = 1 Then            'Delay flag is set
                     Call TXMuteDelay             'Wait for TX to power down
                  End If
                  Call SendRemote(2)              'Send to remote unit and get response
                  If bytTuneMan = 0 Then          'Permanent mute is not set
                     bitTXmute = Off              'TX-Mute voltage off
                     bitTXPTT = Off               'TX-PTT relay off
                  End If
               End If
               bitLC_changed = 0
            End If                                'End If bitComBusy = 0
         End If                                   'End If bitLC_changed = 1
      End If                                      'End If bitRemote = On
   End If                                         'End If bytMode <=2

Loop                                              'End of main program ********

'******************************************************************************
'Interrupt service routines
'******************************************************************************
'=================================================================== Timer0_isr
Timer0_isr:
'Interrupt Service Routine for Timer0
'Input:     bytTimer0_cnt     Overflow counter for counter gate
'           bytTimer0_runs    Max Overflow counts for counter gate
'Output:    wrdTRXfreqCnt     Measured frequency in kHz
'           bitCounterNull    =1: Signal lost within gate time
'                                 Frequency counting is not correct
'           bitCounterDone    =1: Gate closed, frequency is available
'           wrdCountNull & wrdCountDone to be initialized with 0 outside

   Timer0 = Presettimer0

   'Counter -------------------------------------------------------------------
   If bytTimer0_cnt = 0 Then                      'start counter
      Enable Timer1                               'Counter Overflow Interrupts on
      Start Timer1
      bytTimer0_cnt = 1
      wrdCountSav = Timer1
   Elseif bytTimer0_cnt < bytTimer0_runs Then     'wait for bytTimer0_cnt Timer0 overflows
      wrdCountDif = wrdCountSav - Timer1
      wrdCountSav = Timer1
      'Was TX turned off within the gate time?
      If wrdCountDif < 10 Then                    'Counts not increased: TX off
         bitCounterNull = 1                       'Null signal detected
      End If
      Incr bytTimer0_cnt                          'increment Timer1 Overflow counter
   Else                                           'close gate, get Timer1 counter
      Stop Timer1
      Disable Timer1
      wrdCountLo = Timer1                         'get low word
      dwdTmp0 = wrdCountHi                        'High word
      Shift dwdTmp0 , Left , 16                   'Shift up for low word (mult. by 2^16)
      dwdTmp0 = dwdTmp0 + wrdCountLo              'add low word
      Shift dwdTmp0 , Right , 4                   'Divide by 16 (2^4)=frequency in kHz
      wrdTRXfreqCnt = dwdTmp0                     'TX frequency in kHz
      bitCounterDone = 1
      Timer1 = 0                                  'reset timer1
      bytTimer0_cnt = 0
      wrdCountHi = 0                              'reset counter
      wrdCountLo = 0
   End If

Return

'=================================================================== Timer1_isr
Timer1_isr:
'Interrupt Service Routine for Timer1 (Counter)
'increment counter high word (Count+65536) with each Timer1 interrupt

   Incr wrdCountHi

Return

'=================================================================== Timer2_isr
Timer2_isr:
'Interrupt Service Routine for Timer2 (Keys and Encoder)
'Encoder interrupt on Timer2 overflow
'While data are displayed Timer2 should be stopped
'Switching pattern of ALPS STEC11 encoder:
'Turning clockwise              ... counter clockwise
'A    B                             A     B
'1    1  start detent position      1     1
'1    0  intermediate position 1    0     1
'0    0  intermediate position 2    0     0
'0    1  intermediate position 3    1     0
'1    1  next detent position       1     1
'Output: bytEnc_Incr   = 1: decremented (turned counter clockwise)
'                      = 2: incremented (turned         clockwise)
'        Having evaluated bytEnc_Incr in the main program reset it (=0)
'        bytTimer2_LED     LED flashing counter
'        bitLED_on         LED flashing signal
'        bytTimeout_cnt    ATU communication timeout counter
'        bytCallTRX_cnt    TRX frequency request scheduler
'        wrdTimer2_PEP     PEP peak hold scheduler
'Timer sampling rate 120 Hz (8.3 msec) should be sufficient
'to be set with PresetTimer2 (greater value -> faster)

   Timer2 = Presettimer2

   'Keys ----------------------------------------------------------------------
   'got from Thomas Baer in Microcontroller.net
   bytKey_save = bytKey_state                     'save old status before keys were pressed
   bytIwr0 = Key_port                             'get Bits of the KeyPorts
   bytIwr0 = Not bytIwr0
   bytIwr0 = bytIwr0 Xor bytKey_state
   bytKey_ct0 = bytKey_ct0 And bytIwr0
   bytKey_ct1 = bytKey_ct1 And bytIwr0
   bytKey_ct0 = Not bytKey_ct0
   bytKey_ct1 = bytKey_ct1 Xor bytKey_ct0
   bytIwr0 = bytIwr0 And bytKey_ct0
   bytIwr0 = bytIwr0 And bytKey_ct1
   bytKey_state = bytKey_state Xor bytIwr0
   bytIwr0 = bytIwr0 And bytKey_state
   bytKey_press = bytKey_press Or bytIwr0

   'Encoder -------------------------------------------------------------------
   bytEnc_new.0 = bitEnc_A                        'set Bit0 of bytEnc_new to Enc_A
   bytEnc_new.1 = bitEnc_B                        'set Bit1 of bytEnc_new to Enc_B

   'Check encoder pattern if encoder is changed
   'bytEnc_Incr = 1: decremented (turned counter clockwise)
   'bytEnc_Incr = 2: incremented (turned         clockwise)
   If bytEnc_new <> bytEnc_old Then               'encoder is changed
      'from intermediate position 3 to quiescent position
      If bytEnc_new = &B00000011 And bytEnc_old = &B00000010 Then bytEnc_Incr = 2
      If bytEnc_new = &B00000011 And bytEnc_old = &B00000001 Then bytEnc_Incr = 1
      'from intermediate position 1 to intermediate position 2
      'DoubleStep = 1:  intermediate position 2 gives an additional tick
      '                 2 ticks between 2 detent positions on slow rotation
      '           = 0:  1 tick  between 2 detent positions
      #If DoubleStep = 1
         If bytEnc_new = &B00000000 And bytEnc_old = &B00000001 Then bytEnc_Incr = 2
         If bytEnc_new = &B00000000 And bytEnc_old = &B00000010 Then bytEnc_Incr = 1
      #EndIf
      bytEnc_old = bytEnc_new                     'old <- new for the next time
   End If

   'LED flashing --------------------------------------------------------------
   '~ 1 sec sec pulses for LED flashing and for remote unit time out
   If bytTimer2_LED < 65 Then                     'Bit-Toggle for LED flashing
      Incr bytTimer2_LED                          '~ 0.55 sec
   Else
      Toggle bitLED_on
      bytTimer2_LED = 0
   End If

   'Communication timeout -----------------------------------------------------
   Incr bytTimeout_cnt                            'Counter for receive time out

   'TRX frequency request scheduler -------------------------------------------
   Incr bytCallTRX_cnt                            'Counter for TRX frequency request

   'PEP peak hold scheduler ---------------------------------------------------
   Incr wrdTimer2_PEP                             'Counter for SSB PEP peak hold

Return

'====================================================================== ADC_isr
ADC_isr:
'Interrupt Service Routine for ADC, conversion complete
'ADC is configured as single mode. Must be explicitely started outside.
'Once started, ADC runs until it is disabled outside.
'Read 10bit ADC Data Registers ADCL & ADCH
'bytADCLo and bytADCHi are overlayed with wrdADC
'Set next MUX channel and start ADC again
'Input:  bytADChannel   Current ADC channel to be read
'        bytADCUsed     Number of ADC channels to be read minus 1
'Output: bytADCChannel  Next ADC channel, =0 if all channels are read
'        bitADCdone     =1: All specified ADC channels are read

   bytADCLo = ADCL                                'Read ADCL first...
   bytADCHi = ADCH                                '... then ADCH
   bytADCValIdx = bytADChannel + 1                'Next value array position
   wrdADCVal(bytADCValIdx) = wrdADC               '10bit ADC value
   Incr bytADChannel                              'Next MUX channel
   If bytADChannel > bytADCUsed Then
      bytADChannel = 0
      bitADCdone = 1                              'Accumulation complete
   End If
   Admux = bytADMUX Or bytADChannel               'Set next MUX channel
   ADCSRA.ADSC = 1                                'Start ADC again

Return

'==============================================================================
'URXC receive interrupt service routine
'Output:  bytBuffInp(1...byt_Count), Input buffer
'         byt_Count, current length of input buffer

RXD_isr:
   Incr bytBuffCount
   bytBuffInp(bytBuffCount) = UDR                 'Put next byte into buffer
Return

'******************************************************************************
'ADC & SWR/Power routines
'******************************************************************************
'===================================================================== AutoTune
Sub AutoTune
'Automatic tuning, find ATU settings with defined min SWR (=max return loss)
'1    If SWR < 2 search better (higher) RL in the current region, step #3
'2    If not, search for return loss peak > 6dB with defined coarse L/C steps
'        If found, continue with #3, set RLPeakFound = 1
'        If not found, toggle hi/lopass and repeate search for RL peak > 6dB
'           If found, continue with #3, set RLPeakFound = 1
'           If not found "No RL peak found", abort scan
'3    If RL > 6dB peak found, refine L/C steps to get the highest RL (lowest SWR)
'     If defined target SWR found: bytRLPeakFound = 2, set ATU and display
'     If not, take the best setting found

   'Check muted TX power ------------------------------------------------------
   Cls
   Locate 1 , 1
   LCD "***  Auto tune   ***"
   Locate 4 , 1
   LCD "Cancel -> press Tune"
   bytPass = 0
   bitTXmute = On                                 'TX-Mute voltage on
   bitTXPTT = On                                  'TX-PTT relay on
   If bytTXDelay > 9 Then
      Call TXMuteDelay                            'Wait for TX to power down
   End If
   Do
      Call IsTXOn
      If bitTXOn = On Then                        'TX is on
         Call GetPower
         If sngFwdPwr > sngPWRmax Then            'TX power not OK for ATU switching
            Call ClearLCDLine(2 , bytChar)
            LCD "Power " ; Fusing(sngFwdPwr , "#.#") ; "W"
            Locate 3 , 1
            LCD "Please reduce power "
            bytPass = 1
         Else
            If bytPass = 1 Then
               Locate 3 , 1
               LCD "Power is OK         "
               Waitms 500
            End If
            Exit Do                               '
         End If
      Else                                        'TX is off
         Call ClearLCDLine(2 , bytChar)
         LCD "Turn on TX please   "
         Call ClearLCDLine(3 , bytChar)
      End If
      If bytKey_press.Key_tune = 1 Then           'Tune key pressed
         bytKey_press.Key_tune = 0                'Reset tune key flag
         bitTXmute = Off                          'TX-Mute voltage off
         bitTXPTT = Off                           'TX-PTT relay off
         Exit Sub                                 'Leave Auto tune
      End If
      Waitms 500
   Loop
   Cls

   'Calculate best return loss from target SWR --------------------------------
   wrdAddr = 15
   ReadEeprom bytSWRAutoMin , wrdAddr
   sngTmp0 = bytSWRAutoMin                        'Target SWR (SWR*100)
   sngTmp0 = 0.01 * sngTmp0                       'Target SWR
   sngTmp1 = sngTmp0 - 1
   sngTmp2 = sngTmp0 + 1
   sngTmp0 = sngTmp1 / sngTmp2
   sngRLBest = -20 * Log10(sngTmp0)               'Best return loss(dB)

   'Check current band --------------------------------------------------------
   bytBand_Set = 0
   For bytTmp1 = 1 To bytBand_Num                 'Get the band
      If wrdTRXfreq >= wrdBand_Lo(bytTmp1) And wrdTRXfreq < wrdBand_Hi(bytTmp1) Then
         bytBand_Set = bytTmp1                    'Current band index
         Exit For                                 'Got the band
      End If
   Next bytTmp1
   If bytBand_Set = 0 Then                        'Undefined band index
      Call ClearLCDLine(3 , bytChar)
      LCD "Unknown band"
      Wait 2
      bitTXmute = Off                             'TX-Mute voltage off
      bitTXPTT = Off                              'TX-PTT relay off
      Exit Sub
   End If

   'Check current return loss -------------------------------------------------
   sngRL = ReturnLoss(5)                          'RL (dB)
   If sngRL >= sngRLBest Then
      Call ClearLCDLine(3 , bytChar)
      LCD "Current SWR is OK"
      Wait 2
      bitTXmute = Off                             'TX-Mute voltage off
      bitTXPTT = Off                              'TX-PTT relay off
      Exit Sub
   End If

   'SWR is greater than best SWR, auto tuning is to be done -------------------
   'Prepare data for auto tuning
   bytEncL_old = bytEnc_L                         'Save last ATU settings
   bytEncC_old = bytEnc_C
   bitHL_old = bitHL_pass
   sngRLFind = 6                                  'Threshhold RL (6 dB) to find peak

   'Read auto tune presettings from EEPROM (auto tuning setup) for current band
   wrdAddr = AutoTuneAdr(0 , bytBand_Set)
   ReadEeprom bytStepMaxC , wrdAddr               'Max step width C
   wrdAddr = AutoTuneAdr(1 , bytBand_Set)
   ReadEeprom bytStepMaxL , wrdAddr               'Max step width L
   wrdAddr = AutoTuneAdr(2 , bytBand_Set)
   ReadEeprom bytPassBand , wrdAddr               'Start with lo/hipass
   bitHL_pass = bytPassBand
   wrdAddr = AutoTuneAdr(3 , bytBand_Set)
   ReadEeprom bytStartC , wrdAddr                 'Start with ATU C value
   wrdAddr = AutoTuneAdr(4 , bytBand_Set)
   ReadEeprom bytEndC , wrdAddr                   'End with ATU C value
   wrdAddr = AutoTuneAdr(5 , bytBand_Set)
   ReadEeprom bytStartL , wrdAddr                 'Start with ATU L value
   wrdAddr = AutoTuneAdr(6 , bytBand_Set)
   ReadEeprom bytEndL , wrdAddr                   'End with ATU L value
   If byt8Coils = 0 Then                          'Christian tuner, only 7 coils
      If bytEndL > 127 Then bytEndL = 127
   End If
   bytPass = 0                                    'First try with preset hi/lopass
   wrdRun = 0
   bytRLPeakFound = 0

RetryPass:                                        'Entry point for reverse hi/lopass

   Cls                                            'Row 1: display L, C and hi/lopass
   Locate 1 , 1
   LCD "L        C"
   strTmp1 = "L"
   If bitHL_pass = 1 Then strTmp1 = "H"
   Locate 1 , 20
   LCD strTmp1

   bytTuneStepL = bytStepMaxL
   bytTuneStepC = bytStepMaxC

   'If SWR is < 2 (RL>9.54) take current L/C setting as start condition
   'Else search RL peak
   If sngRL >= 9.5 Then                           'SWR <=2
      'For rather good SWR<2 check preset hi/lopass (bitHL_pass)
      'Preset hi/lopass = EEPROM data set in auto tuning setup
      'If it is other than current setting (bitHL_old) toggle it
      If bitHL_pass <> bitHL_old Then             'Preset hi/lopass other than current
         strTmp10 = "Hi"
         If bitHL_pass = 0 Then strTmp10 = "Lo"
         Locate 2 , 1
         LCD strTmp10 ; "pass does not fit "
         Toggle bitHL_pass                        'hi to lopass or lo to hipass
         If bitHL_pass = 0 Then
            strTmp10 = "lo"
            strTmp1 = "L"
         Else
            strTmp10 = "hi"
            strTmp1 = "H"
         End If
         Locate 3 , 1
         LCD "Changed to " ; strTmp10 ; "pass    "
         Locate 1 , 20
         LCD strTmp1
      End If
      bytLMax = bytEnc_L
      bytCMax = bytEnc_C
      Call HalfStep                               'Halve step sizes
      intTmp0 = bytEnc_L - bytTuneStepL
      If intTmp0 > 0 Then
         bytEnc_LStart = intTmp0
      Else
         bytEnc_LStart = 0
      End If
      intTmp0 = bytEnc_L + bytTuneStepL
      If intTmp0 < 255 Then
         bytEnc_LStop = intTmp0
      Else
         bytEnc_LStop = 255
      End If
      intTmp0 = bytEnc_C - bytTuneStepC
      If intTmp0 > 0 Then
         bytEnc_CStart = intTmp0
      Else
         bytEnc_CStart = 0
      End If
      intTmp0 = bytEnc_C + bytTuneStepC
      If intTmp0 < 255 Then
         bytEnc_CStop = intTmp0
      Else
         bytEnc_CStop = 255
      End If
      If byt8Coils = 0 Then                       'Christian tuner, only 7 coils
         If bytEnc_LStop > 127 Then bytEnc_LStop = 127
      End If
      sngRLMax = sngRL
      sngRLPeak = sngRL
      bytRLPeakFound = 1
      Call DisplayTune(1)
   Else                                           'SWR > 2, search RL peak
      bytEnc_LStart = bytStartL
      bytEnc_CStart = bytStartC
      bytEnc_LStop = bytEndL
      bytEnc_CStop = bytEndC

      'Start RL peak search (Cold start) --------------------------------------
      'If RL peak > 6dB is found, bytRLPeakFound = 1
      sngRLMax = 0
      sngRLPeak = 0
      bytRLPeakFound = 0
      bytADChannel = 0                            'First ADC channel ADC0 (FWD)
      bitADCdone = 0

      For bytEnc_C = bytEnc_CStart To bytEnc_CStop Step bytTuneStepC

         For bytEnc_L = bytEnc_LStart To bytEnc_LStop Step bytTuneStepL

            Call DisplayTune(1)
            Incr wrdRun
            If bytKey_press.Key_tune = 1 Then     'Tune key pressed
               bytRLPeakFound = 3                 'Tuning aborted
               bytKey_press.Key_tune = 0          'Reset tune key flag
               Exit For
            End If
            If IsComBusy() = 0 Then               'Bus is not busy
               Call SendRemote(3)
               Waitms 10                          'Wait for relay to settle
               sngRL = ReturnLoss(3)              'Measure return loss & power
               If sngFwddBm < -40 Then            'AD8307 input power < -40dBm
                  bytRLPeakFound = 9              'TX turned off
                  Exit For                        'Exit inner loop
               End If
               If sngRL > sngRLMax Then           'Local RL maximum found
                  sngRLMax = sngRL
                  bytLMax = bytEnc_L
                  bytCMax = bytEnc_C
               End If
               If sngRL > sngRLFind Then          'RL above threshhold
                  If sngRL > sngRLPeak Then
                     sngRLPeak = sngRL
                  End If
               End If
               'Peak detected and RL decreased by 10% again?
               If sngRLPeak > 0 Then
                  sngTmp0 = 0.9 * sngRLPeak
                  If sngRL < sngTmp0 Then         'This must be a RL peak
                     bytRLPeakFound = 1
                     Exit For                     'Peak found, exit inner loop
                  End If
               End If
            End If

            'Workaround for bug in BASCOM-AVR 2.0.8.2, build 003
            wrdEnc_L = bytEnc_L                   'Force exit at loop end
            wrdTmp0 = bytTuneStepL
            wrdEnc_L = wrdEnc_L + wrdTmp0
            If wrdEnc_L > 255 Then Exit For
         Next bytEnc_L
                                    'End of inner L loop
         If bytRLPeakFound = 1 Then Exit For      'Peak found, exit outer loop
         If bytRLPeakFound = 3 Then Exit For      'Scan manually aborted
         If bytRLPeakFound = 9 Then Exit For      'TX turned off, exit outer loop

         'Workaround for bug in BASCOM-AVR 2.0.8.2, build 003
         wrdEnc_C = bytEnc_C                      'Force exit at loop end
         wrdTmp0 = bytTuneStepC
         wrdEnc_C = wrdEnc_C + wrdTmp0
         If wrdEnc_C > 255 Then Exit For
      Next bytEnc_C                               'End outer C loop

   End If                                         'End of If sngRL >= 9.5

   If bytRLPeakFound = 0 Then                     'Return loss peak not found
      'Reverse hi/lopass and try it again
      If bytPass = 0 Then                         'Was 1st run with preset hi/lopass
         Locate 3 , 1
         If bitHL_pass = 0 Then
            strTmp10 = "Lo"
         Else
            strTmp10 = "Hi"
         End If
         LCD "** " ; strTmp10 ; "pass failed ** "
         Toggle bitHL_pass                        'hi->lo or lo->hipass
         Locate 4 , 1
         If bitHL_pass = 0 Then
            strTmp10 = "lo"
         Else
            strTmp10 = "hi"
         End If
         LCD "Try " ; strTmp10 ; "pass now"
         Wait 2
         bytPass = 1                              '2nd try with reverse hi/lopass
         bytRLPeakFound = 0
         sngRL = 0
         GoTo RetryPass                           'Back to the top
      End If

   ElseIf bytRLPeakFound = 1 Then                 'Return loss peak found
      'Now find the max. return loss (min SWR)                             '
      Call NewBoundaries(1 , bytCMax , bytTuneStepC)       'C boundaries
      Call NewBoundaries(2 , bytLMax , bytTuneStepL)       'L boundaries
      Call HalfStep                               'Halve step sizes
      bytLMaxOld = bytLMax
      bytCMaxOld = bytCMax
      sngRLMaxOld = sngRLMax

      Do
         If bytKey_press.Key_tune = 1 Then        'Tune key pressed
            bytRLPeakFound = 3                    'Tuning aborted
            bytKey_press.Key_tune = 0             'Reset tune key flag
            Exit Do
         End If

         For bytEnc_C = bytEnc_CStart To bytEnc_CStop Step bytTuneStepC

            For bytEnc_L = bytEnc_LStart To bytEnc_LStop Step bytTuneStepL

               Call DisplayTune(1)
               If IsComBusy() = 0 Then            'Bus is not busy
                  Call SendRemote(3)
                  Waitms 10                       'Wait for relay to settle
                  sngRL = ReturnLoss(3)           'Measure return loss
                  If sngFwddBm < -40 Then         'AD8307 input power < -40dBm
                     bytRLPeakFound = 9           'TX turned off
                     Exit Do                      'Exit Do...loop
                  End If
                  If sngRL > sngRLMax Then        'Better RL found
                     sngRLMax = sngRL
                     bytLMax = bytEnc_L
                     bytCMax = bytEnc_C
                  End If
               End If
               If sngRLMax >= sngRLBest Then
                  bytRLPeakFound = 2              'Best SWR found
                  Exit Do                         'Thats it, evaluate scan result
               End If

               'Workaround for bug in BASCOM-AVR 2.0.8.2, build 003
               wrdEnc_L = bytEnc_L                'Force exit at loop end
               wrdTmp0 = bytTuneStepL
               wrdEnc_L = wrdEnc_L + wrdTmp0
               If wrdEnc_L > 255 Then Exit For
            Next bytEnc_L

            'Workaround for bug in BASCOM-AVR 2.0.8.2, build 003
            wrdEnc_C = bytEnc_C                   'Force exit at loop end
            wrdTmp0 = bytTuneStepC
            wrdEnc_C = wrdEnc_C + wrdTmp0
            If wrdEnc_C > 255 Then Exit For
         Next bytEnc_C

         If sngRLMax > sngRLMaxOld Then           'New maximum found
            'Shift search square to new maximum, leave the current step size
            If bytCMax <> bytCMaxOld Or bytLMax <> bytLMaxOld Then
               Call ShiftSearch(1 , bytCMax , bytCMaxOld , bytTuneStepC)       'Shift C axis
               Call ShiftSearch(2 , bytLMax , bytLMaxOld , bytTuneStepL)       'Shift L axis
               bytCMaxOld = bytCMax
               bytLMaxOld = bytLMax
               sngRLMaxOld = sngRLMax
               bytRefine = 0
            Else                                  'L & C unchanged
               bytRefine = 1                      'Refine step size
            End If
         Else                                     'No new maximum found
            bytRefine = 1                         'Refine step size
         End If
         If bytRefine = 1 Then                    'Refine the step size to one half
            If bytTuneStepC = 1 And bytTuneStepL = 1 Then       'Step 1 was the last trial
               Exit Do
            End If
            Call HalfStep                         'Halve step sizes
            Call NewBoundaries(1 , bytCMax , bytTuneStepC)       'C boundaries
            Call NewBoundaries(2 , bytLMax , bytTuneStepL)       'L boundaries
         End If
      Loop
   End If

   'Evaluate scan result ------------------------------------------------------
   'bytRLPeakFound = 0: No RL peak > 6dB found, no antenna system resonance
   'bytRLPeakFound = 1: RL peak > 6dB found but no best SWR
   'bytRLPeakFound = 2: RL peak > 6dB found and best SWR
   'bytRLPeakFound = 3: Scan manually aborted
   'bytRLPeakFound = 9: TX turned off, power at AD8307 input < -40dBm

   'Locate 3 , 1
   'LCD bytRLPeakFound ; ": " ; Fusing(sngRLMax , "#.##")
   'Wait 3

   Select Case bytRLPeakFound
      Case 0                                      'No RL peak > 6dB found
         Locate 3 , 1
         LCD "* No RL peak found *"
         Locate 4 , 1
         LCD "*** Scan aborted ***"
         Wait 3
         bytEnc_L = bytEncL_old                   'Recall old ATU settings
         bytEnc_C = bytEncC_old
         bitHL_pass = bitHL_old
      Case 1                                      'RL peak > 6dB found but no best SWR
         sngTmp0 = bytSWRAutoMin                  'Min SWR (SWR*100)
         sngTmp0 = 0.01 * sngTmp0
         strTmp10 = Fusing(sngTmp0 , "#.##")
         Locate 3 , 1
         LCD "SWR < " ; strTmp10 ; " not found"
         Locate 4 , 1
         LCD "Set best SWR found  "
         Wait 2
         bytEnc_L = bytLMax                       'ATU L for best RL found
         bytEnc_C = bytCMax                       'ATU C for best RL found
      Case 2                                      'RL peak > 6dB found and best SWR
         bytEnc_L = bytLMax                       'Best ATU L/C settings
         bytEnc_C = bytCMax
      Case 3                                      'Scan manually aborted
         Locate 4 , 1
         LCD "*** Scan aborted ***"
         Wait 3
         bytEnc_L = bytEncL_old                   'Recall old ATU settings
         bytEnc_C = bytEncC_old
         bitHL_pass = bitHL_old
      Case 9                                      'TX turned off while tuning
         Locate 3 , 1
         LCD "TX was turned off!  "
         Locate 4 , 1
         LCD "*** Scan aborted ***"
         Wait 3
         bytEnc_L = bytEncL_old                   'Recall old ATU settings
         bytEnc_C = bytEncC_old
         bitHL_pass = bitHL_old
   End Select

   If IsComBusy() = 0 Then                        'Bus is not busy
      Call SendRemote(3)                          'Set ATU L/C settings
   End If

'(
   Waitms 2                                       'Wait for relay to settle
   sngRL = ReturnLoss(3)                          'Measure return loss
   Call ClearLCDLine(4 , 20)
   strTmp10 = Fusing(sngRL , "#.##")
   LCD "RL " ; strTmp10 ; " runs " ; wrdRun
   Wait 5
')

   bitTXmute = Off                                'TX-Mute voltage off
   bitTXPTT = Off                                 'TX-PTT relay off

End Sub

'===================================================================== HalfStep
Sub HalfStep
'Halve current auto tune step sizes

   If bytTuneStepC > 2 Then
      bytTuneStepC = bytTuneStepC / 2
   Else
      bytTuneStepC = 1
   End If
   If bytTuneStepL > 2 Then
      bytTuneStepL = bytTuneStepL / 2
   Else
      bytTuneStepL = 1
   End If
End Sub

'================================================================ NewBoundaries
Sub NewBoundaries(byVal bytOpt As Byte , byVal bytMax As Byte , byVal bytStep As Byte)
'Set new boundaries around bytMax for L or C
'Input:  bytOpt         =1: C boundaries, =2: L boundaries
'        bytMax         Current max C or L
'        intStep        step width for ATU O or L variation
'Output: bytEnc_CStart  Start boundary for C (0...255)
'        bytEnc_CStop   Stop  boundary for C (0...255)
'        bytEnc_LStart  Start boundary for L (0...255)
'        bytEnc_LStop   Stop  boundary for L (0...255)

Local bytBOStart As Byte
Local bytBOStop As Byte

   intTmp0 = bytMax
   intTmp1 = bytStep
   intTmp2 = intTmp0 - intTmp1                    'Start boundary
   intTmp3 = intTmp0 + intTmp1                    'Stop boundary
   If intTmp2 > 0 Then
      bytBoStart = intTmp2
   Else
      bytBoStart = 0
   End If
   If intTmp3 < 256 Then
      bytBoStop = intTmp3
   Else
      bytBoStop = 255
   End If

   If bytOpt = 1 Then                             'C axis
      bytEnc_CStart = bytBoStart
      bytEnc_CStop = bytBoStop
   Else                                           'L axis
      bytEnc_LStart = bytBoStart
      bytEnc_LStop = bytBoStop
   End If
   If byt8Coils = 0 Then                          'Christian tuner, only 7 coils
      If bytEnc_LStart > 127 Then bytEnc_LStart = 127
      If bytEnc_LStop > 127 Then bytEnc_LStop = 127
   End If

End Sub

'================================================================== ShiftSearch
Sub ShiftSearch(byVal bytOpt As Byte , byVal bytMax As Byte , byVal bytMaxOld As Byte , byVal bytStep As Byte)
'Shift search square around current maximum L or C
'Input:  bytOpt         =1: C square, =2: L square
'        bytMax         Current max C or L
'        bytMaxOld      Old     max C or L
'        intStep        +/- step width for ATU L or L variation
'                       Positive for up direction, negative for down direction
'Output: bytEnc_CStart  Start boundary for C (0...255)
'        bytEnc_CStop   Stop  boundary for C (0...255)
'        bytEnc_LStart  Start boundary for L (0...255)
'        bytEnc_LStop   Stop  boundary for L (0...255)
Local bytBOStart As Byte
Local bytBOStop As Byte

   intTmp0 = bytMax
   intTmp1 = 2 * bytStep
   intTmp2 = 3 * bytStep
   intTmp4 = bytStep
   Select Case bytMax
      Case Is < bytMaxOld                         'New max is lower
         'Shift low  boundary intTmp1 below new max
         'Shift high boundary intStep above new max
         intTmp3 = intTmp0 - intTmp1
         If intTmp3 > 0 Then
            bytBoStart = intTmp3
         Else
            bytBoStart = 0
         End If
         intTmp3 = intTmp0 + intTmp4
         If intTmp3 < 256 Then
            bytBoStop = intTmp3
         Else
            bytBoStop = 255
         End If
      Case Is > bytMaxOld                         'New max is higher
         'Shift low  boundary intStep below new max
         'Shift high boundary intTmp1 above new max
         intTmp3 = intTmp0 - intTmp4
         If intTmp3 > 0 Then
            bytBoStart = intTmp3
         Else
            bytBoStart = 0
         End If
         intTmp3 = intTmp0 + intTmp1
         If intTmp3 < 256 Then
            bytBoStop = intTmp3
         Else
            bytBoStop = 255
         End If
      Case Else                                   'Max unchanged
         'Set low  boundary intStep below max
         'Set high boundary intStep above max
         intTmp3 = intTmp0 - intTmp4
         If intTmp3 > 0 Then
            bytBoStart = intTmp3
         Else
            bytBoStart = 0
         End If
         intTmp3 = intTmp0 + intTmp4
         If intTmp3 < 256 Then
            bytBoStop = intTmp3
         Else
            bytBoStop = 255
         End If
   End Select

   If bytOpt = 1 Then
      bytEnc_CStart = bytBoStart
      bytEnc_CStop = bytBoStop
   Else
      bytEnc_LStart = bytBoStart
      bytEnc_LStop = bytBoStop
   End If
   If byt8Coils = 0 Then                          'Christian tuner, only 7 coils
      If bytEnc_LStart > 127 Then bytEnc_LStart = 127
      If bytEnc_LStop > 127 Then bytEnc_LStop = 127
   End If

End Sub

'=================================================================== CalibMenu
Sub CalibMenu
'Display coupler calibration menu and initialize data
'Output: bytSetupAct
'        bytSetup

   Cls
   Locate 1 , 1
   LCD "Calibrate coupler"
   Locate 2 , 1
   LCD "Select with encoder"
   Locate 3 , 1
   LCD "With TX " ; Chr(223) ; "Encoder key"
   Locate 4 , 1
   LCD "ESC"
   bytSetupAct = 0                                'Initialize calibration
   bytSetup = 0

End Sub

'================================================================ TuneSetupMenu
Sub TuneSetupMenu
'Display Setup auto tune menu and initialize data
'Output: bytSetupAct
'        bytSetup
'        bytBand_Set

   Cls
   Locate 1 , 1
   LCD "Setup auto tune"
   Locate 4 , 1
   LCD "ESC Sto  Ret Next   "
   bytSetupAct = 0                                'Initialize setup
   bytSetup = 0
   bytBand_Set = 1                                'Band index

End Sub

'================================================================= CalCouplerTX
Sub CalCouplerTX(byVal wrdAdd1 As Word , byVal wrdAdd2 As Word)
'Calibrate coupler log amps, normal calibration with TX
'Input:  bytSetup    =1: FWD, lower calibration level
'                    =2: REV, lower calibration level
'                    =3: FWD, upper calibration level
'                    =4: REV, upper calibration level
'        bytSetupAct =1: Read & show calibration level hi/lo
'                    =2: Wait for go for measurement
'                    =3: Read old ADC value, measure new ADC value
'                    =4: ADC counts measured, wait for store
'        wrdAdd1     EEPROM read  address of calibration level hi/lo
'        wrdAdd2     EEPROM write address of ADC data FWD/REV, hi/lo

   Select Case bytSetupAct
      Case 1                                      'Read & show EEPROM data
         wrdAddr = wrdAdd1
         Call EEPROMReadInt(intTmp0)              'Calibration point (dBm x 10)
         intTmp0 = intTmp0 + wrdCoupFac           'Add Coupling factor (dB x 10)
         intTmp0 = intTmp0 + wrdAmpAtt            'Add Log amp attenuator (dB x 10)
         strTmp10 = Str(intTmp0)
         strTmp10 = Format(strTmp10 , "0.0")
         Call ClearLCDLine(2 , bytChar)
         If bytSetup < 3 Then                     'Lo level calibration
            LCD "Lo level "
         Else
            LCD "Hi level"
         End If
         Locate 2 , 10
         LCD strTmp10 ; "dBm"
         Call ClearLCDLine(3 , bytChar)
         'Connect TRX out and dummy load at coupler ports TX/Ant and vice versa
         If bytSetup = 1 Or bytSetup = 3 Then
            LCD "TX " ; Chr(223) ; "TX in   ->Go" 'Forward lo or hi
         Else
            LCD "TX " ; Chr(223) ; "Ant out ->Go" 'Reverse lo or hi
         End If
         Locate 4 , 1
         LCD "ESC           Go    "
         bytSetupAct = 2

      Case 2                                      'Wait for Go
         If bytKey_lock = 1 Then                  'Lock key pressed (Go)
            Call ClearLCDLine(3 , bytChar)
            If bytSetup = 1 Or bytSetup = 3 Then  'Forward
               LCD "ADC FWD "
            Else                                  'Reverse
               LCD "ADC REV "
            End If
            bytSetupAct = 3
            ADCSRA.ADSC = 1                       'Start ADC
            bytADCRun = 0
            bytADChannel = 0                      'First ADC channel ADC0
            bitADCdone = 0
            sngADCFwd = 0                         'ADC mean counts forward
            sngADCRev = 0                         'ADC mean counts reverse
            wrdADCMean(1) = 0
            wrdADCMean(2) = 0
         End If

      Case 3                                      'Measure ADC data
         wrdAddr = wrdAdd2
         Call EEPROMReadWrd(wrdTmp0)              'Read old ADC value
         strTmp10 = str(wrdTmp0)
         strTmp10 = Format(strTmp10 , "   0")
         Locate 3 , 9
         LCD strTmp10                             '... and show it
         Locate 4 , 1
         LCD "ESC Sto Ret  Next   "
         If bitADCdone = 1 Then                   'ADC accumulation complete
            Call GetADCData                       'Handle ADC values
            Toggle bitLock
            Led_lock = bitLock                    'Lock LED on/off
            If bitADCMean = 1 Then                'ADC mean data available
               Led_lock = Off
               bitLock = Off
               Select Case bytSetup
                  Case 1
                     wrdADCCal = sngADCFwd        'FWD lo
                  Case 2
                     wrdADCCal = sngADCRev        'Rev lo
                  Case 3
                     wrdADCCal = sngADCFwd        'FWD hi
                  Case Else
                     wrdADCCal = sngADCRev        'REV hi
               End Select
               strTmp10 = str(wrdADCCal)          'Current measured ADC value
               strTmp10 = Format(strTmp10 , "   0")
               Locate 3 , 14
               LCD strTmp10
               bytSetupAct = 4
            End If
         End If

      Case Else                                   'Wait for action
         ADCSRA.ADSC = 0                          'Stop ADC
         If bytKey_store = 1 Then                 'Store key pressed
            wrdAddr = wrdAdd2
            Call EEPROMWriteWrd(wrdADCCal)
            Locate 3 , bytChar
            LCD "S"
            Wait 1
            Incr bytSetup                         'Next calibration
            bytSetupAct = 1
            Exit Sub
         End If
         If bytKey_lock = 1 Then                  'Lock key pressed (Next)
            Incr bytSetup                         'Next calibration
            bytSetupAct = 1
            Exit Sub
         End If
         If bytKey_reset = 1 Then                 'Reset key pressed (Return)
            If bytSetup > 1 Then
               Decr bytSetup                      'Previous calibration
               bytSetupAct = 1
               Exit Sub
            End If
         End If

   End Select
End Sub

'=================================================================== CalCouplerTX
Sub CalCouplerMan(byVal wrdAdd1 As Word , byVal wrdAdd2 As Word)
'Calibrate coupler log amps, enter cal data by hand
'Input:  bytSetup    =1: FWD, lower calibration level
'                    =2: REV, lower calibration level
'                    =3: FWD, upper calibration level
'                    =4: REV, upper calibration level
'        bytSetupAct =1: Read & show calibration level & old ADC value hi/lo
'                    =2: Change item (next, foregoing)
'                        Enter ADC counts, wait for store
'        wrdAdd1     EEPROM read  address of calibration level hi/lo
'        wrdAdd2     EEPROM write address of ADC data FWD/REV, hi/lo

   Select Case bytSetupAct
      Case 1                                      'Read & show EEPROM data
         wrdAddr = wrdAdd1
         Call EEPROMReadInt(intTmp0)              'Calibration point (dBm x 10)
         strTmp10 = Str(intTmp0)
         strTmp10 = Format(strTmp10 , "0.0")
         Call ClearLCDLine(2 , bytChar)
         If bytSetup < 3 Then                     'Lo level calibration
            LCD "Lo level "
         Else
            LCD "Hi level"
         End If
         Locate 2 , 10
         LCD strTmp10 ; "dBm"
         Call ClearLCDLine(3 , bytChar)
         If bytSetup = 1 Or bytSetup = 3 Then     'Forward
            LCD "ADC FWD "
         Else                                     'Reverse
            LCD "ADC REV "
         End If
         wrdAddr = wrdAdd2
         Call EEPROMReadWrd(wrdADCCal)            'Read old ADC value
         strTmp10 = str(wrdADCCal)
         strTmp10 = Format(strTmp10 , "   0")
         Locate 3 , 9
         LCD strTmp10                             '... and show it
         bytSetupAct = 2

      Case 2                                      'Change ADC data
         If bytEnc_Incr > 0 Then                  'Encoder turned
            Select Case bytEnc_Incr
               Case 1                             'Encoder decremented
                  intTmp0 = wrdADCCal - 1         'Decrement by 1
                  If intTmp0 > 0 Then
                     wrdADCCal = intTmp0
                  Else
                     wrdADCCal = 0
                  End If
               Case 2                             'Encoder incremented
                  wrdADCCal = wrdADCCal + 10      'Increment by 10
                  If wrdADCCal > 1023 Then wrdADCCal = 1023
            End Select
            bytEnc_Incr = 0                       'Reset Encoder
            strTmp10 = str(wrdADCCal)
            strTmp10 = Format(strTmp10 , "   0")
            Locate 3 , 9
            LCD strTmp10                          'Show ADC value
         End If
         If bytKey_store = 1 Then                 'Store key pressed
            wrdAddr = wrdAdd2
            Call EEPROMWriteWrd(wrdADCCal)
            Locate 3 , bytChar
            LCD "S"
            Wait 1
            Incr bytSetup                         'Next calibration
            bytSetupAct = 1
            Exit Sub
         End If
         If bytKey_lock = 1 Then                  'Lock key pressed (Next)
            Incr bytSetup                         'Next calibration
            bytSetupAct = 1
            Exit Sub
         End If
         If bytKey_reset = 1 Then                 'Reset key pressed (Return)
            If bytSetup > 1 Then
               Decr bytSetup                      'Previous calibration
               bytSetupAct = 1
               Exit Sub
            End If
         End If

   End Select

End Sub

'================================================================== TwoPointCal
Sub TwoPointCal(byVal bytOpt As Byte)
'Calculate the linear equation y=mx+b from the 2 calibration points
'm = (y2-y1)/(x2-x1), b=(y1*x2-y2*x1)/(x2-x1) calculated for FWD and REV
'Input:  bytOpt               =1: Forward, =2: Reverse
'        intCaldBmLo    = y1  Low  dBm calibration point(dBm x 10)
'        intCaldBmHi    = y2  High dBm calibration point (dBm x 10)
'        Forward calibration
'        wrdCalFwdADCLo = x1  Forward ADC counts at low  cal. point
'        wrdCalFwdADCHi = x2  Forward ADC counts at high cal. point
'        Reverse calibration
'        wrdCalRevADCLo = x1  Reverse ADC counts at low  cal. point
'        wrdCalRevADCHi = x2  Reverse ADC counts at high cal. point
'Output: sngFwdGrd            Gradient m forward
'        sngFwdOrd            Ordinate b forward
'        sngRevGrd            Gradient m reverse
'        sngRevOrd            Ordinate b reverse

   Select Case bytOpt
      Case 1                                      'Forward
         wrdTmp0 = wrdCalFwdADCLo                 'x1
         wrdTmp1 = wrdCalFwdADCHi                 'x2
      Case Else                                   'Reverse
         wrdTmp0 = wrdCalRevADCLo                 'x1
         wrdTmp1 = wrdCalRevADCHi                 'x2
   End Select
   wrdTmp4 = wrdTmp1 - wrdTmp0                    'x2-x1
   intTmp0 = intCaldBmLo                          'y1 (dBm x 10)
   intTmp1 = intCaldBmHi                          'y2 (dBm x 10)

   'Calculate gradient m forward or reverse, m = (y2-y1)/(x2-x1)
   intTmp2 = intTmp1 - intTmp0                    'y2-y1 (dBm x 10)
   sngTmp0 = intTmp2                              'y2-y1 (dBm x 10)
   sngTmp0 = 0.1 * sngTmp0                        'y2-y1 (dBm)
   sngTmp1 = wrdTmp4                              'x2-x1
   sngTmp2 = sngTmp0 / sngTmp1                    '[y2-y1]/[x2-x1]=gradient m
   Select Case bytOpt
      Case 1                                      'Forward
         sngFwdGrd = sngTmp2
      Case Else                                   'Reverse
         sngRevGrd = sngTmp2
   End Select

   'Calculate ordinate b forward or reverse, b=(y1*x2-y2*x1)/(x2-x1)
   sngTmp0 = intTmp0                              'y1 (dBm x 10)
   sngTmp0 = 0.1 * sngTmp0                        'y1 (dBm)
   sngTmp1 = wrdTmp1                              'x2
   sngTmp0 = sngTmp0 * sngTmp1                    'y1*x2
   sngTmp1 = intTmp1                              'y2 (dBm x 10)
   sngTmp1 = 0.1 * sngTmp1                        'y2 (dBm)
   sngTmp2 = wrdTmp0                              'x1
   sngTmp1 = sngTmp1 * sngTmp2                    'y2*x1
   sngTmp0 = sngTmp0 - sngTmp1                    'y1*x2-y2*x1
   sngTmp1 = wrdTmp4                              'x2-x1
   sngTmp2 = sngTmp0 / sngTmp1                    '[y1*x2-y2*x1]/[x2-x1]=ordinate b

   Select Case bytOpt
      Case 1                                      'Forward
         sngFwdOrd = sngTmp2
      Case Else                                   'Reverse
         sngRevOrd = sngTmp2
   End Select

End Sub

'=================================================================== GetADCData
Sub GetADCData
'Get accumulated ADC Data
'Input:  bitADCdone     =1: ADC conversion complete (from ADC_isr)
'        wrdADCVal(1)   Current ADC counts forward
'        wrdADCVal(2)   Current ADC counts reverse
'        bytADCRep      Number of repetitive ADC accumulations
'Output: bitADCMean     =1: new ADC mean counts available
'        sngADCFwd      ADC mean counts forward
'        sngADCRev      ADC mean counts reverse
'        wrdADCPEP      ADC max counts forward (PEP)
'        bytADCRun      Current ADC accumulation number
'        bitADCdone     =0 if ADC data were read and summed

   bytADCOvl = 0                                  'Check for ADC overflow
   If wrdADCVal(1) > 1022 Then bytADCOvl = 1
   If wrdADCVal(2) > 1022 Then bytADCOvl = 2

   'Get max ADC count for PEP in FWD channel 1
   If wrdTimer2_PEP < wrdTimer2_PEPhold Then      'Current PEP evaluation period
      If wrdADCVal(1) > wrdADCPEP Then            'Index 1 = FWD power
         wrdADCPEP = wrdADCVal(1)                 'New max ADC PEP value
      End If
   Else
      wrdPEP = wrdADCPEP                          'Accumulated PEP
      wrdTimer2_PEP = 0                           'Next PEP evaluation period
      wrdADCPEP = 0
   End If

   'Get mean values for SWR and avg. power
   If bytADCRun < bytADCRep Then                  'Accumulate bytADCRep times
      wrdADCMean(1) = wrdADCMean(1) + wrdADCVal(1)       'ADC sum forward
      wrdADCMean(2) = wrdADCMean(2) + wrdADCVal(2)       'ADC sum reverse
      Incr bytADCRun
      bitADCMean = 0                              'ADC accumulating
   Else                                           'Mean values complete
      sngTmp0 = bytADCRep
      sngTmp1 = wrdADCMean(1)
      sngADCFwd = sngTmp1 / sngTmp0               'Mean value FWD
      sngTmp1 = wrdADCMean(2)
      sngADCRev = sngTmp1 / sngTmp0               'Mean value REV
      wrdADCMean(1) = 0
      wrdADCMean(2) = 0
      bitADCMean = 1                              'ADC mean value available
   End If
   bitADCdone = 0                                 'Clear ADC complete flag

End Sub

'=================================================================== ReturnLoss
Function ReturnLoss(byVal bytRep As Byte) As Single
'Measure FWD and REV ADC counts
'Calculate return loss (dB) from calibration data
'Input:  bytRep      Number of repetitive ADC measuremants
'        sngGrdFWD   Calibration gradient FWD
'        sngOrdFWD   Calibration ordinate FWD
'        sngGrdREV   Calibration gradient REV
'        sngOrdREV   Calibration ordinate REV
'Output: ReturnLoss (dBx100)

   sngADCFwd = 0
   sngADCRev = 0
   bitADCdone = 0
   ADCSRA.ADSC = 1                                'Start ADC
   For bytTmp0 = 1 To bytRep
      BitWait bitADCdone , Set                    'ADC accumulation complete
      sngTmp0 = wrdADCVal(1)
      sngADCFwd = sngADCFwd + sngTmp0             'FWD
      sngTmp0 = wrdADCVal(2)
      sngADCRev = sngADCRev + sngTmp0             'Rev
      bitADCdone = 0
   Next bytTmp0
   ADCSRA.ADSC = 0                                'Stop ADC
   sngTmp2 = bytRep
   sngADCFwd = sngADCFwd / sngTmp2
   sngFwddBm = sngADCFwd * sngFwdGrd
   sngFwddBm = sngFwddBm + sngFwdOrd              'FWD power (dBm)
   sngADCRev = sngADCRev / sngTmp2
   sngRevdBm = sngADCRev * sngRevGrd
   sngRevdBm = sngRevdBm + sngRevOrd              'REV power (dBm)
   ReturnLoss = sngFwddBm - sngRevdBm             'Return loss (dB)

End Function

'===================================================================== GetPower
Sub GetPower
'Get TX power from ADC forward channel
'Output: sngFwdPwr      TX power(W)
'        bytPWRerr      Error flag
'                       = 0: no error, power within detction limits
'                       = 1: power below lower detection limit
'                       = 2: power above upper detection limit
'                       = 3: ADC overflow
'Calls:                 CalcFwdPower

   bytADChannel = 0                               'First ADC channel ADC0 (FWD)
   bitADCdone = 0
   sngADCFwd = 0
   ADCSRA.ADSC = 1                                'Start ADC
   BitWait bitADCdone , Set                       'ADC accumulation complete                    '
   bitADCdone = 0
   wrdADCVal(1) = 0
   For bytTmp0 = 1 To 5                           'Measure FWD 5 times
      BitWait bitADCdone , Set                    'ADC accumulation complete
      sngTmp0 = wrdADCVal(1)
      sngADCFwd = sngADCFwd + sngTmp0             'ADC sum forward
      If wrdADCVal(1) > 1022 Then bytADCOvl = 1   'ADC overflow
      bitADCdone = 0
   Next bytTmp0
   ADCSRA.ADSC = 0                                'Stop ADC
   sngADCFwd = 0.2 * sngADCFwd                    'ADC FWD mean value
   Call CalcFwdPower                              'Forward power(W)
   If bytPWRerr = 1 Then                          'power < lower ADC detection limit
      sngFwdPwr = 0                               'Set TX power = 0 W
   End If
   sngADCFwd = 0

End Sub

'================================================================== CheckTXMute
Sub CheckTXMute
'Check TX mute function and evaluate remaining TX power if muted
'Input:  sngPWRmax      Max TX power allowed for tuning (W)
'        bytSWLopt      =1: SWL option, no TX is available
'Output: sngPWRmuted    Muted TX power (TX PTT and/or TX mute activated)
'        bytPwr_high    =0: is OK for ATU switching (relay protection)
'                       =1: TX mute is not checked
'                       =2: muted TX power > max tuning power
'                       =3: muted TX power below or above detection limit
'

   bitTXmute = On                                 'TX-Mute voltage on
   bitTXPTT = On                                  'TX-PTT relay on
   If bytTXDelay > 9 Then
      Call TXMuteDelay                            'Wait for TX to power down
   End If
   bytADCOvl = 0
   Call GetPower                                  'FWD power(W)

   Select Case bytPWRerr
      Call ClearLCDLine(2 , bytChar)
      Case 0                                      'TX is on
         If bytSWLopt = OFF Then                  'TRX option
            If sngFwdPwr < sngPWRmax Then         'Check max power for ATU switching
               bytPwr_high = 0                    'Is OK, flag for ATU switching
            Else
               bytPwr_high = 2                    'Is too high, ATU switching disabled
            End If
            strDis = Fusing(sngFwdPwr , "#.#")
            strDis = "TX Mute " + strDis + "W"
            If bytPwr_high = 0 Then
               strDis = strDis + " is OK"
            Else
               strDis = strDis + " notOK"
            End If
            Lcd strDis
         Else                                     'SWL option
            LCD "SWL option.TX is on!"
            bytPwr_high = 2                       'ATU switching disabled
         End If
      Case Else                                   'Power error
         If bytSWLopt = OFF Then                  'TRX option
            Select Case bytPWRerr
               Case 1
                  LCD "Power below limit"
               Case else
                  LCD "Power above limit"
            End Select
            bytPwr_high = 3                       'ATU switching disabled
         Else                                     'SWL option
            LCD "SWL option active   "
            Call ClearLCDLine(4 , bytChar)
            LCD "Remote unit enabled"
            bytPwr_high = 0                       'ATU switching enabled
         End If
   End Select
   bitTXmute = Off                                'TX-Mute voltage off
   bitTXPTT = Off                                 'TX-PTT relay off
   Wait 1
   sngADCFwd = 0
   bitADCdone = 0
   Call ClearLCDLine(2 , bytChar)
   Call ClearLCDLine(4 , bytChar)

End Sub

'======================================================================= IsTXOn
Sub IsTXOn
'Check if TX is on from FWD ADC channel, TX power > 0.1W
'Input:  wrdADCVal(1)   Counting result of ADC0 (FWD channel)
'        sngFwdGrd      Gradient of AD8307 calibration line
'        sngFwdOrd      Ordinate of AD8307 calibration line
'        sngCoupAtt     Total coupler attenuation (coupling factor+Attenuator)
'Output: bitTXOn        =Off: TX is off, =On: TX is on

   sngADCFwd = 0
   bytADChannel = 0                               'First ADC channel ADC0 (FWD)
   bitADCdone = 0
   ADCSRA.ADSC = 1                                'Start ADC
   For bytTmp0 = 1 To 3                           'Measure FWD 3 times
      BitWait bitADCdone , Set                    'ADC accumulation complete
      sngTmp0 = wrdADCVal(1)
      sngADCFwd = sngADCFwd + sngTmp0             'FWD
      bitADCdone = 0
   Next bytTmp0
   ADCSRA.ADSC = 0                                'Stop ADC
   sngADCFwd = 0.33333 * sngADCFwd                'ADC FWD mean value
   sngFwddBm = sngADCFwd * sngFwdGrd
   sngFwddBm = sngFwddBm + sngFwdOrd              'FWD AD8307 measured power (dBm)
   sngFwddBm = sngFwddBm + sngCoupAtt             'Forward TX power (dBm)
   If sngFwddBm > 20 Then                         'TX power > 20dBm (0.1W)
      bitTXOn = On
   Else
      bitTXOn = Off
   End If
   sngADCFwd = 0
   bitADCdone = 0

End Sub

'================================================================= CalcFwdPower
Sub CalcFwdPower
'Calculate forward avg. power (W) for TX mute function
'Input:  sngADCFwd   Mean ADC value for avg. power
'        sngFwdGrd   2 point calibration line gradient forward
'        sngFwdOrd   2 point calibration line ordinate forward
'        sngCoupAtt  Total coupler attenuation (coupling factor+Attenuator)
'Output: sngFwdPwr   Calculated forward power in W
'        bytPWRerr   Error flag
'                    = 0: no error, power within detction limits
'                    = 1: power below lower detection limit
'                    = 2: power above upper detection limit
'                    = 3: ADC overflow

   'Calculate forward power from 2 point calibration line
   'Add the total coupler attenuation to get the real TX power
   bytPWRerr = 0
   sngFwddBm = sngFwdGrd * sngADCFwd
   sngFwddBm = sngFwddBm + sngFwdOrd              'Coupled forward power (dBm)
   If bytADCOvl = 1 Then                          'ADC overflow FWD
      bytPWRerr = 3
   Else
      Select Case sngFwddBm
         Case Is < sngMindBm                      'Below lower detection limit
            bytPWRerr = 1
         Case Is > sngMaxdBm                      'Above upper detection limit
            bytPWRerr = 2
         Case Else
            sngTmp1 = sngFwddBm + sngCoupAtt      'Forward TX power (dBm)
            sngFwdPwr = dBm2Watt(sngTmp1)         'Forward TX power (W)
      End Select
   End If

End Sub

'================================================================= CalcSWRPower
Sub CalcSWRPower
'Calculate forward & reverse RF Power and SWR
'Input:  wrdADCPEP      ADC max peak for PEP calculation
'        sngADCFwd      ADC mean forward for SWR or avg. power
'        sngADCRev      ADC mean reverse for SWR
'        sngFwdGrd      2 point calibration line gradient forward
'        sngFwdOrd      2 point calibration line ordinate forward
'        sngRevGrd      2 point calibration line gradient reverse
'        sngRevOrd      2 point calibration line ordinate reverse
'        bitShowAvg     =0: peak power (PEP), =1: average power
'Output: sngFwdPwr      Forward power(W)
'        sngRevPwr      Reverse power(W)
'        sngSWR         SWR
'        bytPWRerr      Error flag
'                       = 0: no error
'                       = 1: forward power below lower detection limit
'                       = 2: forward power above upper detection limit
'                       = 3: forward power ADC overflow
'                       = 4: reverse power below lower detection limit
'                       = 5: reverse power above upper detection limit
'                       = 6: reverse power ADC overflow
'                       = 7: reverse > forward
'                       = 8: No coupler signal, TX is off
'                       =18: No checks performed except overflow

   If bitShowAvg = 1 Then                         'avg. power
      sngTmp0 = sngADCFwd
   Else                                           'PEP
      sngTmp0 = wrdPEP
   End If
   bytPWRerr = 18

   'Forward power -------------------------------------------------------------
   'Calculate forward power from 2 point calibration line
   'Add the total coupler attenuation to get the real TX power
   sngFwddBm = sngFwdGrd * sngTmp0
   sngFwddBm = sngFwddBm + sngFwdOrd              'Coupled forward power (dBm)


   If bytADCOvl = 1 Then                          'ADC overflow FWD
      bytPWRerr = 3
   Else
      If bitTXOn = Off Then                       'TX is off
         bytPWRerr = 8
         sngSWR = 0
         Exit Sub
      Else
         Select Case sngFwddBm
            Case Is < sngMindBm                   'Below lower detection limit
               bytPWRerr = 1
            Case Is > sngMaxdBm                   'Above upper detection limit
               bytPWRerr = 2
            Case Else
               sngTmp1 = sngFwddBm + sngCoupAtt   'Forward TX power (dBm)
               sngFwdPwr = dBm2Watt(sngTmp1)      'Forward TX power (W)
               bytPWRerr = bytPWRerr - 9          '=9, next check for reverse
         End Select
      End If
   End If

   'Reverse Power -------------------------------------------------------------
   'Calculate reverse power from 2 point calibration line
   'Add the total coupler attenuation to get the real reflected power
   sngRevdBm = sngRevGrd * sngADCRev
   sngRevdBm = sngRevdBm + sngRevOrd              'Reverse power (dBm)

   If bytADCOvl = 2 Then                          'ADC overflow REV
      bytPWRerr = 6
   Else
      Select case sngRevdBm
         Case Is >= sngFwddBm                     'Reverse >= Forward
            bytPWRerr = 7
         'Case Is < sngMindBm                      'Below lower detection limit
         '   bytPWRerr = 4
         Case Is > sngMaxdBm                      'Above upper detection limit
            bytPWRerr = 5
         Case Else
            sngTmp1 = sngRevdBm + sngCoupAtt      'Reverse TX power (dBm)
            sngRevPwr = dBm2Watt(sngTmp1)         'Reverse TX power (W)
            bytPWRerr = bytPWRerr - 9             '=0 if FWD power was OK
      End Select
   End If

   'Calculate SWR from return loss if no error --------------------------------
   If bytPWRerr = 0 Then
      sngTmp0 = sngFwddBm - sngRevdBm             'Return loss
      sngTmp1 = sngTmp0 / 20
      sngTmp1 = -sngTmp1
      sngTmp2 = 10
      sngTmp1 = Power(sngTmp2 , sngTmp1)          'Reflexion coeff. Gamma
      sngTmp0 = 1 + sngTmp1
      sngTmp1 = 1 - sngTmp1
      If sngTmp1 < 1 Then
         sngSWR = sngTmp0 / sngTmp1               'SWR
      Else
         sngSWR = 100                             'SWR undefinded
      End If
   End If

 End Sub

'================================================================= ShowSWRPower
Sub ShowSWRPower(byVal bytRowS As Byte , byVal bytRowP As Byte)
'Calculate an show forward & reverse RF Power
'Input:  bytRowP        LCD row power
'        bytRowS        LCD row SWR
'        sngFwddBm      Forward power (dBm)
'        sngRevPwr      Reverse power (dBm)
'        sngFwdPwr      Forward power (W)
'        sngRevPwr      Reverse power (W)
'        sngMindBm      Lower detection limit (dBm)
'        sngMaxdBm      Upper detection limit (dBm)
'        sngSWR         SWR
'        sngSWRBarMax   Max SWR for Bar
'        sngSWRBarCal   Bar calibration

   If bitShowAvg = 1 Then                         'avg. power
      strDis = "FWA "
      Waitms 100                                  'Slow down flickering
   Else                                           'PEP
      strDis = "FWP "
   End If

   'Forward power -------------------------------------------------------------
   If bytADCOvl = 1 Then                          'ADC overflow FWD
      strDis = strDis + "Ovfl "
   Else
      Select Case sngFwddBm
         Case Is < sngMindBm                      'Below lower detection limit
            strDis = strDis + "<<<<<"
         Case Is > sngMaxdBm                      'Above upper detection limit
            strDis = strDis + ">>>>>"
         Case Else
            wrdTmp0 = sngFwdPwr * 10              'Forward TX power, show 1 decimal
            strTmp10 = Str(wrdTmp0)
            strTmp10 = Format(strTmp10 , "  0.0")
            strDis = strDis + strTmp10
      End Select
   End If

   'Reverse Power -------------------------------------------------------------
   strDis = strDis + " REV "
   If bytADCOvl = 2 Then                          'ADC overflow REV
      strDis = strDis + "Ovfl "
   Else
      Select case sngRevdBm
         Case Is >= sngFwddBm                     'Reverse >= Forward
            strDis = "**** REV >= FWD ****"
         Case Is < sngMindBm                      'Below lower detection limit
            strDis = strDis + "<<<<<"
         Case Is > sngMaxdBm                      'Above upper detection limit
            strDis = strDis + ">>>>>"
         Case Else
            wrdTmp0 = sngRevPwr * 100             'Reverse TX power, show 2 decimals
            strTmp10 = Str(wrdTmp0)
            strTmp10 = Format(strTmp10 , " 0.00")
            strDis = strDis + strTmp10 + "W"
      End Select
   End If
   Locate bytRowP , 1
   LCD strDis                                     'Show Power

   'SWR -----------------------------------------------------------------------
   If bytPWRerr > 0 Then
      Call ClearLCDLine(bytRowS , bytChar)
      Exit Sub
   End If
   If sngSWR > 9.99 Then
      sngSWR = 9.99                               'SWR display limit
   End If

   If sngSWR > sngSWRBarMax Then                  'Suppress bargraph for SWR = 1
      sngTmp0 = sngSWRBarMax - 1                  'Display limit
   Else
      sngTmp0 = sngSWR - 1                        'Current SWR
   End If

   sngTmp0 = sngSWRBarCal * sngTmp0               'Bar calibration
   wrdTmp2 = sngTmp0                              'Number of bar dots
   wrdTmp0 = wrdTmp2 / 5                          'Number of 5 dot full blocks
   wrdTmp1 = wrdTmp2 Mod 5                        'Remaining partial single bars

   strDis = ""
   If wrdTmp0 > 0 Then                            'Full 5 dot blocks
      For bytTmp0 = 1 To wrdTmp0
         strDis = strDis + strBar(5)
      Next bytTmp0
   End If
   If wrdTmp1 > 0 Then
      strDis = strDis + strBar(wrdTmp1)           'Last bar(s) < 5 dots
   End If

   wrdTmp1 = bytBarLen - 4                        'Space for Tag 'SWR'?
   If wrdTmp0 < wrdTmp1 Then
      bytTmp0 = 1                                 'yes
   Else
      bytTmp0 = 0                                 'no
   End If

   wrdTmp0 = Len(strDis)                          'string length up to now
   wrdTmp1 = bytChar
   wrdTmp1 = wrdTmp1 - wrdTmp0                    'remaining characters
   wrdTmp1 = wrdTmp1 - 4                          '4 char's for value
   If wrdTmp1 > 0 Then
      strDis = strDis + Space(wrdTmp1)            'Fill with spaces
   End If
   If bytTmp0 = 1 Then                            'Add tag 'SWR'
      wrdTmp0 = Len(strDis)
      wrdTmp1 = wrdTmp0 - 4
      strDis = Left(strDis , wrdTmp1)
      strDis = strDis + "SWR "
   End If

   wrdTmp0 = sngSWR * 100                         'show 2 decimals
   strTmp10 = str(wrdTmp0)
   strTmp10 = Format(strTmp10 , "0.00")
   strDis = strDis + strTmp10                     'Add sngSWR value at the end
   Locate bytRowS , 1
   LCD strDis                                     'Show me

 End Sub

'===================================================================== dBm2Watt
Function dBm2Watt(byVal sngdBm As Single) As Single
'Convert dbm to Watt, W = [10^(dBm/10)]/1000
'Input:  sngdBm     dBm

   sngTmp0 = 10
   sngTmp1 = 0.1 * sngdBm
   dBm2Watt = Power(sngTmp0 , sngTmp1)            'mW
   dBm2Watt = 0.001 * dBm2Watt                    'W

End Function

'******************************************************************************
'Display & key routines
'******************************************************************************

'====================================================================== LCDInit
Sub LCDInit
'Special Init for EA DIP203-4 & EA DIP204-4 LCD

   Call LCDWriteInstruction(&B00101100)           'Function set RE=1
   Call LCDWriteInstruction(&B00001001)           'Extended function set, 4 lines, 5dot
   Call LCDWriteInstruction(&B00101000)           'Function set RE=0
   Call LCDWriteInstruction(&B00000110)           'Entry mode set cursor auto-increment
   Call LCDWriteInstruction(&B00001100)           'Display on, cursor off, blink off

End Sub

'=============================================================== LCDWriteNibble
Sub LCDWriteNibble(byVal bytLCDChar As Byte)
'Write 4 high bits (high nibble) of byte bytLCDChar to 4 bit mode LCD

   LCD_D4 = bytLCDChar.4
   LCD_D5 = bytLCDChar.5
   LCD_D6 = bytLCDChar.6
   LCD_D7 = bytLCDChar.7
   LCD_E = 1                                      'Enable, send high nibble
   Waitms 1                                       'Wait for LCD to execute
   LCD_E = 0

End Sub

'===================================================================== LCDWrite
Sub LCDWriteInstruction(byVal bytLCDChar As Byte)
'Send instruction to 4 bit mode LCD. First high nibble, then low nibble

   Call LCDWriteNibble(bytLCDChar)                'High nibble
   Waitms 1                                       'Wait for LCD to execute
   Shift bytLCDChar , Left , 4                    'Shift low nibble up 4 bits
   Call LCDWriteNibble(bytLCDChar)                'Low nibble is now at high position

End Sub

'================================================================= ClearLCDLine
Sub ClearLCDLine(byVal bytRow As Byte , byVal bytChars As Byte)
'Clear LCD bytChars characters in line bytRow

   Locate bytRow , 1
   LCD SPC(bytChars)
   Locate bytRow , 1

End Sub

'==================================================================== DisplayLC
Sub DisplayLC(byVal bytRow As Byte , byVal bytEncEnab As Byte)
'Set and display L/C & High-/Low pass in row bytRow
'Input:  bytRow        Display row
'        bytEncEnab    =1: Encoder enabled, set L, C & Hi-/Lo pass by encoder
'                      =0: No encoder action, show L, C & Hi-/Lo pass data
'        bytEnc_L      Encoder value L
'        bytEnc_C      Encoder value C
'        bitHL_pass    =HI: High pass, =LO: Low pass
'        bytLC_Active  =1: L active, =2: C active, =3: Hi/Lo pass active
'In lowpass  configuration C's are connected in series at antenna port
'In highpass configuration L's are connected in series at antenna port

   'Prepare output mask
   strDis = Space(20)                             'Initialize display string
   bytDis(1) = "L"                                'Set characters in strDis
   bytDis(8) = "u"
   bytDis(10) = "C"
   bytDis(16) = "p"

   If bitLock = On Then                           'Show "-" = disabled
      bytDis(2) = "-"
      bytDis(11) = "-"
      bytDis(19) = "-"
   Else
      Select Case bytLC_Active
         Case 1                                   'L active
            bytTmp0 = 2
         Case 2                                   'C active
            bytTmp0 = 11
         Case 3                                   'High-/Low pass active
            bytTmp0 = 19
         Case Else
            bytTmp0 = 20
      End Select
      If bytMode = 2 Then                         'Manual mode
         bytDis(bytTmp0) = "*"                    'Mark active position L-C-L/H
      End If
   End If

   If bytEncEnab = 1 Then                         'Encoder active, stop timer2
      Disable TIMER2
   End If

   'Prepare Hi/Lo pass
   If bytEncEnab = 1 And bytLC_Active = 3 Then    'Encoder enabled: Set Hi/Lo pass
      If bitLock = Off Then                       'Encoder not locked
         bitHL_old = bitHL_pass                   'Save Hi/Lo pass before change
         If bytEnc_Incr > 0 Then
            Toggle bitHL_pass
            bitLC_changed = 1                     'L/C or Hi/Lo pass changed
         End If
      End If
   End If
   If bitHL_pass = Hi Then                        'High pass
      bytDis(20) = "H"
   Else                                           'Low pass
      bytDis(20) = "L"
   End If

   'Prepare L-value
   If bytEncEnab = 1 And bytLC_Active = 1 Then    'Encoder enabled: Set L
      If bitLock = Off Then                       'Encoder not locked
         bytEncL_old = bytEnc_L                   'Save L value before change
         If bitL_reset = 1 Then                   'L was manually resetted
            bytEnc_L = 0
            bitLC_changed = 1                     'L/C or Hi/Lo pass changed
            bitL_reset = 0
         End If
         Select Case bytEnc_Incr
            Case 1                                'Decrement
               Decr bytEnc_L
               bitLC_changed = 1                  'L/C or Hi/Lo pass changed
            Case 2                                'Increment
               Incr bytEnc_L
               bitLC_changed = 1                  'L/C or Hi/Lo pass changed
         End Select
      End If
   End If
   dwdTmp0 = CalcLC(bytEnc_L , wrdDelta_L , wrdBias_L, 1)
   strTmp10 = Str(dwdTmp0)
   strTmp10 = Format(strTmp10 , " 0.00")          'L in uH
   bytTmp0 = Len(strTmp10)
   bytTmp0 = bytTmp0 - 1
   strTmp10 = Left(strTmp10 , bytTmp0)            'Cut last digit: xx.x uH
   bytTmp1 = 3
   For bytTmp2 = 1 To bytTmp0                     'Put L-value into strDis
      Incr bytTmp1
      bytDis(bytTmp1) = bytTmp10(bytTmp2)
   Next bytTmp2

   'Prepare C-value
   If bytEncEnab = 1 And bytLC_Active = 2 Then    'Encoder enabled: Set C
      If bitLock = Off Then                       'Encoder not locked
         bytEncC_old = bytEnc_C                   'Save C value before change
         If bitC_reset = 1 Then                   'C was manually resetted
            bytEnc_C = 0
            bitLC_changed = 1                     'L/C or Hi/Lo pass changed
            bitC_reset = 0
         End If
         Select Case bytEnc_Incr
            Case 1                                'Decrement
               Decr bytEnc_C
               bitLC_changed = 1                  'L/C or Hi/Lo pass changed
            Case 2                                'Increment
               Incr bytEnc_C
               bitLC_changed = 1                  'L/C or Hi/Lo pass changed
         End Select
      End If
   End If
   bytEnc_Incr = 0                                'Reset encoder action

   'Prepare display string and show it
   dwdTmp0 = CalcLC(bytEnc_C , wrdDelta_C , wrdBias_C, 0)
   strTmp10 = Str(dwdTmp0)
   strTmp10 = Format(strTmp10 , "  0.00")         'C in pF
   bytTmp0 = Len(strTmp10)
   bytTmp0 = bytTmp0 - 3
   strTmp10 = Left(strTmp10 , bytTmp0)            'Cut last 2 digits: xxx pF
   bytTmp1 = 12
   For bytTmp2 = 1 To bytTmp0                     'Put C-value into strDis
      Incr bytTmp1
      bytDis(bytTmp1) = bytTmp10(bytTmp2)
   Next bytTmp2

   Locate bytRow , 1
   LCD strDis                                     'Show me

   If bytEncEnab = 1 Then                         'Encoder active, restart timer2
      bytEnc_Incr = 0                             'Reset encoder
      Enable TIMER2
   End If

End Sub

'================================================================== DisplayTune
Sub DisplayTune(byVal bytRow As Byte)
'Display L/C & High-/Low pass in row bytRow for auto tune
'Input:  bytRow        Display row
'        bytEnc_L      Encoder value L
'        bytEnc_C      Encoder value C
'        bitHL_pass    =HI: High pass, =LO: Low pass

   dwdTmp0 = CalcLC(bytEnc_L , wrdDelta_L , wrdBias_L, 1)
   dwdTmp0 = dwdTmp0 / 10                         'L in uH
   strTmp10 = Str(dwdTmp0)
   strTmp10 = Format(strTmp10 , "   0.0")
   Locate bytRow , 2
   LCD strTmp10 ; "u"
   dwdTmp0 = CalcLC(bytEnc_C , wrdDelta_C , wrdBias_C , 0)
   dwdTmp0 = dwdTmp0 / 100                        'C in pF
   strTmp10 = Str(dwdTmp0)
   strTmp10 = Format(strTmp10 , "   0")
   Locate bytRow , 12
   LCD strTmp10 ; "p"

End Sub

'======================================================================= CalcLC
Function CalcLC(byVal bytEnc As Byte , byVal wrdDelta As Word , byVal wrdBias As Word , byVal bytIsL As Byte) As Dword
'Calculate ATU L or C value from Encoder code
'Input:  bytEnc       Encoder code (0...255)
'        wrdDelta     Lowest change L or C (uH x 100 or pF x 100)
'        wrdBias      L or C bias (uH x 100 or pF x 100)
'        bytIsL       =1: L is calculated, =0: C is calculated
'        bitHL_pass   Hi: Highpass, Lo: Lowpass
'Output: CalcLC       L or C value (uH x 100 or pF x 100)

   dwdTmp0 = bytEnc
   dwdTmp1 = wrdDelta
   dwdTmp0 = dwdTmp0 * dwdTmp1
   If bytLCseries = 1 Then                        'L/C banks wired in series in Hi/Lopass switch
      If bytIsL = 1 Then                          'L is calculated
         If bitHL_pass = Hi Then                  'Highpass: L's in series
            Shift dwdTmp0 , Left , 1              '2 L's in series = 2 * L
         End If
      Else                                        'C is calculated
         If bitHL_pass = Lo Then                  'Lowpass: C's in series
            Shift dwdTmp0 , Right , 1             '2 C's in series = C / 2
         End If
      End If
   End If
   dwdTmp1 = wrdBias
   CalcLC = dwdTmp0 + dwdTmp1

End Function

'================================================================== DisplayFreq
Sub DisplayFreq(byVal bytRow As Byte)
'Display Counter-/Band frequencies in row bytRow
'Input:  bytRow        Display row
'        bytMode =1:   Automatic   TX frequency and EEPROM band segment
'                =2:   Manual      show TX frequency only
'        wrdTRXfreq    Counter or com transferred frequency (kHz)
'        wrdSeg_freq   Center frequency band segment from EEPROM
'        wrdProm_adr   Start address of band segment in EEPROM
'        wrdBand_Lo(1) First lower band limit (160 m, 1.800 kHz)

   strDis = Space(bytChar)                        'Initialize display string
   bytDis(1) = bytMode_state(bytMode)
   bytDis(2) = ":"

   If wrdTRXfreq > 0 Then                         'Counter or com transferred freq
      strTmp10 = Str(wrdTRXfreq)
      strTmp10 = Format(strTmp10 , " 0.000")
   Else                                           'Frequency not available
      strTmp10 = "TX off"
   End If

   bytTmp0 = Len(strTmp10)
   bytTmp1 = 4
   For bytTmp2 = 1 To bytTmp0                     'Put frequency into strDis
       Incr bytTmp1
       bytDis(bytTmp1) = bytTmp10(bytTmp2)
   Next bytTmp2

   bytDis(12) = "-"
   bytDis(13) = ">"
   If bytSeg_found > 0 Then                       'Band segment available
      wrdSeg_FreqCe = SegCenterFreq()
      strTmp10 = Str(wrdSeg_freqCe)
      strTmp10 = Format(strTmp10 , " 0.000")
   Else
      strTmp10 = "NoBand"
   End If
   bytTmp0 = Len(strTmp10)
   bytTmp1 = 14
   For bytTmp2 = 1 To bytTmp0                     'Put center freq into strDis
       Incr bytTmp1
       bytDis(bytTmp1) = bytTmp10(bytTmp2)
   Next bytTmp2

   Locate bytRow , 1
   LCD strDis                                     'Show me

End Sub

'================================================================== TXMuteDelay
Sub TXMuteDelay
'Wait for bytTXDelay msec for TX mute
'bytTXDelay in msec (10, 20,...200, steps 10msec)

   bytTmp1 = bytTXDelay / 10
   If bytTmp1 = 0 Then bytTmp1 = 1
   For bytTmp0 = 1 To bytTmp1
      Waitms 10
   Next bytTmp0
End Sub

'================================================================== ChristanATU
Sub ChristanATU
'Reduce 8 bit L-Encoder value to 7 bit value for Chrstian ATU (7 coils)

   If bytEnc_L > 127 Then
      bytEnc_L = bytEnc_L - 128
   End If

End Sub

'===================================================================== FindBand
Sub FindBand
'Find band segment in EEPROM to fit TRX frequency
'Input:  wrdTRXfreq     Counter or com transferred TRX frequency (kHz)
'Output: wrdEEPROM_Addr Start address EEPROM center freq.
'        wrdSeg_FreqHi  Upper band segment freq. (kHz)
'        wrdSeg_FreqLo  Lower band segment freq. (kHz)
'        wrdSeg_FreqCe  Band segment center freq. (kHz)
'        wrdSeg_width   Band segment width (kHz)
'        bytSeg_C       Encoder value C for this band segment
'        bytSeg_L       Encoder value L for this band segment
'        bytSeg_HL      High pass(1), Low pass(0) for this band segment
'        bytBand_Cur    Index of current band, =0: no band found
'        bytSeg_found   =0: No band segment found
'                       =1: Frequency fits current band segment
'                       =2: new band segment found
'        If bytSeg_found = 2 (new band found):
'        bitLC_changed =1: L/C or Hi/Lo pass changed
'        bytEnc_L       Encoder value for ATU L
'        bytEnc_C       Encoder value for ATU C
'        bitHL_pass     Hi(1)/Lopass(0)

   bytSeg_found = 0                               'No band segment available

   'Find the band segment, get the Amateur band 1...bytBand_Num first
   bytTmp2 = bytBand_Cur                          'Save current band index
   bytBand_Cur = 0
   wrdEEPROM_Addr = 0

   For bytTmp1 = 1 To bytBand_Num                 '11 amateur bands
      If wrdTRXfreq >= wrdBand_Lo(bytTmp1) And wrdTRXfreq < wrdBand_Hi(bytTmp1) Then
         wrdAddr = wrdBand_Addr(bytTmp1)          'Start address for search in the band
         bytBand_Cur = bytTmp1                    'Current Band index
         Exit For                                 'Got the band
      End If
    Next bytTmp1
    If bytBand_Cur = 0 Then                       'No band found
      'Locate 4 , 1
      'LCD "No band found   "
      'Wait 1
      Exit Sub
    End If

    'Now look for the band segment in the Amateur band
    wrdTmp1 = wrdBand_Lo(bytBand_Cur)
    Do
      wrdTmp2 = wrdTmp1 + bytBand_Width(bytBand_Cur)       'upper segment limit
      If wrdTRXfreq >= wrdTmp1 And wrdTRXfreq < wrdTmp2 Then
         wrdEEPROM_Addr = wrdAddr                 'Start address
         Call ReadBandSegment(wrdAddr , 1 , 7)    'Read band segment from EEPROM
         wrdSeg_FreqCe = SegCenterFreq()
         'Different ATU data in that band segment?
         bytTmp1 = bitHL_pass
         If bytSeg_L <> bytEnc_L OR bytSeg_C <> bytEnc_C OR bytSeg_HL <> bytTmp1 Then
            bitLC_changed = 1                     'L/C or Hi/Lo pass changed
         End If
         bytEnc_L = bytSeg_L                      'Set data from EEPROM
         bytEnc_C = bytSeg_C
         bitHL_pass = bytSeg_HL
         bytSeg_found = 2                         'New band segment
         Exit Sub                                 'Done, segment found
      End If
      wrdAddr = wrdAddr + wrdSeg_Bytes            'Try next band segment
      wrdTmp1 = wrdTmp2
    Loop Until wrdTmp1 >= wrdBand_Hi(bytBand_Cur)

End Sub

'==================================================================== SetValue
Sub SetValue(byVal bytRow As Byte , byVal bytType As Byte , byVal wrdScale As Word , byVal bytComp As Byte)
'Set EEPROM value via encoder and restore it to EEPROM
'EEPROM data are byte, Word or Integer, perhaps scaled with factors 10, 100 or 1000
'e.g. 12.34pF ist stored as 1234 in EEPROM (scale factor 100)
'Input:  bytRow         LCD row to display value
'                       Error message in row bytRow + 1
'        bytType        Type of value
'                       1 = Byte
'                       2 = Word
'                       3 = Integer
'        wrdScale       Scale factor of EEPROM value, 1, 10 or 100
'                       EEPROM value = real value * wrdScale
'                       e.g. Word variable (pFx100) to be shown as pF:
'                       wrdScale=100, 123 (1/100pF) divided by 100, shown as 1.23pF
'                       stored as 1.23 * 100 = 123 (Word)
'        bytComp        Compare input variable to reference variable
'                       =0: no compare
'                       =1: input variable must be <  reference variable
'                       =2: input variable must be >  reference variable
'        bytData        Input variable for bytType = 1 (Byte)
'        wrdData        Input variable for bytType = 2 (Word)
'        intData        Input variable for bytType = 3 (Integer)
'        bytDataRef     Reference variable for bytType = 1 (Byte)
'        wrdDataRef     Reference variable for bytType = 2 (Word)
'        intDataRef     Reference variable for bytType = 3 (Integer)
'        bytDataMin     Input variable min value for bytType = 1 (Byte)
'        wrdDataMin     Input variable min value for bytType = 2 (Word)
'        intDataMin     Input variable min value for bytType = 3 (Integer)
'        bytDataMax     Input variable max value for bytType = 1 (Byte)
'        wrdDataMax     Input variable max value for bytType = 2 (Word)
'        intDataMax     Input variable max value for bytType = 3 (Integer)
'        bytDataInc     Data increase when encoder increments
'        bytDataDec     Data decrease when encoder decrements
'        bytCursorFirst First LCD column to show data
'        wrdAddr        EEPROM address to store value
'Output: bytPass        =0:Input data is OK or no comparison to reference
'                       =1: input variable is >=  reference variable
'                       =2: input variable is <=  reference variable

   Select Case bytType
      Case 1                                      'Input byte variable bytData
         strTmp10 = Str(bytData)
      Case 2                                      'Input Word variable wrdData
         strTmp10 = Str(wrdData)
      Case Else                                   'Input Integer variable intData
         strTmp10 = Str(intData)
   End Select

   Select Case wrdScale
      Case 1
         strTmp10 = Format(strTmp10 , "  0")
      Case 10
         strTmp10 = Format(strTmp10 , " 0.0")
      Case 100
         strTmp10 = Format(strTmp10 , " 0.00")
   End Select
   strTmp10 = strTmp10 + strUnit
   Locate bytRow , bytCursorFirst
   LCD strTmp10

   If bytEnc_Incr > 0 Then                        'Encoder turned
      Select Case bytEnc_Incr
         Case 1                                   'Encoder decremented
            Select Case bytType
               Case 1
                  intTmp0 = bytData - bytDataDec
                  If intTmp0 > bytDataMin Then
                     bytData = intTmp0
                  Else
                     bytData = bytDataMin
                  End If
               Case 2
                  intTmp0 = wrdData - bytDataDec
                  If intTmp0 > wrdDataMin Then
                     wrdData = intTmp0
                  Else
                     wrdData = wrdDataMin
                  End If
               Case Else
                  intData = intData - bytDataDec
                  If intData < intDataMin Then intData = intDataMin
            End Select
         Case 2                                   'Encoder incremented
            Select Case bytType
               Case 1
                  wrdData = bytData + bytDataInc
                  If wrdData < 256 Then
                     bytData = wrdData
                  Else
                     bytData = 255
                  End If
                  If bytData > bytDataMax Then bytData = bytDataMax
               Case 2
                  wrdTmp0 = bytDataInc
                  wrdData = wrdData + wrdTmp0
                  If wrdData > wrdDataMax Then wrdData = wrdDataMax
               Case Else
                  intTmp0 = bytDataInc
                  intData = intData + intTmp0
                  If intData > intDataMax Then intData = intDataMax
            End Select
      End Select
      bytEnc_Incr = 0                             'Reset Encoder
   End If

   'Compare input data to reference data if bytComp > 0
   bytPass = 0                                    'Error flag
   Select case bytComp
      Case 1                                      'input <  reference
         Select Case bytType
            Case 1
               If bytData >= bytDataRef Then bytPass = 1
            Case 2
               If wrdData >= wrdDataRef Then bytPass = 1
            Case 3
               If intData >= intDataRef Then bytPass = 1
         End Select
      Case 2                                      'input >  reference
         Select Case bytType
            Case 1
               If bytData <= bytDataRef Then bytPass = 2
            Case 2
               If wrdData <= wrdDataRef Then bytPass = 2
            Case 3
               If intData <= intDataRef Then bytPass = 2
         End Select                               '
   End Select

   bytRow = bytRow + 1
   If bytPass = 0 Then                            'OK, store data to EEPROM
      Call ClearLCDLine(bytRow , bytChar)
      If bytKey_store = 1 Then                    'Store key pressed
         Select Case bytType
            Case 1                                'Input byte variable bytData
               WriteEEPROM bytData , wrdAddr
            Case 2                                'Input Word variable wrdData
               Call EEPROMWriteWrd(wrdData)
            Case Else                             'Input Integer variable intData
               Call EEPROMWriteInt(intData)
         End Select
         LCD strTmp10
         Locate 3 , 7
         LCD "stored -> "
         strTmp10 = Str(wrdAddr)
         strTmp10 = Format(strTmp10 , "   0")
         Locate bytRow , 17
         LCD strTmp10
         Wait 2
         Incr bytSetup
         bytSetupAct = 0
      End If
   End If

End Sub

'==================================================================== SetupErr
Sub SetupErr(byVal bytRow As Byte , byVal bytErr As Byte)
'Error message for Auto tune setup
'Input:  bytRow      LCD row for message
'        bytErr      Error condition
'                    =1: setup variable is >=  reference variable
'                    =2: setup variable is <=  reference variable
'        bytData     Setup variable
'        bytDataRef  Reference variable

   Select Case bytErr
      Case 1                                      'Start >= End
         strTmp10 = Str(bytData)
         strTmp10 = Format(strTmp10 , "  0")
         Locate bytRow , 1
         LCD "Start " ; strTmp10 ; " <= " ; "End "
         strTmp10 = Str(bytDataRef)
         strTmp10 = Format(strTmp10 , "  0")
         Locate bytRow , 17
         LCD strTmp10
      Case 2                                      'End <= Start
         strTmp10 = Str(bytData)
         strTmp10 = Format(strTmp10 , "  0")
         Locate bytRow , 1
         LCD "End " ; strTmp10 ; " <= " ; "Start "
         strTmp10 = Str(bytDataRef)
         strTmp10 = Format(strTmp10 , "  0")
         Locate bytRow , 17
         LCD strTmp10
   End Select

End Sub


'******************************************************************************
'EEPROM routines
'******************************************************************************
'================================================================== CheckEEPROM
Sub CheckEEPROM
'Check if coupler and band segment data are stored in EEPROM
'if not, write data to EEPROM
'if yes, read data from EEPROM
'In a new chip or erased EEPROM data = 255
'Data structure header data:
'Addr Variable       Bytes
'   1 bytSeg_Bands     1   Nmbr. of band segments
'   2 ---              1   =1: L/C data are stored in band segments
'   3 ---              1   =0: EEPROM initialized with band segments
'   4 bytSWLopt        1   =1: SWL option, =0: TRX option
'   5 byt8Coils        1   =1: 8 tuner coils, =0: 7 coils (Christian tuner)
'   6 bytTXDelay       1   TX mute delay (10 msec)
'   7 wrdDelta_C       2   Lowest change of tuner C (1/100 pF)
'   9 wrdDelta_L       2   Lowest change of tuner L (1/100 uH)
'  11 wrdBias_C        2   Bias capacity    for all C's out (1/100 pF)
'  13 wrdBias_L        2   Bias inductivity for all L's out (1/100 uH)
'  15 bytSWRAutoMin    1   Min SWR for auto tuning (SWR x 100)
'  16 bytPWRmax        1   Max HF power for ATU tuning (W)
'  17 bytADCRep        1   Repetitive ADC runs for mean values
'  18 bytCoupCal       1   =0: coupler data initialized, =1: calibrated
'  19 wrdCoupFac       2   Coupling factor (dB x 10)
'  21 wrdAmpAtt        2   Log amp attenuator (dB x 10)
'  23 intCaldBmLo      2   Low  log amp calibration point (dBm x 10)
'  25 intCaldBmHi      2   High log amp calibration point (dBm x 10)
'  27 wrdCalFwdADCLo   2   Forward ADC counts at low  calibration point
'  29 wrdCalRevADCLo   2   Reverse ADC counts at low  calibration point
'  31 wrdCalFwdADCHi   2   Forward ADC counts at high calibration point
'  33 wrdCalRevADCHi   2   Reverse ADC counts at high calibration point
'  35 intMindBm        2   Lower detection limit (dBm x 10)
'  37 intMaxdBm        2   Upper detection limit (dBm x 10)
'  39 bytPEPhold       1   PEP peak sample & hold time (sec)
'  40 bytUseTRXFreq    1   Use counter measured freq. or TRX-COM freq.
'  41 bytLNonInv       1   =1: L bank byte not inverted, =0: inverted
'  42 bytLCseries      1   =1: L/C banks in series, =0: Only 1 bank in Hi/Lopass switch

'Data structure auto tune settings
'Start address        50
'Addr Variable       Bytes
'  50 bytStepC         1   Maximal step size C variation 160m
'  51 bytStepC         1   Maximal step size C variation 80m
'  ...                     60, 40, 30, 20, 17, 15, 12, 10m
'  60 bytStepC         1   Maximal step size L variation 6m

'  61 bytStepL         1   Maximal step size L variation 160m
'  62 bytStepL         1   Maximal step size C variation 80m
'  ...                     60, 40, 30, 20, 17, 15, 12, 10m
'  71 bytStepL         1   Maximal step size L variation 6m

'  72 bytPassBand      1   Start auto tune with lowpass(0)/highpass(1) 160m
'  73 bytPassBand      1   Start auto tune with lowpass(0)/highpass(1) 80m
'  ...                     60, 40, 30, 20, 17, 15, 12, 10m
'  82 bytPassBand      1   Start auto tune with lowpass(0)/highpass(1) 6m

'  83 bytStartC        1   Start auto tune with C value (0...255) 160m
'  84 bytStartC        1   Start auto tune with C value (0...255) 80m
'  ...                     60, 40, 30, 20, 17, 15, 12, 10m
'  93 bytStartC        1   Start auto tune with C value (0...255) 6m

'  94 bytEndC          1   End auto tune with C value (0...255) 160m
'  95 bytEndC          1   End auto tune with C value (0...255) 80m
'  ...                     60, 40, 30, 20, 17, 15, 12, 10m
' 104 bytEndC          1   End auto tune with C value (0...255) 6m

' 105 bytStartL        1   Start auto tune with L value (0...255) 160m
' 106 bytStartL        1   Start auto tune with L value (0...255) 80m
'  ...                     60, 40, 30, 20, 17, 15, 12, 10m
' 116 bytStartL        1   Start auto tune with L value (0...255) 6m

' 117 bytEndL          1   End auto tune with L value (0...255) 160m
' 118 bytEndL          1   End auto tune with L value (0...255) 80m
'  ...                     60, 40, 30, 20, 17, 15, 12, 10m
' 126 bytEndL          1   End auto tune with L value (0...255) 6m

'Data structure band segment data, 7 bytes per band segment:
'     Variable       Bytes
'     wrdSeg_FreqLo    2   Segment low  limit (kHz)
'     wrdSeg_FreqHi    2   Segment high limit (kHz)
'     bytSeg_L         1   Value of tuner L for this segment (0...255) or (0...127)
'     bytSeg_C         1   Value of tuner C for this segment (0...255)
'     bytSeg_HL        1   =0: low pass, =1: high pass
'     bytSeg_Arr(7)    7   Byte array overlayed, starting at wrdSeg_FreqLo
'Start address       150
'Band segment data are defined by band data
'- wrdBand_Lo(i)           Lower band limit (kHz)
'- wrdBand_Hi(i)           Upper band limit (kHz)
'- bytBand_Width           Width of band segments (kHz)

'Output: bytInit_LC = 1:   L/C data are stored
'        bytSeg_Bands:     Number of band segments
'        wrdBand_Adr(i):   Band addresses, 1 = 1...11

   'Check 3 first Addresses ---------------------------------------------------
   'Address 1         Number of band segments
   'Address 2         =1 if L/C data are stored
   'Address 3         =0 if EEPROM initialized with band segments
   '                  New chip or erased EEPROM: all Data = 255

   ReadEEPROM bytTmp10(1) , 1                     'Addr. 1: Number of band segments
   ReadEEPROM bytTmp10(2) , 2                     'Addr. 2: L/C segment data
   ReadEEPROM bytTmp10(3) , 3                     'Addr. 3: EEPROM initialized
   bytTmp1 = 0
   For bytTmp0 = 1 To 3
      If bytTmp10(bytTmp0) = 255 Then
         Incr bytTmp1
      End If
   Next bytTmp0
   bytInit_EEP = 0
   If bytTmp1 > 1 Then                            'Is a brandnew controller
      bytInit_EEP = 1                             'Initialize EEPROM
   Else                                           'Is a used controller
      If bytTmp10(1) = 255 Then                   'Nmbr. of segments not set
         bytInit_EEP = 1                          'Initialize EEPROM
      End If
   End If
   If bytTmp10(2) = 1 Then                        'L/C data are stored
      bytInit_LC = 1                              'Do not overwrite L/C data
   Else
      bytInit_LC = 0                              'Initialize L/C data
   End If

   'Initialize EEPROM *********************************************************
   If bytInit_EEP = 1 Then
      Call ClearLCDLine(1 , bytChar)
      If bytInit_LC = 0 Then                      'All addresses to be initialized
         LCD "Init EEPROM-total"
      Else                                        'Only band segments to be initialized
         LCD "Init EEPROM-partial"
      End If
      Call ClearLCDLine(2 , bytChar)
      LCD "Please wait ..."
      bytSeg_Bands = 0                            'Number of written band segments

      wrdAddr = 18
      ReadEEPROM bytCoupCal , wrdAddr             'Coupler calibration, =1: calibrated

      'Store EEPROM Header data, addresses 4, 5, ... 39 -----------------------
      wrdAddr = 4
      WriteEEPROM bytSWLopt , wrdAddr             'SWL option
      Incr wrdAddr
      WriteEEPROM byt8Coils , wrdAddr             '7 or 8 coils version
      Incr wrdAddr
      WriteEEPROM bytTXDelay , wrdAddr            'TX mute delay
      Incr wrdAddr
      Call EEPROMWriteWrd(wrdDelta_C)             'Lowest Change of C (1/100 pF)
      Call EEPROMWriteWrd(wrdDelta_L)             'Lowest Change of L (1/100 uH)
      Call EEPROMWriteWrd(wrdBias_C)              'Bias capacity for all C's out (1/100 pF)
      Call EEPROMWriteWrd(wrdBias_L)              'Bias inductivity for all L's out (1/100 uH)
      WriteEEPROM bytSWRAutoMin , wrdAddr         'Min SWR for auto tuning (100*SWR)
      Incr wrdAddr
      WriteEEPROM bytPWRmax , wrdAddr             'Max. power to be switched in ATU (W)
      Incr wrdAddr
      WriteEEPROM bytADCRep , wrdAddr             'ADC mean repetition
      Incr wrdAddr
      If bytCoupCal <> 1 Then                     'Coupler not yet calibrated
         bytTmp2 = 0
         WriteEEPROM bytTmp2 , wrdAddr            '=0: Coupler data initialized
      End If
      Incr wrdAddr
      Call EEPROMWriteWrd(wrdCoupFac)             'Coupling factor (dB x 10)
      Call EEPROMWriteWrd(wrdAmpAtt)              'Log amp attenuators (dB x 10)
      Call EEPROMWriteInt(intCaldBmLo)            'Low  log amp calibration point (dBm x 10)
      Call EEPROMWriteInt(intCaldBmHi)            'High log amp calibration point (dBm x 10)

      'Check if calibration data are already stored. If yes they are preserved.
      If bytCoupCal = 1 Then                      'Coupler is calibrated
         wrdAddr = wrdCal_FirstAddr + 8           'Skip addresses
      Else                                        'Coupler not yet calibrated
         wrdCalFwdADCLo = 0
         Call EEPROMWriteWrd(wrdCalFwdADCLo)      'Forward ADC counts @ lo cal point
         wrdCalRevADCLo = 0
         Call EEPROMWriteWrd(wrdCalRevADCLo)      'Reverse ADC counts @ lo cal point
         wrdCalFwdADCHi = 0
         Call EEPROMWriteWrd(wrdCalFwdADCHi)      'Forward ADC counts @ hi cal point
         wrdCalRevADCHi = 0
         Call EEPROMWriteWrd(wrdCalRevADCHi)      'Reverse ADC counts @ hi cal point
      End If

      Call EEPROMWriteInt(intMindBm)              'Lower detection limit (dBm x 10)
      Call EEPROMWriteInt(intMaxdBm)              'Upper detection limit (dBm x 10)
      WriteEEPROM bytPEPhold , wrdAddr            'PEP peak sample & hold time (sec x 10)
      Incr wrdAddr
      WriteEEPROM bytUseTRXFreq , wrdAddr         'Use TRX-COM freq in RX & TX mode
      Incr wrdAddr
      WriteEEPROM bytLNonInv , wrdAddr            'L bank inverted/not inverted
      Incr wrdAddr
      WriteEEPROM bytLCseries , wrdAddr           'L/C banks in series/not in series

      'Store auto tuning settings, addresses 50 ... 126 -----------------------
      bytStepC = 16                               'Step size C variation
      bytStepL = 16                               'Step size L variation
      bytPassBand = 0                             'Lowpass
      bytStartC = 0                               'Start value C
      bytStartL = 0                               'Start value L
      bytEndC = 255                               'End value C
      If byt8Coils = 1 Then
         bytEndL = 255                            'End value L
      Else
         bytEndL = 127                            'End value L (Christian tuner)
      End If
      wrdAddr = wrdTune_FirstAddr                 'First address auto tune settings
      For bytTmp0 = 1 To bytBand_Num
         WriteEEPROM bytStepC , wrdAddr
         Incr wrdAddr
      Next bytTmp0
      For bytTmp0 = 1 To bytBand_Num
         WriteEEPROM bytStepL , wrdAddr
         Incr wrdAddr
      Next bytTmp0
      For bytTmp0 = 1 To bytBand_Num
         WriteEEPROM bytPassBand , wrdAddr
         Incr wrdAddr
      Next bytTmp0
      For bytTmp0 = 1 To bytBand_Num
         WriteEEPROM bytStartC , wrdAddr
         Incr wrdAddr
      Next bytTmp0
      For bytTmp0 = 1 To bytBand_Num
         WriteEEPROM bytEndC , wrdAddr
         Incr wrdAddr
      Next bytTmp0
      For bytTmp0 = 1 To bytBand_Num
         WriteEEPROM bytStartL , wrdAddr
         Incr wrdAddr
      Next bytTmp0
      For bytTmp0 = 1 To bytBand_Num
         WriteEEPROM bytEndL , wrdAddr
         Incr wrdAddr
      Next bytTmp0

      'Store band segments, addresses 150 ... 1151 ----------------------------
      wrdAddr = wrdSeg_FirstAddr                  'First segment address
      'Loop the amateur bands
      For bytBand_Cur = 1 To bytBand_Num
         wrdBand_FreqLo = wrdBand_Lo(bytBand_Cur) 'Lower band limit
         wrdSeg_width = bytBand_Width(bytBand_Cur)       'Segment width
         wrdBand_FreqHi = wrdBand_Hi(bytBand_Cur) 'Upper band limit
         wrdSeg = wrdBand_FreqLo
         'Loop the segments in the current band
         Do
            wrdSeg_FreqLo = wrdSeg                'Lower segment limit
            wrdSeg_FreqHi = wrdSeg + wrdSeg_width 'Upper segment limit
           'Write segment lo & hi freq. to EEPROM, Bytes 1 to 4
            For bytTmp1 = 1 To 4                  'Write band segment
               WriteEEPROM bytSeg_Arr(bytTmp1) , wrdAddr
               Incr wrdAddr
            Next bytTmp1
           'Initialize encoder L, C and Hi/lo pass to EEPROM, bytes 5 to 7
            If bytInit_LC = 0 Then                'Init L/C and Hi/lo pass
               bytSeg_L = 0                       'Encoder value L (=0)
               bytSeg_C = 0                       'Encoder value C (=0)
               bytSeg_HL = 0                      'Low pass (=0)
               For bytTmp1 = 5 To 7
                  WriteEEPROM bytSeg_Arr(bytTmp1) , wrdAddr
                  Incr wrdAddr
               Next bytTmp1
            Else                                  'L/C data preserved
               wrdAddr = wrdAddr + 3              'Skip addresses 5...7
            End If
            wrdSeg = wrdSeg + wrdSeg_width        'Next segment
            Incr bytSeg_Bands                     'Segment counter
         Loop Until wrdSeg >= wrdBand_FreqHi
      Next bytBand_Cur

      bytTmp2 = 0
      WriteEEPROM bytTmp2 , 1                     'write 0 to address 1
      If bytInit_LC = 0 Then                      'No L/C data stored before
         WriteEEPROM bytTmp2 , 2                  'write 0 to address 2
      End If
      WriteEEPROM bytTmp2 , 3                     'write 0 to address 3

      'Write number of band segments to Address 1
      WriteEEPROM bytSeg_Bands , 1
      bytTmp2 = 0
      If bytInit_LC = 0 Then                      'No L/C data stored before
         WriteEEPROM bytTmp2 , 2                  'write 0 to address 2
      End If
      WriteEEPROM bytTmp2 , 3                     'write 0 to address 3
      Call ClearLCDLine(2 , bytChar)
      Lcd "... Write done"
      Wait 1
   End If

   'Read EEPROM ***************************************************************
   ReadEEPROM bytSeg_Bands , 1                    'number of band segments
   'Set segment start addresses per band in wrdBand_Adr(1...11)
   wrdTmp0 = wrdSeg_FirstAddr                     'First segment address
   For bytTmp3 = 1 To bytBand_Num                 'Loop the bands
      wrdTmp1 = wrdBand_Lo(bytTmp3)               'Lower band limit
      wrdSeg_width = bytBand_Width(bytTmp3)       'Segment width
      bytInit_EEP = 0
      Do
         wrdTmp2 = wrdTmp1 + wrdSeg_width         'upper Segment limit
         If bytInit_EEP = 0 Then
            wrdBand_Addr(bytTmp3) = wrdTmp0
            bytInit_EEP = 1                       'Address(bytTmp3) stored
         End If
         wrdTmp0 = wrdTmp0 + wrdSeg_Bytes         'next Band segment, next EEPROM block
         wrdTmp1 = wrdTmp2
      Loop Until wrdTmp1 >= wrdBand_Hi(bytTmp3)
   Next bytTmp3

   'Check if tuner data are stored in band segments
   For bytTmp3 = 1 To bytBand_Num                 'Loop the bands
      wrdTmp1 = wrdBand_Lo(bytTmp3)               'Lower band limit (kHz)
      bytBand_Data(bytTmp3) = 0                   'Band has no LC data
      wrdAddr = wrdBand_Addr(bytTmp3)             'Segment start address of band
      wrdSeg_width = bytBand_Width(bytTmp3)       'Segment width
      Do                                          'Loop the band segments
         Call ReadBandSegment(wrdAddr , 1 , 7)
        If bytSeg_L > 0 Or bytSeg_C > 0 Then      'L or C values are set
            bytBand_Data(bytTmp3) = 1             'Band has LC data
            Exit Do
         End If
         wrdAddr = wrdAddr + wrdSeg_Bytes         'next Band segment, next EEPROM block
         wrdTmp1 = wrdTmp1 + wrdSeg_width         'upper Segment limit                       '
      Loop Until wrdTmp1 >= wrdBand_Hi(bytTmp3)
   Next bytTmp3

   'Read header data from EEPROM
   wrdAddr = 4
   ReadEEPROM bytSWLopt , wrdAddr                 'SWL option
   Incr wrdAddr
   ReadEEPROM byt8Coils , wrdAddr                 '7 or 8 coils version
   Incr wrdAddr
   ReadEEPROM bytTXDelay , wrdAddr                'TX mute delay
   Incr wrdAddr
   Call EEPROMReadWrd(wrdDelta_C)                 'Lowest Change of C (1/100 pF)
   Call EEPROMReadWrd(wrdDelta_L)                 'Lowest Change of L (1/100 uH)
   Call EEPROMReadWrd(wrdBias_C)                  'Bias capacity for all C's out (1/100 pF)
   Call EEPROMReadWrd(wrdBias_L)                  'Bias inductivity for all L's out (1/100 uH)
   ReadEEPROM bytSWRAutoMin , wrdAddr             'Min SWR for auto tuning (10*SWR)
   Incr wrdAddr
   ReadEEPROM bytPWRmax , wrdAddr                 'Max. power to be switched in ATU
   sngPWRmax = bytPWRmax
   Incr wrdAddr
   ReadEEPROM bytADCRep , wrdAddr                 'ADC mean repetition
   If bytADCRep = 0 Then bytADCRep = 1
   Incr wrdAddr
   ReadEEPROM bytCoupCal , wrdAddr                'Coupler data flag, =1: calibrated
   Incr wrdAddr
   Call EEPROMReadWrd(wrdCoupFac)                 'Coupling factor (dB x 10)
   Call EEPROMReadWrd(wrdAmpAtt)                  'Log amp attenuators (dB x 10)
   Call EEPROMReadInt(intCaldBmLo)                'Low  log amp calibration point (dBm x 10)
   Call EEPROMReadInt(intCaldBmHi)                'High log amp calibration point (dBm x 10)
   Call EEPROMReadWrd(wrdCalFwdADCLo)             'Forward ADC counts @ lo cal point
   Call EEPROMReadWrd(wrdCalRevADCLo)             'Reverse ADC counts @ lo cal point
   Call EEPROMReadWrd(wrdCalFwdADCHi)             'Forward ADC counts @ hi cal point
   Call EEPROMReadWrd(wrdCalRevADCHi)             'Reverse ADC counts @ hi cal point
   Call EEPROMReadInt(intMindBm)                  'Lower detection limit (dBm x 10)
   Call EEPROMReadInt(intMaxdBm)                  'Upper detection limit (dBm x 10)
   ReadEEPROM bytPEPhold , wrdAddr                'PEP peak sample & hold time (sec)
   Incr wrdAddr
   ReadEEPROM bytUseTRXFreq , wrdAddr             'Use TRX-COM freq in RX & TX mode
   Incr wrdAddr
   ReadEEPROM bytLNonInv , wrdAddr                'L bank inverted/not inverted
   Incr wrdAddr
   ReadEEPROM bytLCseries , wrdAddr               'L/C banks in series/not in series

   Call ClearLCDLine(1 , bytChar)
   Lcd "EEPROM has data "
   Call ClearLCDLine(2 , bytChar)
   Lcd bytSeg_Bands ; " Band Segments"
   Wait 1

End Sub

'================================================================== StoreLCData
Sub StoreLCData
'Store current L/C data to EEPROM
'If no LC data are stored (bytBand_Data(i)=0)in the band i
'the current LC data are written to all band segments of the band
'else LC data are written to the current segment
'Input:  wrdEEPROM_addr EEPROM band segment start address set in Sub Findband
'        bytBand_cur    Current band index (1...11) set in Sub Findband
'        bytEnc_L       ATU L value
'        bytEnc_C       ATU C value
'        bitHL_pass     hi/lopass

   If wrdEEPROM_addr > 0 And bytBand_cur > 0 Then 'No EEPROM addr
      bytSeg_L = bytEnc_L
      bytSeg_C = bytEnc_C
      bytSeg_HL = bitHL_pass
      'Write Encoder values L, C & High-/Low pass (#5...7) to EEPROM
      wrdAddr = wrdEEPROM_Addr                    'Segment start address
      If bytBand_Data(bytBand_cur) = 1 Then       'LC data stored already
         Call WriteBandSegment(5 , 7 , 0 , 1)     'Only this band segment
      Else
         Call WriteBandSegment(5 , 7 , 1 , 1)     'Initialize all segments
         bytBand_Data(bytBand_cur) = 1
      End If
      If bytInit_LC = 0 Then
         bytInit_LC = 1
         bytTmp2 = 1
         WriteEeprom bytTmp2 , 2                  '=1: L/C data are stored
      End If
   Else
      Call ClearLCDLine(bytRowPwr , bytChar)
      Lcd "No EEPROM Address!"
      Wait 1
   End If

End Sub

'============================================================= WriteBandSegment
Sub WriteBandSegment(byVal bytFrom As Byte , byVal bytTo As Byte , byVal bytAll As Byte , byVal bytMsg As Byte)
'Write band segment data to EEPROM @ start address
'Input:  bytFrom     First byte to be stored (1...7)
'        bytTo       Last  byte to be stored (1...7)
'                    Band segment data overlayed with bytSeg_Arr(1...7):
'                    Byte 1: HiByte(Lo Segment frequency, kHz) -> wrdSeg_FreqLo
'                    Byte 2: LoByte(Lo Segment frequency, kHz) -> wrdSeg_FreqLo
'                    Byte 3: HiByte(Hi Segment frequency, kHz) -> wrdSeg_FreqHi
'                    Byte 4: LoByte(Hi Segment frequency, kHz) -> wrdSeg_FreqHi
'                    Byte 5: Encoder value L                   -> bytSeg_L
'                    Byte 6: Encoder value C                   -> bytSeg_C
'                    Byte 7: High pass(1), low pass(0)         -> bytSeg_HL
'        bytAll      = 0: only one band segment @ address wrdAddr
'                    = 1: all band segments of the amateur band
'                         only 'null'-values (L=C=0, lowpass) are overwritten
'        bytMsg      = 0: no display message
'                    = 1: address display message
'        wrdAddr     bytAll = 0: EEPROM band segment start address
'                    if only one band segment is to be written
'        wrdTRXfreq  bytAll = 1: TRX frequency
'                    EEPROM start address got from wrdBand_Addr(i)
'                    if all band segments for freq. are to be written

   If bytMsg = 1 Then
      Call ClearLCDLine(bytRowPwr , bytChar)
   End If

   'Write just one band segment -----------------------------------------------
   If bytAll = 0 Then
      If bytMsg = 1 Then
         strTmp10 = Str(wrdAddr)
         strTmp10 = Format(strTmp10 , "0000")
         Lcd "Addr. " ; strTmp10 ; " -> "         'EEPROM segment start address
      End If
      wrdTmp4 = bytFrom
      wrdTmp4 = wrdAddr + wrdTmp4
      wrdTmp4 = wrdTmp4 - 1                       'Address for byte bytFrom
      For bytTmp0 = bytFrom To bytTo
         WriteEEPROM bytSeg_Arr(bytTmp0) , wrdTmp4
         If bytMsg = 1 Then
            strTmp10 = Str(wrdTmp4)
            strTmp10 = Format(strTmp10 , "0000")
            Locate bytRowPwr , 17
            Lcd strTmp10                          'EEPROM address
            Waitms 300
         End If
         Incr wrdTmp4
      Next bytTmp0

   'Write all band segments of the current amateur band -----------------------
   Else
      'Get the Amateur band 1...bytBand_Num first
      bytTmp3 = 0
      For bytTmp1 = 1 To bytBand_Num              'Loop the bands
         If wrdTRXfreq >= wrdBand_Lo(bytTmp1) And wrdTRXfreq < wrdBand_Hi(bytTmp1) Then
            wrdAddr = wrdBand_Addr(bytTmp1)       'EEPROM start address of the band
            bytTmp3 = bytTmp1
            Exit For                              'Got the band index
         End If
      Next bytTmp1
      If bytTmp3 > 0 Then                         'band found
         If bytMsg = 1 Then
            strTmp10 = Str(wrdAddr)
            strTmp10 = Format(strTmp10 , "0000")
            Lcd "Addr. " ; strTmp10 ; " -> "      'EEPROM start address
         End If
         'Now loop the band segments in the band
         wrdTmp1 = wrdBand_Lo(bytTmp3)            'Lower segment limit
         wrdAddr = wrdBand_Addr(bytTmp3)          'Segment start address of band
         wrdSeg_width = bytBand_Width(bytTmp3)    'Segment width
         Do
            wrdTmp4 = bytFrom
            wrdTmp4 = wrdAddr + wrdTmp4
            wrdTmp4 = wrdTmp4 - 1                 'Address for byte bytFrom
            For bytTmp1 = bytFrom To bytTo        'Write band segment
               WriteEEPROM bytSeg_Arr(bytTmp1) , wrdTmp4
               If bytMsg = 1 Then
                  strTmp10 = Str(wrdTmp4)
                  strTmp10 = Format(strTmp10 , "0000")
                  Locate bytRowPwr , 17
                  Lcd strTmp10                    'EEPROM address
                  Waitms 300
               End If
               Incr wrdTmp4
            Next bytTmp1
            wrdAddr = wrdAddr + wrdSeg_Bytes      'next Band segment, next EEPROM block
            wrdTmp1 = wrdTmp1 + wrdSeg_width      'upper Segment limit                  '
         Loop Until wrdTmp1 >= wrdBand_Hi(bytTmp3)
         If bytMsg = 1 Then
            Waitms 300
            Call ClearLCDLine(bytRowPwr , bytChar)
         End If
      End If
   End If

   If bytMsg = 1 Then
      Call ClearLCDLine(bytRowPwr , bytChar)
   End If
End Sub

'============================================================== ReadBandSegment
Sub ReadBandSegment(byVal wrdStartAddr As Word , byVal bytFrom As Byte , byVal bytTo As Byte)
'Read band segment data from EEPROM @ start address wrdStartAddr
'from Byte bytFrom to Byte bytTo (1 ... 7)
'Band segments:
'Byte 1: HiByte(Lo Segment frequency, kHz) -> wrdSeg_FreqLo
'Byte 2: LoByte(Lo Segment frequency, kHz) -> wrdSeg_FreqLo
'Byte 3: HiByte(Hi Segment frequency, kHz) -> wrdSeg_FreqHi
'Byte 4: LoByte(Hi Segment frequency, kHz) -> wrdSeg_FreqHi
'Byte 5: Encoder value L                   -> bytSeg_L
'Byte 6: Encoder value C                   -> bytSeg_C
'Byte 7: High pass(1), low pass(0)         -> bytSeg_HL
'Variables and bytes bytSeg_Arr(1...7) are overlayed

   For bytTmp0 = bytFrom To bytTo
      ReadEEPROM bytSeg_Arr(bytTmp0) , wrdStartAddr
      Incr wrdStartAddr
   Next bytTmp0

End Sub

'================================================================ SegCenterFreq
Function SegCenterFreq() As Word
'Calculate band segment center frequency
'Input:  wrdSeg_freqLo  Band low  freq limit
'        wrdSeg_freqHi  Band high freq limit

   dwdTmp0 = wrdSeg_freqLo + wrdSeg_freqHi
   Shift dwdTmp0 , Right , 1                      'Divide by 2 (2^1)=center freq
   SegCenterFreq = dwdTmp0

End Function

'=============================================================== EEPROMWriteWrd
Sub EEPROMWriteWrd(byRef wrdWord As Word)
'Decode wrdWord to 2 bytes of bytWrdEEP for writing to EEPROM
'Input:  wrdWord to be decoded & written to EEPROM
'        wrdAddr first EEPROM address

   wrdEEPROM = wrdWord                            'wrdEEPROM overlayed @ bytWrdEEP
   For bytTmp0 = 1 To 2
      WriteEEPROM bytWrdEEP(bytTmp0) , wrdAddr
      Incr wrdAddr
   Next bytTmp0
End Sub

'================================================================ EEPROMReadWrd
Sub EEPROMReadWrd(byRef wrdWord As Word)
'Encode 2 bytes read from EEPROM to wrdWord
'Input:  wrdAddr first EEPROM address
'Output: wrdWord encoded unsigned integer (word) variable

   For bytTmp0 = 1 To 2
      ReadEEPROM bytWrdEEP(bytTmp0) , wrdAddr
      Incr wrdAddr
   Next bytTmp0
   wrdWord = wrdEEPROM                            'wrdEEPROM overlayed @ bytWrdEEP
End Sub

'=============================================================== EEPROMWriteInt
Sub EEPROMWriteInt(byRef intVal As Integer)
'Decode intVal to 2 bytes of bytIntEEP for writing to EEPROM
'Input:  intVal to be decoded & written to EEPROM
'        wrdAddr first EEPROM address

   intEEPROM = intVal                             'intEEPROM overlayed @ bytIntEEP
   For bytTmp0 = 1 To 2
      WriteEEPROM bytIntEEP(bytTmp0) , wrdAddr
      Incr wrdAddr
   Next bytTmp0
End Sub

'================================================================ EEPROMReadInt
Sub EEPROMReadInt(byRef intVal As Integer)
'Encode 2 bytes read from EEPROM to intVal
'Input:  wrdAddr first EEPROM address
'Output: intVal encoded signed integer variable

   For bytTmp0 = 1 To 2
      ReadEEPROM bytIntEEP(bytTmp0) , wrdAddr
      Incr wrdAddr
   Next bytTmp0
   intVal = intEEPROM                             'intEEPROM overlayed @ bytIntEEP
End Sub

'==================================================================== EEPROMAdr
Function EEPROMAdr(byval bytEncoder As Byte) As Word
'Calculate EEPROM Address for band segment from encoder value bytEncoder
'Input:  bytEncoder    Encoder value
'        wrdSeg_Bytes  Numer of bytes per band segment

   wrdTmp1 = bytEncoder - 1
   EEPROMAdr = wrdSeg_Bytes * wrdTmp1
   EEPROMAdr = EEPROMAdr + wrdSeg_FirstAddr

End Function

'================================================================= AutoTuneAddr
Function AutoTuneAdr(byVal bytOpt As Byte , byVal bytBand As Byte) As Word
'Calculate EEPROM address of band auto tune data
'Input:  bytBand           Index of band (1...11)
'        bytBand_Num       Number of bands (11)
'        wrdTune_FirstAddr First EEPROM address
'        bytOpt            =0: maximal step size C
'                          =1: maximal step size L
'                          =2: Start  lo/hi pass
'                          =3: Start auto tune with C value
'                          =4: Start auto tune with L value
'                          =5: Scan direction L/C
'Output: AutoTuneAdr       EEPROM address

   bytTmp0 = bytBand - 1
   bytTmp1 = bytOpt * bytBand_Num
   wrdTmp1 = bytTmp1 + bytTmp0
   AutoTuneAdr = wrdTune_FirstAddr + wrdTmp1

End Function

'******************************************************************************
'Communication routines
'******************************************************************************

'==================================================================== IsComBusy
Function IsComBusy() As Byte
'Test if RS485 communication is busy
'If yes wait until its free
'Input:  bitComBusy  =0: is not busy, =1; is busy

   IsComBusy = 1
   Do
      NOP
   Loop Until bitComBusy = 0
   IsComBusy = 0

End Function

'=================================================================== SendRemote
Sub SendRemote(byVal bytRow As Byte)
'Send data to the Remote Unit
'While switching the relay turn TX Mute on
'Input:  bytRow        LCD row to display error message
'        bytATUAdr     destination address (ATU)
'        bytConAdr     Sender address (ATU controller)
'        bytATUCall    Request ATU switching command
'        bytEnc_L      Encoder value L
'        bytEnc_C      Encoder value C
'        bitHL_pass    High pass(1), Low pass(0)
'Output: bytComErr     = 0: ACK, Message acknoledged
'                      = 1: Time out, Input Buffer bytBuffInp not complete
'                      = 2: Message destination is not me, the controller
'                      = 3: Message source is not the remote unit
'                      = 4: Checksum is not OK
'                      = 5: NACK, Message not acknoledged
'                      = 6: Garbage in response

   'Send tune request to ATU remote unit --------------------------------------
   'RS485 protocoll to ATU remote unit:
   '1     Address of message destination (bytATUAdr)
   '2     Address of message source  (bytConAdr)
   '3     Command (bytATUCall)
   '4     Value L (0 ... 255)
   '5     Value C (0 ... 255)
   '6     Value H/T (1 = High pass, 0 = Low pass)
   '7     Checksum of bytes 1 to 6 (CRC8, 1 byte)

   bitComBusy = 1                                 'ATU communication busy
   bitRS485Send = On                              'set MAX485 to transmit
   Waitms 1                                       'Wait for MAX485 to switch
   'During communication switch LED's off, after that on again
   Select Case bytMode
      Case 1
         Toggle Led_auto                          'Auto LED off
      Case 2
         Toggle Led_man                           'Man LED off
   End Select

   'Send ATU switching request, configure protocoll buffer
   UCSR0A.6 = 1                                   'Clear UART Transmit Complete bit
   bytBuffOut(1) = bytATUAdr                      'destination address (ATU)
   bytBuffOut(2) = bytConAdr                      'Sender address (ATU controller)
   bytBuffOut(3) = bytATUCall                     'Request ATU switching
   If bytLNonInv = 1 Then                         'Encoder L value not to be inverted
      bytBuffOut(4) = bytEnc_L                    'Encoder value L
   Else                                           'Invert encoder value L
      bytBuffOut(4) = Not bytEnc_L                'Inverted encoder value L
   End If
   bytBuffOut(5) = bytEnc_C                       'Encoder value C
   bytBuffOut(6) = bitHL_pass                     'High pass(1), Low pass(0)
   bytCRC8 = Crc8(bytBuffOut(1) , bytFrameLenMinus1)
   bytBuffOut(7) = bytCRC8                        'CRC8

   'Call ClearLCDLine(3,20)
   'LCD bytBuffOut(1) ; " " ; bytBuffOut(2) ; " " ; bytBuffOut(3)

   Printbin bytBuffOut(1) ; bytFrameLen           'Send array bytBuffOut
   Do
   Loop Until UCSR0A.6 = 1                        'Wait until all bytes are sent
   bitRS485Send = Off                             'Set MAX485 to receive
   Waitms 1                                       'Wait for MAX485 to switch

   'Wait for response from ATU remote unit ------------------------------------
   'Protocoll from ATU remote unit:
   '1     Address of message destination (bytConAdr)
   '2     Address of message source  (bytATUAdr)
   '3     Command (bytATUCall)
   '4     ACK  (6), Acknoledge , CRC OK, message confirmed or
   '      NAK (21), Non-ACK,     CRC not OK, message not confirmed
   '5     --- (bytDummy)
   '6     --- (bytDummy)
   '7     Checksum of bytes 1 to 6 (CRC8, 1 byte)

   bytTimeout_cnt = 0                             'Start Receive time out (Timer2)
   bytBuffCount = 0                               'Initialize input buffer counter
   bytComErr = 0                                  'Initialize Com error flag

   Do
      If bytBuffCount = bytFrameLen Then          'input buffer full
         Disable URXC                             'Disable Character receive interrupt
         Waitms 15
         bytBuffCount = 0                         'Reset input buffer for next msg
         'Call ClearLCDLine(4 , 20)
         'LCD bytBuffInp(1) ; " " ; bytBuffInp(2) ; " " ; bytBuffInp(3) ; " " ; bytBuffInp(4)
         If bytBuffInp(1) = bytConAdr Then        'It's for me, the ATU controller
            bytCRC8 = Crc8(bytBuffInp(1) , bytFrameLenMinus1)
            If bytCRC8 = bytBuffInp(7) Then       'Checksum is OK
               If bytBuffInp(2) = bytATUAdr Then  'It's from the remote unit
                  Select Case bytBuffInp(4)       'Ack or NACK
                     Case 6                       'Ack
                        bytComErr = 0             'Msg acknoledged, OK
                     Case 21                      'NACK
                        bytComErr = 5
                     Case Else                    'Garbage
                        bytComErr = 6
                  End Select
               Else
                  bytComErr = 3                   'Msg is not from the remote unit
               End If
            Else
               bytComErr = 4                      'Checksum not OK
            End If
         Else
            bytComErr = 2                         'Msg is not for the ATU controller
         End If
         Exit Do
      Else
         If bytTimeout_cnt > bytTimeout_runs Then
            bytComErr = 1                         'Time out, response not complete
            Exit Do
         End If
      End If
   Loop

   bytTimeout_cnt = 0                             'Reset timeout counter

   Select Case bytComErr
      Case 0                                      'OK, got it, LED on again
         Select Case bytMode
            Case 1
               Toggle Led_auto                    'Auto LED on
            Case 2
               Toggle Led_man                     'Man LED on
         End Select
      Case 1                                      'Timeout
         Call ClearLCDLine(bytRow , bytChar)
         Lcd "ATU timeout "
         Waitms 500
         Call ClearLCDLine(bytRow , bytChar)
      Case 2 To 6
         Call ClearLCDLine(bytRow , bytChar)
         Lcd "ATU Com Err " ; bytComErr
         Waitms 500
         Call ClearLCDLine(bytRow , bytChar)
   End Select

   If bytComErr > 0 Then                          'Set old values if error
      bytEnc_L = bytEncL_old
      bytEnc_C = bytEncC_old
      bitHL_pass = bitHL_old
   End If
   Enable URXC                                    'Enable Character receive interrupt
   bitComBusy = 0                                 'ATU communication not busy

End Sub

'=================================================================== GetTRXFreq
Sub GetTRXFreq(byVal bytRow As Byte)
'Call frequency (kHz) from TRX
'Input:  bytRow         LCD row to display error message
'        bytTRXAdr      destination address (TRX)
'        bytConAdr      Sender address (ATU controller)
'        bytFreqCall    Request TRX frequency command
'Output: wrdTRXfreqCom  Frequency (kHz) received from TRX, overlayed with bytTRXfreqCom
'        bytComErr      = 0: OK, frequency received
'                       = 1: Time out, Input Buffer bytBuffInp not complete
'                       = 2: Message destination is not me, the controller
'                       = 3: Message source is not the TRX
'                       = 4: Checksum is not OK
'                       = 5: No frequency received
'                       = 6: Command not OK

   'Send frequency request to TRX ---------------------------------------------
   'RS485 protocoll to TRX for frequency request:
   '1     Address of message destination (bytTRXAdr)
   '2     Address of message source  (bytConAdr)
   '3     Command (bytFreqCall)
   '4     Data byte 1 (bytDummy)
   '5     Data byte 2 (bytDummy)
   '6     Data byte 3 (bytDummy)
   '7     Checksum of bytes 1 to 6 (CRC8, 1 byte)

   bitComBusy = 1                                 'TRX communication busy
   bitRS485Send = On                              'set MAX485 to transmit
   Waitms 1                                       'Wait for MAX485 to switch
   UCSR0A.6 = 1                                   'Clear UART Transmit Complete bit
   bytBuffOut(1) = bytTRXAdr                      'destination address (TRX)
   bytBuffOut(2) = bytConAdr                      'Sender address (ATU controller)
   bytBuffOut(3) = bytFreqCall                    'Request TRX frequency
   bytBuffOut(4) = bytDummy                       'not used, dummy
   bytBuffOut(5) = bytDummy                       'not used, dummy
   bytBuffOut(6) = bytDummy                       'not used, dummy
   bytCRC8 = Crc8(bytBuffOut(1) , bytFrameLenMinus1)
   bytBuffOut(7) = bytCRC8                        'CRC8
   Printbin bytBuffOut(1) ; bytFrameLen           'Send array bytBuffOut
   Do
   Loop Until UCSR0A.6 = 1                        'Wait until all bytes are sent
   bitRS485Send = Off                             'Set MAX485 to receive
   Waitms 1                                       'Wait for MAX485 to switch

   'Wait for response from TRX ------------------------------------------------
   'RS485 protocoll from TRX:
   '1     Address of message destination (bytConAdr)
   '2     Address of message source  (bytTRXAdr)
   '3     Command (bytFreqCall)
   '4     Error code from TRX, =0: frequency is sent
   '5     bytTRXfreqCom(1)
   '6     bytTRXfreqCom(2), bytTRXfreqCom(1,2) overlayed with wrdTRXfreqCom
   '7     Checksum of bytes 1 to 6 (CRC8, 1 byte)

   bytTimeout_cnt = 0                             'Start Receive time out (Timer2)
   bytBuffCount = 0                               'Initialize input buffer counter
   bytComErr = 0                                  'Initialize Com error flag

   Do
      If bytBuffCount = bytFrameLen Then          'Response is complete
      Disable URXC                                'Disable Character receive interrupt
      Waitms 15
      bytBuffCount = 0                            'Reset input buffer for next msg
      'Locate 3 , 1
      'LCD bytBuffInp(1) ; " " ; bytBuffInp(2) ; " " ; bytBuffInp(3) ; " " ; bytBuffInp(4) ; " " ; wrdTRXfreqCom ; " " ; bytBuffCount
         If bytBuffInp(1) = bytConAdr Then        'It's me, the ATU controller
            bytCRC8 = Crc8(bytBuffInp(1) , bytFrameLenMinus1)
            If bytCRC8 = bytBuffInp(7) Then       'Checksum is OK
               If bytBuffInp(2) = bytTRXAdr Then  'It's from the TRX
                  If bytBuffInp(3) = bytFreqCall Then       'It's frequency request
                     If bytBuffInp(4) = 0 Then    'Error code 0: OK, freq is sent
                        bytTRXfreqCom(1) = bytBuffInp(5)
                        bytTRXfreqCom(2) = bytBuffInp(6)
                        bytComErr = 0             'Frequency is received
                     Else
                        wrdTRXfreqCom = 0
                        bytComErr = 5             'No freq sent
                     End If
                  Else
                     bytComErr = 6                'command not OK
                  End If
               Else
                  bytComErr = 3                   'Source address not OK
               End If
            Else
               bytComErr = 4                      'Checksum not OK
            End If
         Else
            bytComErr = 2                         'destination address not OK
         End If
         Exit Do
      Else                                        'Response not complete or missing
         If bytTimeout_cnt > bytTimeout_runs Then
            bytComErr = 1                         'Time out, response not complete
            Exit Do
         End If
      End If
   Loop

   bytTimeout_cnt = 0                             'Reset timeout counter

   Select Case bytComErr
      Case 0                                      'OK, got the frequency

      Case 1                                      'Timeout
         Call ClearLCDLine(bytRow , bytChar)
         Lcd "TRX timeout "
         Waitms 500
         Call ClearLCDLine(bytRow , bytChar)
      Case 2 To 6
         Call ClearLCDLine(bytRow , bytChar)
         Lcd "TRX Com Err " ; bytComErr
         Waitms 500
         Call ClearLCDLine(bytRow , bytChar)
   End Select
   Enable URXC                                    'Enable Character receive interrupt
   bitComBusy = 0                                 'TRX communication not busy

End Sub

'==============================================================================

End